#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//                Programmer: James A. Green
//                Update: May 31, 2002.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//  starz5ag.cpp - Stellar Age, Evolution, Perturbations,
//                 Core Collapse, Scenarios, Precision depth scanning.
//
//
//  PROJECT STARRANGER -
//	   starranger.prj is the project file
//	       producing starranger.exe from
//		     starz5.cpp - main module.
//		      starz5ag.cpp - Stellar ages and evolution, perturbations.
//		      starz5au.cpp - Neutrino luminosity & stellar wind.
//		      starz5fu.cpp - basic functions.
//		      starz5bo.cpp - bolometric corrections.
//		      starz5op.cpp - supplemental ops.
//		      starz5sc.cpp - stellar ranging.
//
//
//               Greenwood Research, http://grnwd.tripod.com.
//
//  ROUTINES in starz4ag.cpp:
//
//    evolver() - Reports to file fp, Star Age,
//                Evolutionary Perturbations,
//                Core Collapse, White Dwarf formation,
//                Planetary Nebula ejection, valid for
//                lower main sequence < 4 solar masses,
//                as soon as our CNO H-burning code is perfect.
//                Presently valid for stars < 2 solar masses,
//                as the carbon cycle is tiny then, and p-p
//                fusion dominates.
//
//    dwarfradius(dwarfmass) - Calculates the radius of a
//                           White Dwarf from a table prepared
//                           by S.Chandrasehar using linear interpolation.
//
//  STAR AGE ESTIMATION & PERTURBATION ANALYSIS
//  - Model perturbed by mu(r) variance
//    resulting from the consumption of the core.
//    Temperature varies as mu(r).
//  
//  Compute the age of the star from the radius of the hydrogen-burning core,
//  the average core density, the observed lumininosity of the star, and
//  delh, the change in the hydrogen abundance required to normalize Clayton p-p fusion.
//  delh = 0.5976 - claynormx is the change in the hydrogen abundance reqd
//  to normalize the Clayton p-p fusion for Clayton-Aller 1961 abundances.   <A> - Age Section.
//
//  starage = delh*(ro_core/(4.0*m_H))*(4*pi/3.0)((rhcore)**3)*(R_sun**3)*delE/L(R) sec.
//
//        L(R) is the stellar luminosity in watts. 1 year = 3.156e7 sec.
//
//   EXTERNAL VARIABLES
//
extern FILE *fp;                         // file pointer fp to starman.txt
extern int status;                       // file status
extern double cuberoot(double v);        // cuberoot(v)
extern double nthpower(double v, int n); // v**n
extern double pi;		   //  pi, 3.14159.
extern double gnewton;		   // Newtons constant of gravitation
extern double msun;                // mass of the Sun in kg.
extern double xmstar[1];           // mass of the star in kg.
extern double zage;                // Zombeck stellar lifetime.
extern double ageschw;             // Schwarzschild star age.
extern double age;                 // Greens Main Sequence Lifetime.
extern double hkage;               // Helmholtz contraction time.
extern double claynormx;           // X(H) to normalize L_nuclear_Clayton to L(R) observed.
extern double chemx[1];            // X(H) - initial hydrogen abundance by weight.
extern double chemy[1];            // Y(He) - initial helium abundance by weight.
extern double chemz[1];            // Z(hvy) - initial heavy element abundance by weight.
extern double chemmu[1];                 // initial molecular weight.
extern double xtc[1];                    // initial central temperature.
extern double rocoremin;                 // minimum density at far edge of H-burning core.
extern double xdensitycentral[1];        // max density at center of H-burning core.
extern double mh;                        // mass of the hydrogen atom in kg.
extern double rhcore;			 // radius of the core in solar radii.
extern double rsun;                      // radius of the Sun in meters.
extern double totalluminosity;          // total nuclear luminosity P-P, CNO, He-burning
extern double totalhluminosity;         // total P-P plus CNO H-burning.
extern double bclratio[1];              // L_star/L_sun
extern double lsun;                     // Luminosity of the Sun in Watts
extern double xrstar[1];                // Radius of the Sun in meters.
extern double photontime;               // The stellar photon random-walk diffusion time.
extern double lightyears[1];            // Original Distance in lightyears for depthscanner.
extern void probe(int i);               // Probe identifies self as ith, waits for keystroke.
//
//
//    evolver() - Reports to file fp, Star Age,
//                Evolutionary Perturbations,
//                Core Collapse, White Dwarf formation,
//                Planetary Nebula ejection, valid for
//                lower main sequence < 4 solar masses,
//                as soon as our CNO H-burning code is perfect.
//                Presently valid for stars < 2 solar masses,
//                as the carbon cycle is tiny then, and p-p
//                fusion dominates.
//
void evolver(void)
{                           // local variables
	double dwarfradius(double dwarfmass); // Compute collapsed core radius from core mass.
	double delh;             //  the change in X(H) required to normalize
//                                   so that L_nuclear equals L(R) observed.
	double dele;             // potenwhite/2.0 - launchenergy is kinetic energy.
	double kboltz = 1.3805e-23;  // Boltzmann's constant in J/K.
	double c = 2.9979e8;     // the speed of light in meters per second.
	double freefalltime;    // The core free fall collapse time, sec.
	double menvel;		// mass of the envelope around the H-burning core.
	double corephotonmfp;   // core photon mean free path
	double coresurfacetemp; // core surface temperature
	double coreti;         //  core mean internal temperature
	double corephotontime;  // core photon diffusion time
	double vmax;            // ideal, lossless velocity of ejection envelope at infinity.
	double rocoreavg;        //  the average density of the H-burning core.
	double masscorekg;         //  the mass of the H-burning core, kg.
	double masscoresolar;     //  mass of the H-burning core in solar units.
	double massenvelratio; //  mass of the star ejected by w.dwarf formation in Star units.
	double masscoreratio;    //   mass H-burning core/star_mass.
	double corepoten;        // Potential energy of the core, Integral (GM(r)/r)dM,
//                        using  the linear approx ro(r) = ro(0) - ((ro(0) - ro(1))/r_core)*r,
//        is 4*pi*G*(ro(0)**2)/2  + 4*pi*G*((ro(1) - ro(0))/r_core)*((r_core)**3)/3.
	double potenwhite;       // approx. white dwarf remnant potential energy
//          (6.0/7.0)*G*M*M/(0.01*R_sun), n=1.5 polytrope
	double deltenvel;  // Max temperature rise in stellar envelope after core collapse
	double nparticles;  // Number of particles in the stellar envelope.
	double launchenergy;     // Energy to put envelope around core back to infinity.
	double lenratio;         // Launch energy ratio:
//                                  Available core collapse energy/Envelope_to_infinity energy.
// 	                            Only half of the white dwarf potential energy
//                                  is available for launching the envelope.
	double ageratio;         // Stellar lifetime measure over H-burning core age.
	double roc0, roc1;       //  H-burning core central density roc0 to final ro roc1.
	double rcore;            //  radius of the core in meters, rhcore*rsun;
	double starage;          //  the age of the star in years.
	double secperyear = 3.156e7;   // seconds per year.
	double corecollapsetime; //  Helmholtz-Kelvin collapse time of the core to W.D.state.
	double chemmucore;       //  molecular weight of the core.
	double mufinal;          //  final molecular weight in H-burning core.
	double drr;              //  delta R over R_star, for entire star from core delta_r.
	double dpp;              //  delta_p over p in the H-burning core
	double dtt;              //  delta_t over t for the H-burning core temperature.
	double coreratio;         // r_core/R_star for the H-burning core.
	double muratio;           // delta_mu/mu_original for the H-burning core.
	double collapsetime;      // time from now until H-burning core collapse in years.
	double hburnlifetime;     // Total Main Sequence lifetime from core collapse model, yrs.
	double deltarratio;       // MAX(deltarratio/R_star_original) for star growth
//                                   from fusion function. (mu_avg/mu_original)**4
//                                   raises the temperature. Then the star expands
//                      until 1 = ((R_star_original/R)**6)*(mu_avg/mu_original)**4
//                      brings the p-p fusion function back to its old value if the
//		        star is oscillating, and to some R < R_max such that
//                      1 < ((R_star_original/R)**6)*(mu_avg/mu_original)**4
//                      to merely grow. See the supplemental notes to
//			Thermonuclear Fusion in Stars for the theoretical underpinning.
//
//
//  Clayton age & perturbation analysis of star.
//
	    delh = chemx[0] - claynormx;
	    rocoreavg = (xdensitycentral[0] + rocoremin)/2.0;
	    starage  = (delh*rocoreavg/(4.0*mh))*(4.0/3.0)*3.14159*rhcore*rhcore*rhcore;
	    starage = starage*rsun*rsun*rsun*(4.1973e-12)/(lsun*bclratio[0]);  // 4.1973e-13 J is 26.2 MeV.
	    starage = starage/3.156e7;  // Find years age from seconds.
	    fprintf(fp,"\nClayton Star age = %e yr based on the H-burning core\n",starage);
	    fprintf(fp,"  depletion required to normalize L(R) for Clayton p-p fusion.\n");

	    if(totalhluminosity < bclratio[0])
	       fprintf(fp,"  Total nuclear luminosity < L(R), Clayton Star Age invalid.\n");

	    fprintf(fp,"  10**7 K H-burning core radius = %1.2f solar radii", rhcore);
	    fprintf(fp," = %1.2f star radius.\n", rhcore*rsun/xrstar[0]);
	    fprintf(fp,"  Mantle: X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0],chemy[0],chemz[0],chemmu[0]);
	    chemmucore = 1.0/(2.0*(chemx[0] - delh) + (0.75*(chemy[0] + delh)) + 0.5*chemz[0]);
	    fprintf(fp,"  Core:   X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0] - delh, chemy[0] + delh, chemz[0], chemmucore);

	    masscorekg = rocoreavg*(4.0*3.14159/3.0)*(rhcore*rsun)*(rhcore*rsun)*(rhcore*rsun);
	    masscoresolar = masscorekg/msun;   // mass of the H-burning core, solar units.
	    masscoreratio = masscorekg/xmstar[0];   // M_core/M_star .
	    fprintf(fp,"  H-burningCoreMass = %e kg , = %f Msun , Mcore/Mstar = %f.\n",
	      masscorekg, masscoresolar, masscoreratio);

	    fprintf(fp,"  Core gravitational potential energy = %e Joules.\n", corepoten);

	    coreratio = rhcore*rsun/xrstar[0];   // cal rcore/R_star
	    muratio = 1.0 + ( ((chemmucore - chemmu[0])/chemmu[0])*nthpower(coreratio,3) );
	    fprintf(fp,"  1)Perturbation analysis based on nuclear energy gen & volume-avg mu.\n");
	    fprintf(fp,"    Old T_c = %e K, Perturbed (mu_avg/mu_old)*T_c = %e K\n",
	       xtc[0], xtc[0]*muratio);
	    deltarratio = cuberoot( muratio*muratio ) - 1.0;
	    fprintf(fp,"    Radial expansion perturbation delta_R/R_star < %f.\n", deltarratio);

	    fprintf(fp,"  2)Polytropic analysis based on gas plus radiation laws. Since formation:\n");
	    drr = -(1.0/3.0)*((chemmucore - chemmu[0])/chemmucore)*rhcore;
	    fprintf(fp,"      delta_R/R = -(1/3)(del_mu/mu)(R_core/R)= %f.\n", drr);
	    dtt = -((chemmucore - chemmu[0])/chemmucore);
	    fprintf(fp,"      delta_T/T = -(del_mu/mu) = %f.\n", dtt);
	    dpp = -(4.0/3.0)*((chemmucore - chemmu[0])/chemmucore);
	    fprintf(fp,"      delta_P/P = -(4/3)(del_mu/mu) = %f.\n", dpp);

	    mufinal = 1.0/( (0.75*(chemx[0] + chemy[0])) + 0.5*chemz[0] );
	    fprintf(fp,"    When the core has exhausted its hydrogen, mu = %f, and\n", mufinal);
	    drr = -(1.0/3.0)*((mufinal - chemmu[0])/mufinal)*rhcore;
	    fprintf(fp,"      delta_R/R = -(1/3)(del_mu/mu)(R_core/R)= %f.\n", drr);
	    dtt = -((mufinal - chemmu[0])/mufinal);
	    fprintf(fp,"      delta_T/T = -(del_mu/mu) = %f.\n", dtt);
	    dpp = -(4.0/3.0)*((mufinal - chemmu[0])/mufinal);
	    fprintf(fp,"      delta_P/P = -(4/3)(del_mu/mu) = %f.\n", dpp);
//
//  Collapse of the H-burning core, Planetary Nebula ejection, White Dwarf formation.


	    fprintf(fp,"\nTHE HYDROGEN-BURNING CORE - COLLAPSE & ENVELOPE EJECTION PROBLEM\n\n", dpp);
	    fprintf(fp,"The smallest white dwarf in the Chandrasekhar Table has a mass\n");
	    fprintf(fp,"of 0.22 solar masses. Sun core mass is 0.216073 solar masses.\n");
	    fprintf(fp,"Bowers & Deeming: All stars < 4 solar masses form a white dwarf.\n");

	    delh = claynormx;  // Countdown to core collapse.
	    rocoreavg = (xdensitycentral[0] + rocoremin)/2.0;
	    collapsetime  = (delh*rocoreavg/(4.0*mh))*(4.0/3.0)*3.14159*rhcore*rhcore*rhcore;
	    collapsetime = collapsetime*rsun*rsun*rsun*(4.1973e-12)/(lsun*bclratio[0]);  // 4.1973e-13 J is 26.2 MeV.
	    collapsetime = collapsetime/3.156e7;  // Find years age from seconds.
	    fprintf(fp,"Hydrogen-burning core mass = %f solar masses.\n",masscoresolar);
	    fprintf(fp,"  Core gravitational potential energy = %e Joules.\n", corepoten);

	    potenwhite = (6.0/7.0)*gnewton*masscorekg*masscorekg/(dwarfradius(masscorekg/msun)*rsun);

	    fprintf(fp,"  White dwarf approx. remnant potential energy = %e Joules.\n",potenwhite);
	    launchenergy = (6.0/7.0)*gnewton*xmstar[0]*xmstar[0]/(xrstar[0]);
	    launchenergy = launchenergy - corepoten;
	    fprintf(fp,"  Stellar envelope launch energy for infinity = %e Joules.\n", launchenergy);
	    lenratio = (potenwhite/2.0 - corepoten)/launchenergy;
	    fprintf(fp,"  Available_collapse_energy/envelope_launch energy = %f \n", lenratio);
	    if(lenratio > 1.0){
	       fprintf(fp,"  The star should promptly expand when it forms a white dwarf.\n");
	       menvel = xmstar[0] - masscorekg;
	       dele = potenwhite/2.0 - launchenergy;
	       if(2.0*dele/menvel > 0.0){
		 vmax = sqrt(2.0*dele/menvel);
	       }
	       else{
		 vmax = - 0.1;
		 fprintf(fp,"  Maximum velocity of ejectae at infinity invalid!\n");
	       }
	       fprintf(fp,"  Maximum velocity of ejectae at infinity = %e m/sec,\n", vmax);
	       vmax = (vmax/1.4e11)*3.156e7;
	       fprintf(fp,"                      = %f Astronomical_Units/year,\n", vmax);
	       vmax = 364.25/vmax;  // Compute days to reach Earth.
	       fprintf(fp,"                      = %f days to reach 1 AU,\n", vmax);
	       fprintf(fp,"  assuming no kinetic energy is wasted in heating the envelope.\n");
	       massenvelratio = (xmstar[0] - masscorekg)/xmstar[0];
	       fprintf(fp,"  %f percent of the stellar mass is ejected.\n", massenvelratio*100.0);
	    }
	    else{
	       fprintf(fp,"  Too little energy is present to expand the envelope to infinity.\n");
	    }
	    nparticles = (xmstar[0] - masscorekg)/(chemmu[0]*mh); // Cal number of particles in envelope.

	    if((2.0*(potenwhite/2.0) - corepoten)/(nparticles*kboltz*3.0) > 0.0 ){
	       deltenvel = sqrt((2.0*(potenwhite/2.0) - corepoten)/(nparticles*kboltz*3.0));  // Cal max delta_T in envel.//
	    }
	    else{
	       fprintf(fp,"  Envelope heating calc invalid!\n");
	    }

	    fprintf(fp,"  Conversely, assume all released energy goes to heat the envelope.\n");
	    fprintf(fp,"  Then the max temperature rise in the envelope =%e K.\n", deltenvel);
	    corecollapsetime =  ((potenwhite/2.0) - corepoten)/(bclratio[0]*lsun);
	    corecollapsetime = corecollapsetime/secperyear;
	    fprintf(fp,"  Kelvin-Helmholtz model core collapse time =%e K,\n", corecollapsetime);
	    fprintf(fp,"    but the planetray nebula ejection lasts < 3,000 yr,\n");
	    fprintf(fp,"    and the nebula clears away in say, 25,000 yrs.\n");    

	    freefalltime = rhcore*rhcore*rhcore*rsun*rsun*rsun/((6.67e-11)*masscorekg);
	    freefalltime = 1.11072*sqrt(freefalltime); // The Physics of Stars, A.C.Phillips.
	    freefalltime = freefalltime/60.0;  // A.C.Phillips free-fall time, pg.7.
	    fprintf(fp,"  Core freefall time = %e min \n", freefalltime);
	    fprintf(fp,"     = %e hrs = %e days = %e years.\n",
	      freefalltime/60.0, freefalltime/1440.0, freefalltime/526176.0 );
	    fprintf(fp,"  Stellar photon random-walk escape time = %e sec \n", photontime); // Phillips, p.23.
	    fprintf(fp,"    = %e years.\n", photontime/3.156e7);

// calculate mean free path from core opacity next time.
//
//	    rocoreavg = (xdensitycentral[0] + rocoremin)/2.0;
//	    coresurfacetemp = xtc[0]*cuberoot(rocoremin/xdensitycentral[0]);
//	    coreti = xtc[0]*cuberoot(rocoreavg/xdensitycentral[0]);
//	    corephotonmfp =   coresurfacetemp/coreti;  // Compute photon mean free path.
//	    corephotonmfp = nthpower(corephotonmfp,4);
//	    corephotonmfp = corephotonmfp*rhcore*rsun;
//	    corephotontime = (rhcore*rsun*rhcore*rsun)/(c*corephotonmfp);
//	    fprintf(fp,"  Core photon mean free path = %e meters ", corephotonmfp); // Phillips, p.23.
//	    fprintf(fp,"= %e mm\n", corephotonmfp*1000.0);
//	    fprintf(fp,"  Core photon random-walk escape time = %e sec \n", corephotontime); // Phillips, p.23.
//	    fprintf(fp,"    = %e years.\n", corephotontime/3.156e7);
//
//   Canonical Stellar Lifetimes and the Lifetime of the H-burning core.
//
	    fprintf(fp,"Time from now to core collapse in H-burning core: = %e yr.\n", collapsetime);
	    hburnlifetime = collapsetime + starage;
	    fprintf(fp,"Total Main-Sequence Lifetime until core collapse: = %e yr,\n", hburnlifetime);
	    fprintf(fp,"   according to our simplistic H-burning core collapse theory.\n");
	    ageratio = hburnlifetime/zage;
	    fprintf(fp,"                   H-burning_lifetime/Zombeck_Age = %f \n", ageratio);
	    ageratio = hburnlifetime/ageschw;
	    fprintf(fp,"             H-burning_lifetime/Schwarzschild_Age = %f \n", ageratio);
	    ageratio = hburnlifetime/age;
	    fprintf(fp,"       H-burning_lifetime/Greens_Main_Seq.Lifeime = %f \n", ageratio);
	    ageratio = hburnlifetime/hkage;
	    fprintf(fp,"    H-burning_lifetime/Helmholtz_contraction_time = %f \n", ageratio);

}
// -----------------------------------------------------------------
//
// dwarfradius(dwarfmass) - Compute the radius of a white dwarf from
//                          its mass according to a table from Chandrasekhar.
//                          This is for mu_E = 2/(1 + X) = 2, when the core has
//                          been exhausted of hydrogen.
//
//         in: dwarfmass in solar masses.
//        out: dwarfradius in solar radii.
//
//    M/M_sun     R/R_sun   log(R_sun/R)
//     0.22       0.019952   1.7
//     0.40       0.015488   1.81
//     0.50	  0.013803   1.86         S. Chandrasekhar,Stellar Structure,
//     0.61	  0.012303   1.91         p.427, University of Chicago Press,
//     0.74	  0.010965   1.96         1938. Quoted from Martin Schwarzschild,
//     0.88       0.009333   2.03         Structure and Evolution of the Stars,
//     1.08	  0.007946   2.15         p.232, Dover Publications, 1965.
//     1.22       0.005495   2.26
//     1.33       0.003890   2.41
//     1.38       0.002951   2.53
//     1.44       0.00000    infinity
//
// ------------------------------------------------------------------
double dwarfradius(double dwarfmass)
{
double drad;

     if(dwarfmass < 0.22){   // analytically continue back from first table entry
	drad = 0.019952;
        return(drad);
     }
     if(dwarfmass >= 0.22 && dwarfmass < 0.4 ){
	drad = 0.019952 + (( 0.015488 - 0.019952)/(0.40 - 0.22))*(dwarfmass - 0.22);
        return(drad);
     }
     if(dwarfmass >= 0.40 && dwarfmass < 0.5){
	drad = 0.015488 + ((0.013803 - 0.015488)/(0.50 - 0.40))*(dwarfmass - 0.40);
        return(drad);
     }
     if(dwarfmass >= 0.50 && dwarfmass < 0.61){
	drad = 0.013803 + ((0.012303 - 0.013803)/(0.61 - 0.50))*(dwarfmass - 0.50);
        return(drad);
     }
     if(dwarfmass >= 0.61 && dwarfmass < 0.74){
	drad = 0.012303 + ((0.010965 - 0.012303)/(0.74 - 0.61))*(dwarfmass - 0.61);
        return(drad);
     }
     if(dwarfmass >= 0.74 && dwarfmass < 0.88){
	drad = 0.010965 + ((0.009333 - 0.010965)/(0.88 - 0.74))*(dwarfmass - 0.74);
        return(drad);
     }
     if(dwarfmass >= 0.88 && dwarfmass < 1.08){
	drad = 0.009333 + ((0.007946 - 0.009333)/(1.08 - 0.88))*(dwarfmass - 0.88);
        return(drad);
     }
     if(dwarfmass >= 1.08 && dwarfmass < 1.22){
	drad = 0.007946 + ((0.005495 - 0.007946)/(1.22 - 1.08))*(dwarfmass - 1.08);
        return(drad);
     }
     if(dwarfmass >= 1.22 && dwarfmass < 1.33){
	drad = 0.005495 + ((0.003890 - 0.005495)/(1.33 - 1.22))*(dwarfmass - 1.22);
        return(drad);
     }
     if(dwarfmass >= 1.33 && dwarfmass < 1.38){
	drad = 0.003890 + ((0.002951 - 0.003890)/(1.38 - 1.33))*(dwarfmass - 1.33);
        return(drad);
     }
     if(dwarfmass >= 1.38 && dwarfmass < 1.44){
	drad = 0.002951 + ((0.0000000001 - 0.002951)/(1.44 - 1.38))*(dwarfmass - 1.38);
        return(drad);
     }
     if(dwarfmass >= 1.44){   // analytically continue forwards from last table entry
	drad = 0.0000000001; // avoid computational crisis.
	return(drad); 
     }
}
