#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
/******************************************************************
*                                     June 18, 2002|8:40 AM
*                                                     
* STARCAL  Author: James A. Green,
*          Email: james_a_green_iii@yahoo.com
*          Corrections: Gratefully acknowledged.
*          Online: http://grnwd.tripod.com/home.html .
*          Greenwood Research Catalog: http://grnwd.tripod.com .
*
*    - PROJECT DEPTHSTA
*           depthsta.prj is the project file
*               producing depthsta.exe and depthsta.txt from
*            	     starz5.cpp - main module, principal stellar ranging routines.
*                      starz5ag.cpp - Stellar ages and evolution, perturbations.
*                      starz5au.cpp - Neutrino luminosity & stellar wind.
*                      starz5fu.cpp - basic functions.
*                      starz5bo.cpp - bolometric corrections.
*		       starz5op.cpp - supplemental ops.
*                      starz5sc.cpp - stellar ranging.
*
*    Derivation sequence is
*    STARCAL -> STARVIEW -> STAREXP ->STARZ -> STARZ2 ->STARZ3 ->STARZ4(starman).
*                        -> STAREXP4 -> STAREXP5.              ->STARZ5(depthsta).
*
*    STARZ5 series from
*             June 18, 2002 - Sets xcno to 0.01121036 for ZAMS Sirius at 8.7 LY
*                             using Schwarzschild formulae pg.81, ref [8].
*             June 8, 2002 - White dwarf detector added. Red dwarfs, too, June 9.
*    STARZ4 - June 1, 2002 - Stellar distance ranging moved to DEPTHSTA.
*                            STARMAN is still a testbed for stellar fusion.
*                            DEPTHSTA is for better stellar distance calc,
*                            turned out to require seperate compilation.
*             May 31, 2002 - Last public release of STARMAN until next May.
*             May 23, 2002 - Corrections for total luminosity, carbon cycle, ect.
*             May 19, 2002 - Update for Stellar Evolution! Computes the age
*                    of a star from the exhaustion of a hydrogen-burning core.
*             May 18,2002 - Integer to floating, polytrope table minor errors fixed.
*             May 15,2002 - Modular STARCAL system with seperately compiled
*                            modules using the PROJECT file starman.prj.
*
*
*     Compute characteristics of main sequence stars.
*     Prepare 1 model at a time.
*     From distance (parallax), visual magnitude m, and
*     the spectral type (or surface temperature), the
*     program computes  L/Lsun, M/Msun, R/Rsun, the bolometric
*     correction for atmospheric absorption, and the absolute
*     magnitude M.  Then the program applies stellar strucure
*     theory and nuclear astrophysics to the star in order to
*     compute everything else about it, including it's nuclear
*     luminosity, which is compared to the observed luminosity
*     L/Lsun. The chemistry of the star is also accounted for.
*     For white dwarf, supernova core, and neutron
*     star applications, see CHANDRA, [4] also by James A. Green,
*     and THERMONUCLEAR FUSION IN STARS [5] by James A. Green.
*
*     The method used assumes a homogeneously mixed model star
*     in hydrostatic equilibrium with its gas plus radition pressure,
*     leading to a gamma = 4/3 polytrope. However, the nuclear luminosity
*     is then always too large, because hydrogen is consumed in the core
*     of the star, which is in fact burning less hydrogen there than we
*     first assume.  We can adjust the luminosity by lowering the quantity
*     of hydrogen in the hydrogen-burning core, but this varies the
*     abundances there so that the molecular weight increases, and then
*     the temperature is modified. (Lowered! See Clayton, pg.156, eqn.(2-287)).
*     So this model feaures an initially homogeneously
*     mixed-up Zero Age Main Sequence star that turns out to be
*     inhomogeneous in abundance & mu(r) due to its stellar evolution,
*     which depletes the internal store of hydrogen.
*     It is possible to use the perturbed polytrope with a depleted
*     core abundance to compute the age of the Sun or other ZAMS stars by
*     adjusting the hydrogen abundance to normalize the luminosity,
*     so that one can compute the mass of hydrogen burnt and divide by
*     by the luminosity to find the age. The abundances of the elements
*     can be fine-tuned to zero in on the 4.5 billion-year age of the Earth
*     determined by radioactivity measurements. We find 5.5 billion for the Sun.
*     This gamma = 4/3 polytrope model for a star is perfect at first,
*     but is perturbed as the star burns up the internal store of hydrogen
*     and develops a H-depleted core which ultimately collapses when the
*     star moves off the Main Sequence.
*
*     ERRORS:
*     Let R be the radius of the star,
*     d be the distance to it, imprecisely known from the parallax,
*     Ts be the surface temperature,
*     roc be the central density,
*     epsilon be the nuclear energy generation per unit volume.
*
*    From I*4*pi*(d**2) = L = 4*pi*(R**2)*sigma*(Ts**4),
*    R = (d/Ts**2)*sqrt(I/sigma), so that
*    delta_R/R = delta_d/d.  Since a n=3 polytrope has roc = 54.1825*ro_avg,
*    delta_roc/roc = -3*delta_R/R = -3*delta_d/d, then for p-p fusion, since
*
*                 epsilon = alpha*(X**2)*(ro**2)*(T**4), (approximately)
*                               with T = kappa*ro**(1/3), we have
*                               epsilon = v*(X**2)(ro**(10/3)) and so
*
*    delta_epsilon/epsilon  = -10*delta_R/R = (10/3)*delta_roc/roc
*                           = -10*delta_d/d.
*                 delta_R/R = delta_d/d.  
*             delta_roc/roc =  -3*delta_d/d.
*
*  For a ZAMS star, we can adjust d, the distance to the star, until
*  L_nuclear = L_observed to converge on d, if the composition of the star
*  is known and our nuclear energy formulae are precise. For older stars,
*  we can find L_nuclear > L_observed such that the star age is our best
*  guess when the star age is calculated to obtain a better distance d.
*  We can also institute an optional DepthScan feature to search for the
*  best distance d based on L_nuclear. The star can never have negative age,
*  and can never be older than its Schwarzschild Age (13 x 10**9)(M_sun/M)**3 years.
*  The ages can be stored in a starscan array together with M,R,L,X,Y,Z, ect,
*  for each saved trial distance d based on variation around the nominal 
*  parallax disance. The ZAMS distance is the maximum distance away that
*  a stable star may be. Closer distances are more difficult to rule out.
*  However, stars with L_nuclear < L_observed may be contracting to the
*  Main Sequence. Canopus and Bellatrix may both be in this category.
*
*  "Only for about 700 stars are the parallaxes large enough
*  (about 0.05 sec of arc or more) to be measured with a precision
*  of 10% or better." - Donald D. Clayton [7],1965, pg.5.
*        
*	     Copyright: C  James A. Green, 2002 A.D.
*            GREENWOOD RESEARCH BOOKS & SOFTWARE.
*            Catalog: http:grnwd.tripod.com.
*            Email Reports: james_a_green_iii@yahoo.com,
*                        or grnwdrsch@hotmail.com.
*            Suggestions for new features & improvements
*            will be gratefully recieved with acknowlegement.
*
*            USER-MODIFIABLE SOFTWARE - NOTICES.
*            The customer may copy from DEPTHSTA's software
*            listings & modify sections to explore stellar
*            structure modeling according to his own plan.
*            For instance, the customer may introduce
*            new p-p fusion equations, graphics, or new
*            supernova modeling factors, using old routines. 
*            Resale rights for the Original Listing, the
*            original source code and the associated author's books
*            are retained by James A. Green.  This program can
*            be used like a programmers scratchpad to investigate
*            ideas in theoretical astrophysics.  Have at it!
*            BE UP AMONG THE STARS!
*
*
*  HOW TO RUN DEPTHSTA.EXE
*  To execute depthsta.exe, you may use RUN from the START menu of
*  WINDOWS 95/98, directly from the diskette if desired by using RUN
*  on a:depthsta.exe in drive a.
*  Otherwise you may write depthsta.exe into any Windows directory
*  to install it.
*
*  The file depthsta.txt is automatically rewritten each time DEPTHSTA
*  is executed, so save this result file under a different name for
*  long-term storage.
*
*
* STRUCTURAL TABLE OF CONTENTS - starz5.cpp
*
*  MACRO DEFINITIONS
*  EXTERNAL LINKS
*  GLOBAL VARIABLES
*   Compile-time Options - Opacity, Temp Entry, Bolometric Corrections.
*   Constants
*   Input Variables
*   Process Variables
*   Output Variables
*   Program Start - Main
* MAIN SEQUENCE PROCESSING -
*   main() - Main menu and branch to compute or display options.   <ENTRY>
*     mainseq() - Control for input, processing and report generation
*                 for N model stars.
*       processmodels() - Process model stars from basic data: <SCREEN>
*                 distance, visual magnitude, and spectral type.
*                 Add chemistry and polytropic structural
*                  assumptions to test. Generate terminal screen report.
*          nuclearluminosity() - Compute the nuclear luminosity of our model
*                                star via formulae from several astrophysicists.
*                                Includes pp-fusion chain, CNO cycle,
*                                triple-alpha processes. Search key: <NUC>
*                                Cal. partial and total nuclear luminosities.
*             looplimset() - Set looplim to fit polytrope model table
*                            for star j.
*             lumshell() - Compute shell luminosity, model j, shell k.
*             limset() - Set central ball density, model j.
*             limset1() - Set densities, ro0, ro1 and temps t, t1 & bounds
*                         of kth shell for model j.
*             limset2() - Pick central ball external radius.
*             limset(j,i) = Set spacial shell limits for shell i
*                            in model j, r0, r1.
*                clay() - Compute Clayton factors for p-p fusion chain.
*
*      reportms() - Generate a report for 1 stellar model.
*                   Leave the 1-model report in depthsta.txt. Search key: <R>.
*   starz5ag.cpp - Star age & evolution in evolver() sub, for reportms().
*                   Stellar age estimation. See file: starz5ag.cpp.
*   starz5sc.cpp - Contains depthscanner() for stellar ranging.
* 
*          calaratio() - Compute idealized model a/R core compaction
*                        factor at which pp-fusion in uniform
*                        central region = total output
*                        of energy from the star.  Used for reading
*                        supernova tea-leaves.
*          looplimset2() - Set looplim for reports based on polytrope
*                          table for model j.
*       templookup2() - Look up the surface temperture from fractional
*                       spectral types B2.5, ect.
*       temploopup() - Loop up the surface temperature from integer spectral
*                      types B2, B3, ect.
*       initspectable() - Initialize surface temperature from spectrum type table.
*       setspectype() - Lookup the spectral type from the surface temperature.
*       initpolytrope() - Initialize tables of polytropic constants.
*
*    starz5au.cpp - Neutrino Luminosity and Stellar Wind
*       auxluminosities() - Calculate & Prepare neutrino luminosity
*                           and stellar wind figures.
*
*    starz5fu.cpp - Basic functions.
*
*       findbeta(b,C) - Solve (1 - b)/b**4 = C  for  0 < b < 1.
*       nthpower(x,n) - return  x**n.
*       nthroot(x,n) -  return x**(1/n).
*       cuberoot(x)  - return x**(1/3).
*       seventhroot(x) - return x**(1/7).
*       probe(i) - Test probe. Stops execution at test point,
*                     identifies itself as the ith test point,
*                     waits for user command to continue. Debug tool.
*
*    starz5bo.cpp - bolometric corrections
*
*       bolometric(temp) - Return bolometric correction from surface
*                          temp & spectral type.
*          bcprobe(bc,i) - Test bolometric correction value and
*                          complain if bc > 0.0.
*
*    starz5op.cpp - Supplemental ops
*
*       tsurf() - View relationships between surface temp and spectral type.
*       listmainseq() - List star data & help.
*       brighteststars() - List brightest star input data.
*       neareststar() - List nearest star input data.
*       morehelp() - List References, connections, web links.
*       References - Search key <REF>
*
****************************************************************/
//
//    MACRO DEFINITIONS
//
#define PI 3.14159   // constant pi
//
//    EXTERNAL LINKS
//
//                                    These routines are in file starz5sc.cpp
//
//extern void depthscanner(void);     // Depth scan for distance using L_nuclear and
//                                     L_observed to use star age calc for depth factor.
//
//                                  These routines are in file starz5ag.cpp
//
extern void evolver(void);          // Star Age, stellar evolution report gen.
//
//                                  These routines are in file starz5au.cpp.
//
extern void auxluminosities(void); // L_neutrino(R) luminosities & L_solar_wind(R).
//
//                                  These routines are in file starz5fu.cpp.
//
extern double findbeta(double fbetanumber);  // solve fit for beta
extern double nthpower(double x, int n);     // find the nth power of x.
extern double nthroot(double xx, int nn);    // find the nnth root of xx.
extern double cuberoot(double v);            // find the cube root of v.
extern double seventhroot(double v);         // find the seventh root of v.
extern void probe(int i);                    // pause at the ith probe flag for keystroke.
//
//                                 These routines are in file starz4bo.cpp
//
extern double bolometric(double xt); // prototype for B.C. bolometric correction.
extern void bcprobe(double bc, int i); // test bolometric correction bc
//                                 offered by table
//				   the i_th table equation
//
//                                 These routines are in file starz4op.cpp
//
extern void brighteststars(void); // Display brightest star distance,
//                                   visual magnitude, spectral type.
extern void neareststars(void);  // Display nearest star distance,
//                                  visual magnitude, spectral type.
extern void morehelp(void);     // display references.
extern void tsurf(void);     // DISPLAY Tsurface as function of spectral type.
extern void spectraldata(void); // Display more spectral data.
extern void listmainseq(void);    // DISPLAY typical stars & their parameters.
//
//    GLOBAL VARIABLES
//
//                                 kappa = kappa_bf + kappa_ff + kappa_E.
//                                 Opacity test cases.
// COMPILE-TIME OPACITY OPTIONS
//
int opacityflag = 1;  // 1 -> bound-free, free-free, electron scattering
//                       2 -> free-free, electron scattering only.
//                       3 -> electron scattering only.
// 
//
// COMPILE-TIME TEMPERATURE ENTRY OPTIONS
//
int tempentry = 1;  
//                        0 -> Enter spectral type to specify temperature.
//                        1 -> Allow direct surface temperature
//                             entry instead of spectral type alone.
//                   Use spectral type alone for convenience as default.
//
// COMPILE-TIME BOLOMETRIC CORRECTION OPTIONS
//
int bcflag = 0; //  - Select bolometric correction scheme.
//                         1 -> Schwarzschild B.C.
//                              values [1950s],[8] interpolated.
//			   0 -> Zombeck[9] B.C.
//
//
// RUN-TIME OPTIONS
//
int depthscan = 0;   // Run-time option flag for doing distance optimization
//                      based on L_nuclear > L_observed
//                      and StarAge < Zombeck Age = (13 x 10**9)(M_sun/M)**3.
//                      0 -> No Depth Scan around nominal parallax distance.
//                      1 -> Do Depth Scan around nominal parallax distance.
//                           The flag is monitored in <R> reportms().
FILE *fp;           // file pointer for depthsta.txt
int status;
//
struct tm *tm_now; // time-keeping link for time-stamping depthsta.txt
long secs_now;
char *str_now;
//
//   main sequence batch modeling variables and prototypes
//
//
void mainseq(void);     // batch processing for the main sequence.

void initspectable(void);   // init spectraltype -> surface temp table.
double templookup(char x, int y, int z);  // tempature lookup from spectral type.
double templookup2(char x, double y, int z); // temp lookup from spectral type
//                            with fractional spectnum. B1.5, O2.6, ect. allowed.
void initspectable(void);    // prototype for initialization of
//                              spectype-temp table.
void setspectype(double tempsurf, int i); // prototype for lookup
//                                           spec type equiv.
//                        from the surface temperature for i_th entry.
void initpolytrope(void);    // init xr, theta(xr) for n= 3 polytrope,
//                              xxr, xtheta(xxr) for n=1.5 polytrope.
void processmsmodels(void);  // prototype for routine to process
//                              main seq. models.
void nuclearluminosity(int j); // prototype for calc.nuclear
//                                luminosity, model j.
//                              lumshell computes the cumulative
//                              luminosity of shells.
void setconvtest(double t, double t1, double ro, double ro1,
		 double r0, double r1, double k, double m, int i);
//                     test for convection allowed.
double opacity(double ro, double t, int i); // kappa(density,
//                              values [1990] interpolated.
int looplimset(void);      // set the looplimit from number of polytrope
//                            table entries in the polytrope table.
int looplimset2(int i);   // set looplimit differently,
//                           for reportms(), all-inclusively.
double lumshell(double r0, double r1, double epsilonmks0,
         double epsilonmks1, double ro, double ro1, double tluminosity);
void limset0(void);           // set central ball density
//                               and temperature limits.
void limset1(int j, int i);   // set shell density and temp. limits.
void limset2(int j);          // set r for calc.
void limsetr(int j, int i);   // set shell radial limits r0,r1.
double clay(double t, double r,int j);  // computes Clayton
//                                         PP-chain factor.
double psi(double alpha, double ro, double t, int j); // computes
//                                   Clayton psi(alpha) function.
//  				 for bc > 0.0.[ILLEGAL].
void reportms(void);         // prototype, routine to write report
//                              to starz.txt.
double fsn(int gd);          // delta_visual-magnitude for SN
//                              burst on day gd.
double calaratio(int i);     //  compute core compaction factor
//                               a/R using ro0*exp[-(a/R)**3]
//                               model core density.
double calaratio2(int i);   // compute uniform core compaction
//                             factor#2 = r/R
//     using L = ([4pi/3]r**3)*epsilonmks(xcentraldensity[i], xtc[i]).
//     Gives you a feel for size of central ball reqd to produce Lstar
//     at the computed central temperature and density for the
//     polytropic model under test.  Also compute r/Rsun.
double pi = 3.14159;     // pi as a variable.
double m = 1.6748e-27;  // mass of the proton in kg.
int ilast;                   // last model number.  Archaic. keep!
int msmodels = 0;            // number of main sequence models
//                              to be processed.
double scanscale = 1.0; // 1.0 -> +-5% , 2.0 -> +-10% , 4.0 -> +-20% , 0.2 -> +-1%,
//                      0.02 -> +-.1%, Specifies range to scale in around nominal distance.
double percent;         // percent of range for stellar distance scan, above.
double lightyears[1];      // light-years to model star i.            
double visualmag[1];       // visual magnitude of model star i.
double surfacetemp[1];     // surface temperature of model star i.
double photontime; // Photon random-walk diffusion time, pg.22, Phillips.
int abuncode[1];           // abundance code: 1->Schwarzschild,
//                             2->Green, 3->custom, 4->varied.
int starform[1];           // Assumed structure type:
//                           0->n=3, gamma=4/3, Eddington Standard Model.
//                           1->n=1.5, gamma=5/3 polytrope.
//                           2->n=3.5, gamma = 9/7 polytrope, more
//                                     central concentration.
//                           3->n=4.0, gamma=5/4 polytrope,
//                              still more central density.
//
//           Solar Abundance X(Z) by weight, Pop.I model abundances, Clayton.
//
double x[30] = { 0.0, 0.5976, 0.3825, 0.0, 0.0, 0.0,
                 2.8685e-3, 8.9641e-4, 8.3665e-3, 1.1952e-5, 5.9761e-3,
		 2.9880e-5, 3.5856e-4, 2.9880e-5, 5.3785e-4, 0.0,
		 4.1833e-4, 4.1833e-5, 1.7928e-4, 0.0, 4.7808e-5,
		 0.0, 0.0, 0.0, 0.0, 0.0,
		 1.1952e-4, 0.0, 3.5856e-5, 5.9761e-6 };
//
//            Solar Abundance X(Z) by weight, Pop.I model abundances, Schwarzschild.
//

double xx[30] = { 0.0, 0.74184, 0.2373887, 0.0, 0.0, 
		0.0, 7.4184e-4, 2.2552e-3, 5.93472e-3, 0.0, 
		7.4184e-3, 0.0, 7.4184e-4, 0.0, 7.4184e-4,
		0.0, 7.4184e-4, 0.0, 1.48318e-3, 0.0,
		0.0, 0.0, 0.0, 0.0, 0.0,
		0.0, 7.4184e-4, 0.0, 0.0, 0.0 };

double chemx[1];    // X[i] - abundance by weight
//                                 of H in model i.
double chemy[1];   // Y[i] - abundance by weight of He in model i.
double chemz[1];   // Z[i] - abundance by weight of heavy
//                            elements in model i.
//
//     chem[k] - default Clayton chem abundances, recomputed by zetacal()
//              for a specific user choice of X + Y + Z = 1, with X(Z) shaped like
//              the default set below for the heavy elements. Note Z = 0.01999 for Pop.I.
//                    Used to compute electrostatic screening for p-p fusion.
//		Note chem[0] is undefined, refers to no element.
//
double chem[30] = { 0.0, 0.5976, 0.3825, 0.0, 0.0, 0.0,
                 2.8685e-3, 8.9641e-4, 8.3665e-3, 1.1952e-5, 5.9761e-3,
		 2.9880e-5, 3.5856e-4, 2.9880e-5, 5.3785e-4, 0.0,
		 4.1833e-4, 4.1833e-5, 1.7928e-4, 0.0, 4.7808e-5,
		 0.0, 0.0, 0.0, 0.0, 0.0,
		 1.1952e-4, 0.0, 3.5856e-5, 5.9761e-6 };

double chemmu[1];  // mu[i] - molecular weight of model i.
int varymu[1];     // if = 0, normal calc. if = 1, vary mu
//                            with 0 <= X <= 1-Z,
//     computing mu and nuclearluminosity until L_nuclear = L_observed.
//     The calculation gives the mu for the least squared error
//                            (L_nuclear - Lobserved)**2.
//    if varymu[i] = 2, assume heavy elements are all iron, Z=26, A=56.
double xcno = 0.01121036;  // X_CN 0.01121036 is x3 C and x3 N above Aller 1961,
//  where we have upped the value to get X_CN to yeild the ZAMS L_nuc/L_star
//  ratio for Sirius_A at 8.7 LY.  See STARMODL for table-driven only version.
// typical mass fraction of carbon,nitrogen
//                          & oxygen, CNO "carbon cycle".
//  See M.Schwarzschild, STRUCTURE & EVOLUTION OF THE STARS, [8],p.25.
//  Use real_xcno[j] = (chemz[j]/0.02)*xcno to account for deviation
//  from typical profile of Schwarzschild [8] abundances in the case
//  when the user selects his own chemistry and a different
//  heavy element abundance Z.
//
double real_xcno[1];   // Scaled c,n,o abundance by weight, see above.
double savex,savey,savez,savemu; // chemistry save variables when
//                                          varying mu for
double saveclay,savephillips; // nuclear luminosity optimization
double saveschwarz, savecno, savehelium;  //  save variable buffer.
double savextc, savefbeta,savebeta;
int vmui;                          // vmui counter for optimization calc.
double error, leasterror;      // error variables for optimization calc.
double deltax;                    // increment in variation of X.
struct spectral              // spectral type with temperature.
  {
  char spectcode; // spectral OBAFGKM type.
//                   V specifies the Sun for calibration at 5770 K.
  double spectnumber;  // number within type, such as B3, O2, A4, ect.
  int classification;  // 1=giant (III)  2=regular (V).
  double surftemp;  // surface temperature associated with spectral type,
  int equiv; // =1 if spectral type is deduced from temperature, else 0.
  char starname[50];       // star name.
  };                   //      same as surfacetemp. 

struct spectral sp[1], *spec_ptr; // spectral type & temp for model i.
//
double spectable[1][7][11];  // Define table for storage
//                              of temperatures in
//                      degrees Kelvin.
//                    First index = 1 -> giant (III),
//                                = 2 -> regular (V). default.
//                       2nd index corresponds to OBAFGKM,
//                                                0123456,
//                       3rd index is number within type, as in B3.
//                       spectable is initialized by initspectable().
double ro,ro1,t,tc,tc1,t1,r,r0,r1;  //vars for shell density,
//                                    temperature, radius.
// output variables for main sequence modeling.
// Eddington standard n=3 polytrope.
double rawabsmag[1];      // i_th Mv absolute magnitude before
//                          bolometric correction [M_bol].
double bolo[1];          // B.C.= bolometric correction
//                            for atmospheric absorption i.
double bcabsmag[1];      // M_bol - absolute magnitude after
//                          bolometric correction i.
double rawlratio[1];     // L/Lsun w/o B.C. (bolometric correction).
double bclratio[1];      // L/Lsun with bolometric correction B.C.
double rawmratiobest[1]; // M/Msun w/o bolometric correction, best fit.
double bcmratiobest[1];  // M/Msun with bolometric correction, best fit.
double rawrratio[1];     // R/Rsun w/o B.C.
double bcrratio[1];      // R/Rsun with B.C.
double xrstar[1];	 // Rstar in meters.
double rhcore;           // r/R_sun for the hyrogen-burning core.
double rocoremin;        // mininum core density at outer edge of hydrogen-burning core.
double rocoreavg;        // average density of the hydrogen-burning core, kg per m**3.
double zage, vv;  // Zombeck [9]
//          age=13.0e10*[Msun/M]**2.5., vv auxiliary variable.
double hkage;     // Helholtz-Kelvin Contraction Time in years for gas ball.
//          Using the virial theorem and the expression for n = 3 polytropic poten.energy,
//          hkage  =(4.088 x 10**(-30))(M**2)/((L/Lsolar)*R)  years.
double age;  // stellar evolution lifetime of the star.
double ageschw;  // main sequence lifetime,
//                          1/M**2.409 with Schw. Sun age. Vars below moved to starz4ag.cpp
double starage;		 // hydrogen core-depletion age of the star in years.
double delh;		 // hydrogen-abundance delta reqd. to normalize L(R) if Chiu p-p.
//
//  Define storage for 1 sets of output data
//  for best,low,and high values of the mass. Scaled back to just one model.
//   
double xmstar[1]; // Mstar in kg. Via best fit with bolometric correction.
double xavgdensity[1];	    // avg. density in kg/m**3.
double xdensitycentral[1];  // central density in kg/m**3. gamma = 4/3
double xpressurecentral[1]; // central pressure, Newtons/m**2. gamma=4/3.
double pressurek;           // K(beta) in P = K(beta)*(ro**gamma).
double xtc[1];         // central temperature, degrees Kelvin. gamma=4/3.

double xfbeta[1];    	// (1-b)/b**4
//                    = (P**3/ro**4)*[(mu*mh/k)**4]*(a/3). gamma=4/3.
double xbeta[1];       // b                            gamma=4/3.
double xpgas[1];       // central gas pressure.        gamma=4/3
double xpradiation[1]; // central radiation pressure.  gamma=4/3
double xpratio[1];     // pradiation/pgas.             gamma=4/3
double xomega[1];      // stellar binding energy, Joules.gamma=4/3.
double helmholtzkelvin[1];  // Helmholtz-Kelvin contraction time gamma=4/3 case.
double freefalltime[1];  // t= (R**3 / GM)**(1/2) natural period of stellar pulsation.

double zetacal(void);     // Compute zeta in the electrostatic-screening
//                           accelleration of nuclear reactions. [2,7]
double claytonluminosity[1];  // the Clayton pp-fusion chain luminosity.
double scclaytonluminosity[1];  // the screened Clayton pp-fusion chain luminosity.
double nscclaytonluminosity[1];  // Screened, then normalized Clayton pp-fusion chain L(R).
double normalizeclayton = 2.167; // Clayton normalization for
//  Clayton-Aller 1961 abundances. Else Sun has L_nuc=2.167xL_observed.
//  X,Y,Z is 0.5976,0.3825,0.0199 in the envelope. Aller,1961.
//  X,Y,Z is 0.406,0.574,0.0199 in the H-burning core for normalization, L_nuc=L_observed.
//  Now nuclearluminosity() is called to pre-normalize normalizeclayton
//  for any particular choice of abundances, Pop I, Pop II, demo, ect.
double claynormx;        // X required to normalize Clayton due to core exhaustion.
int j;			 // model number j.
double claylum[1][30];   // claytonluminosity/lsun, model j, shell n.
double unclaylum[1][30]; // unscreened claytonluminosity/lsun,
//                          model j, shell n.
double cnoluminosity[1];       // the unscreened carbon (CNO) cycle luminosity, solar units.
double sccnoluminosity;        // the screened (CNO) cycle luminosity.
double heliumluminosity[1];    // the helium-burning triple-alpha
//                                 process luminosity.  Unscreened.
double scheliumluminosity;     // electrostatically screened helium-burning luminosity
double totalhluminosity; // total H-burning L(R) of the star,
//                          P-P chain plus CNO in solar units.
double totalluminosity;  // total H-burning plus He-burning L(R), in solar units.
//
double radequilum[1][30];  // L_radiative = f(r)*(dT/dr)/lsun,
//                             where dT/dR is determined
//                             by the polytrope and f(r) is determined
//                             by the opacity.
//          In radiative equilibrium, L_radiative = chi*L_nuclear,
//          where chi = L_radiative/L_nuclear.
//          Note L_nuclear = L_radiative + L_conduction + L_convection,
//          so that if 1 - chi  < 1, no convection currents can occur,
//          and material must be locked in place.
//          It seems to stand in place anyway, with short convection cell loops.
//  L_nuclear - L_radiative = (1 - chi)L_nuclear
//                                      = L_conduction + L_convection.
double conduclum[1][30];   // Heat conduction luminosity after
//                            Phillips [6],pg.82-83.in Lsun units.
double conveclum[1][30];   // Convection luminosity =
//                            L_nuclear - L_conduc - L_radiation.
//                            Given in Lsun units.
// Polytropic approximation: standard model n=3 polytrope,
//                           gamma = 4/3 (Eddington).
//   T = Tc*theta(xr), P = Pc*[theta(xr)]**4,(total pressure)
//   ro = roc*[theta(xr)]**3, Pgas = P*beta, Pradiation = P*(1 - beta).
//
//  For degenerate matter, we often formally have gamma = 5/3 and
//  n = 1.5, possibly modifying the internal structure if radiation
//  pressure can be ignored.  The gamma = 5/3 structure may also
//  hold for a perfect gas in the absence of radiation pressure
//  according to Bowers & Deeming [1]. However, adding radiation 
//  pressure produces an n=3, gamma = 4/3 polytrope, our standard model
//  Zero Age Main Sequence star, the structure of which is slightly
//  perturbed by hydrogen depletion in the hydrogen-burning core.
//  Note that the equation of radiative equilibrium does not determine
//  dT/dr deep within the star, as this is done by hydrostatic equilibrium,
//  so that the equation of radiative equilibrium only determines
//  L_radiative(r) in the luminosity partition for energy transmission.
//  When gamma = 5/3,   T = Tc*theta(xr), P= Pc*[theta(xr)]**(5/2),
//                      ro = roc*[theta(xr)]**(3/2).
//
double xr[28];		// n=3 , gamma = 4/3 polytrope radius points.
double theta[28];      // n=3 , gamma = 4/3 polytrope theta(xr) at
//                        points xr[0]-xr[27].
double dthetadxr = 0.04231; // n=3 polytrope -d theta/d(xr) at R.
double xxr[23];        // n=1.5, gamma=5/3 polytrope radius points.
double xtheta[23];     //     and theta function.
double zxr[36];       // n=4.0, gamma = 5/4 radius points.
double zxtheta[36];   // n=4.0, gamma = 5/4 theta function values.
double yxr[35];       // n=3.5, gamma=9/7 radius points.
double yxtheta[35];   // n=3.5, gamma = 9/7theta function values.
double aglobal;     // model core radius in km for supernova check,
//                      current model in report.
double coretemp;  // model core temperature, deg.Kelvin.
//                   report variable, global
double corepress; // model core pressure, Newtons/m**2.
//                   report var, global.
double coredensity; // model core density Kg/m**3. report var, global.
//
double rglobal;    // model#2 core radius in km for
//                    stellar structure check.
double coretemp2;  // model#2 core temperature, deg.Kelvin.
//                    report variable, global
double corepress2; // model#2 core pressure, Newtons/m**2.
//                    report var, global.
double coredensity2; // model#2 core density Kg/m**3.
//                      report var, global.
//                       
// END of main sequence batch processing variables
//      and prototype definitions.
//
//  STAR SKETCH variables and definitions -
//
double mu = 0.59435;  // default Schwarzschild molecular weight [8] for
//                       Main Sequence stars in sketch routines.
//
// New Prototype definitions for Borland C++ Compiler, Nov.30, 1999
// ** Variables and prototypes for white dwarf
//    & supernova core calculations. **
//
double rsolar = 6.95987e10;  /* solar radius in cm */
double rsolkm = 6.95987e5;   /* solar radius in km */
double msolar = 1.9892e33;   /* solar mass in grams */
double lsolar = 3.862e33;    /* solar luminosity */
double hbar   = 1.055e-27;   /* Planck's constant in erg/sec */
double sigma  = 5.670e-5;    /* Stephan-Boltzman constant, cgs */
double kb     = 1.381e-16;   /* Boltzmann constant in ergs/degree */
double k      = 1.3805e-23;  /* Boltzmann's constant in J/K.  */
double mproton = 1.673e-24;  /* proton mass in grams */
double mpion   = 2.49e-25;   /* pion mass in grams */
double alpha = 0.3333;	     /* default D-quark mass =
//                              alpha x neutron_mass */
double gnewton = 6.67e-11;  /* Newtons universal constant of gravitation G */
double d;                   // entry variable.
char value[50];             // entry string for variable entry.
//
//  Global variables had to be used,
//  or the compiler would reach it's limit
//  at 64 K of automatic variables.  The new globalized,
//  formally automatic MKSC vars are:
//
double au = 1.496e11;    // astronomical unit in meters.
double c = 2.9979e8;     // speed of light in meters/sec.
double h = 6.6256e-34;   // Planck's constant in mks units = Joule-sec.
double a = 7.564895e-16; // radiation constant a = 4*sigma/c.
double scale = 9.46e15;  // meters per light-year.
double rsun = 6.96e8;    // radius of the sun in meters.
double msun = 1.9892e30; // solar mass in Kg //
double mh = 1.6748e-27;  // mass of the hydrogen atom in kg.
double ep0 = 8.8544e-12; // vacuum permittivity in mks units.
double ec = 1.6021e-19;  // charge of the proton in Coulombs.
double me = 9.1091e-31;  // mass of the electron in kg. = 9.1091e-31 kg.
double lsun = 3.86e26;   // solar luminosity in Watts.
double na = 6.023e23;	 // Avogadro's number.

int select1; // New global select for initial input routine below.
//              Do not reuse.
int okflag1; // New global okflag for initial input routine below.
//              Do not reuse.
/**************************************************
*
*   main - Introduce the Viewer to DEPTHSTA options,
*          then go and do according to choice.
*
****************************************************/
main()
{
                              
    okflag1 = 1;

    while(okflag1 == 1){
	printf("DEPTHSTA for Windows 98. Programmer:James A. Green, 6/18/2002|8:44 AM.\n");
	printf("Greenwood Research,http://grnwd.tripod.com,james_a_green_iii@yahoo.com.\n");
        printf("Basic results for the model you define are written to depthsta.txt.\n");   
	printf("THIS FILE IS REWRITTEN when depthsta.exe is executed, so save files.\n");
	printf("depthsta.txt is written to the same directory depthsta.exe inhabits.\n");
	printf("Aller-Clayton Main Sequence chemical default: X=0.60, Y=0.38, Z=0.02.\n");
	printf("DEPTHSTA scans around the nominal distance to find Lnuc/Lobserved=1,\n");
	printf("the Zero Age Main Sequence distance, & Lnuc/Lobserved > 1 distances\n");
	printf("such that the corresponding star age < Main Sequence Stellar Lifetime.\n");
	printf("The ZAMS distance is an upper limit for stable star distance. However,\n");
	printf("stars with L_nuc < L_observed may be contracting to the Main Sequence.\n");
	printf("STAR       Derived Properties [Book] || Observed Properties || III=Giant\n");
	printf("NAME(Constella) M/Msun R/Rsun  L/Lsun Distance mag m Spectrum->Tsurf\n");
	printf("Sun             1.0    1.0     1.0    1 AU    -26.74  G2       5770K\n");
	printf("Aldebaran(Tau)        40.0(bhm)125    68 LY    0.86   K5 III   3400K\n");
	printf("Altair(Aquila)                 11     16 LY    0.77   A7 V     8190K\n");
	printf("Bellatrix(Ori)                 4000   470 LY   1.64   B2 III   22100K\n");
	printf("Deneb(Cygnus)   25      60(bhm)60,000 1600 LY  1.26   A2 Ia    9700K\n");
	printf("Denebola(Leo)                  20     43 LY    2.14   A3 V     9360K\n");	
	printf("Hamal(Aries)                   70      75 LY   2.00   K2 III   4210K\n");
	printf("Regulus(Leo)            5(bhm) 160     85 LY   1.36   B7 V     13000K\n");	
	printf("Sirius(Ca.Ma.)   2.35 1.8(bhm) 23     8.7 LY  -1.42   A1       10265K\n");
	printf("Enter 1 to display more sample input - main sequence star data & help.\n");
	printf("      2 to compute from distance, visual magnitude, and spectral type,\n");
	printf("      3 to exit DEPTHSTA, 4 to view spectral-type vs. temp.: ");
	scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select1 = d;
	
	printf("Entered %d\n",select1);
        if(select1 == 1)
         	listmainseq();
        else if(select1 == 2)
         	mainseq();
	else if(select1 == 3)
		okflag1 = 0;
	else if(select1 == 4)
	       tsurf();  // show how temperatures are
//                          connected to spectral types
	else if(select1 == 5)
	       spectraldata();
	else{
	   printf("Input select switch underfined. Try again.");
        }
    }
    printf("Exit DEPTHSTA.\n");
}
/*********************************************************
*
*   mainseq - Main Sequence Modeling. 
*             Enter parameters for 1 star, and
*             compute its characteristics in the
*             output file depthsta.txt.
*             Perfect for the lower Zero Age Main Sequence
*             running on simple p-p fusion, thanks to Clayton.
*             Uses perturbation of the n=3 polytrope model for
*             the effects of hydrogen consumption in the
*             hyrdogen-burning core to compute the actual
*             age of the star. Well-tested on the Sun.  <ENTRY>
*
**********************************************************/
void mainseq(void)
{
    int i;         // model number - archaic remnant. Must keep!
    int lastmodel; // number of last model - an archaic remnant.
    int entry;     // enable initial variable entry = 1.
    int select;    // selector switch
    int gk;        // sp[i].spectnumber if integer after a test.
    double g;      // g = sp[i].spectnumber.
    double xx,yy;  // temp vars for percent by weight H & He.
    double ly;     // distance in light-years.
    double dmeters; // distance in meters.
    double pc;      // distance in parsecs.
    double parallax;  // distance as parallax in arcseconds.   
    double astrounits;  // distance in astronomical units.   
    double vm;          // visual magnitude
    double tempsurf;    // surface temperature.
    char spectstring[10];  // string for entering first char of OBAFGKM.
    char spect;            // spectral type letter code OBAFGKM.
    int spectnum;      // number designator within type - as in B3, ect.
    double dspectnum;  // fractional types allowed - B3.3, A4.2, ect.
    int spectclass;    // 1= giant (III) or 2=regular (V).
    int xspectclass;   // mapping variable to map user-friendly
//                       spect type vars to
//                       internal spect class code.
    int okflag;     // keep inputting type until input valid.
    int starbrand;  //  starform[i] = 0->gamma=4/3, n=3,   P = P_gas + P_radiation.
//                                    1->gamma=5/3, n=1.5, P = P_gas
//     History - Still investigating  2->n=3.5, gamma = 9/7
//       who 1st produced n=3  	         more central concentration.
//       polytropic stars.            3->n=4.0, gamma= 5/4 still
//       Not Homer Lane. Emden?	  	 more central concentration.
//       Quite possibly Eddington.  See also Chandrasekhar.
//
// Central density and total luminosity increase with increasing index n.
// n=3, gamma = (n+1)/n = 4/3 polytropes are the unique solutions for
// a homogeneously mixed star in gravitational equilibrium with its own
// gas plus radiation pressure, determining ro(r), P(r), T(r) from M, R,
// and mu(r), the molecular weight. The equation of radiative equilibrium
// is often misapplied to the total luminosity L(r), instead of the
// partial luminosity L_radiative(r), confusing the structural equations.
// In reality, dT/dr is determined deep inside the star by hydrostatic
// equilibrium and determines L_raditive(r) in the case of static stars.
// When no radiation pressure exists, n=1.5 and gamma = (n+1)/n = 5/3.
// The central density is 54.1825 x the avg density when n=3, gamma = 4/3,
// and is  5.99071 x the average density when n=1.5, gamma = 5/3.
//
    initspectable();  // initialize spectral_type -> surface
//                                                   temperature table.
    initpolytrope();  // initialize rx radial points,
//                       theta(rx) for n=3 polytrope.
    msmodels = 0;     // default number of main sequence
//			 models to process = 0.
    i = 0;            // only 1 model is processed by depthsta.exe.
    entry = 1;
    while(i < 1 && entry == 1){   // Enter the variables describing the model.
       printf("\nMAIN SEQUENCE BATCH MODELING VARIABLE INPUT for 1 star.\n");
       printf("The output report file depthsta.txt is in same directory as DEPTHSTA.\n");
       printf("Model=%d. DEPTHSTA computes distance-displaced models.\n",i);       
       printf("This section enters The distance [or parallax], The name\n");
       printf("of the star as one word, The ZAMS polytropic structure n=3.0,\n");
       printf("The visual magnitude m, The spectral type,[or surface temperature],\n");
       printf("The Clayton-Aller-1961 Pop.I chemistry or alternative.\n");       
    
       okflag = 0;
       while(okflag == 0){        // Enter the distance to the star.
	 depthscan = 0;   // make no depth scan the default decision.
	 scanscale = 1.0;  // 1.0 -> +-5% , 2.0 -> +-10% , 4.0 -> +-20% ,
//	                      8.0 -> +-40% , 0.2 -> +-1%
	 percent = 5.0;      // Default +-5% of nominal distance for scan range
	 printf("\nDEPTH SCAN DISTANCE ENTRY -  1 AU to SUN, 8.7 LY to Sirius.\n");
	 printf("Scans +- 5 percent around the nominal distance for focus in 20 steps.\n");
         printf("Enter 1 to enter the nominal distance in light-years,\n");
         printf("      2 to enter the nominal distance in parsecs,\n");
	 printf("      3 to enter the nominal distance as parallax in arc-seconds,\n");
	 printf("      4 to enter the nominal distance in AU,\n");
	 printf("Coarse scanning options -> +- 10 percent or +- 20 percent,\n");
	 printf("      5 to enter distance in light-years and depth scan\n");
	 printf("        +- 10% around this nominal distance for focus in 20 steps,\n");
	 printf("      6 to enter distance in light-years and depth scan\n");
	 printf("        +- 20% around this nominal distance for focus in 20 steps,\n");
	 printf("      7 to enter distance in light-years and depth scan\n");
	 printf("        +- 40% around this nominal distance for focus in 20 steps,\n");
	 printf("Fine scanning options -> +- 1 percent, \n");
	 printf("      8 to enter distance in light-years and depth scan\n");
	 printf("        +- 1% around this nominal distance for focus in 20 steps\n");
	 printf("      9 to enter distance in light-years and depth scan\n");
	 printf("        +- 0.1% around this nominal distance for focus in 20 steps: ");
         scanf("%s",value);    // This entry procedure is
	 d = atof(value);     //  more immune to invalid inputs
	 select = d;          //  than simple scanf().
	 printf("\nEntered %s.\n", value);
	 if( select >= 1 && select <= 9)
	    okflag = 1;
	 else
	    printf("Entry out of range. Try again.\n");
       }
       if(select == 1){
	  okflag = 0;
	  while(okflag == 0){
	     printf("Enter floating-point distance in light-years: ");
	     scanf("%s",value);  // entry procedure more immune to
	     ly = atof(value);   // invalid inputs than simple scanf
	     if(ly <= 0.0)
		printf("ly = %s is an illegal value. Try again.\n",
		value);
	     else
	       okflag = 1;
          }
          lightyears[i] = ly;
	  printf("\nEntered %e light-years,\n", ly);
          pc = ly/3.26;
	  printf("          %e parsecs,\n",pc);
          dmeters = ly*scale;
          printf("          %e meters,\n", dmeters);
	  parallax = au/(ly*scale);  // cal parallax in radians.
	  parallax = (180.0/3.14159)*parallax; // parallax in degrees.
          parallax = parallax*(60.0)*(60.0); // parallax in arc-seconds.
	  printf("          %e arc-seconds of parallax for model=%d.\n",
	  parallax,i);
       }
       if(select == 2){
	  printf("Enter floating-point distance in parsecs: ");
	  scanf("%s",value);    // This
	  d = atof(value);     // entry procedure is more
//                                immune to invalid inputs
	  pc = d;              // than simple scanf().
	  printf("\nEntered %e parsecs.\n",pc);
	  ly = 3.26*pc;
          lightyears[i] = ly;
	  printf("        %e light-years,\n",ly);
          dmeters = scale*ly;
	  printf("        %e meters,\n",dmeters);
	  parallax = au/(ly*scale); // cal parallax in radians.
	  parallax = parallax*(180.0/3.14159); // in degrees.
          parallax = parallax*(60.0)*(60.0);  // in arc-seconds.
	  printf("        %e arc-seconds of parallax for model=%d.\n",
	  parallax,i);
       }
       if(select == 3){
	  printf("Enter floating-point parallax in arc-seconds: ");
	  scanf("%s",value);    // This entry procedure is
	  d = atof(value);     // more immune to invalid inputs
	  parallax = d;        // than simple scanf().
	  printf("\nEntered %e arc-seconds.\n", parallax);
	  parallax = parallax/((60.0)*(60.0)); // parallax in degrees.
	  parallax = (3.14159/180.0)*parallax; // and in radians.
	  dmeters = au/parallax;
	  printf("        %e meters,\n", dmeters);
	  ly = dmeters/scale;
          lightyears[i] = ly;
	  printf("        %e light-years,\n", ly);
          pc = ly/3.26;
	  printf("        %e parsecs for model=%d.\n", pc,i);
       }
       if(select == 4){
	  printf("Enter floating-point distance in AU: ");
	  scanf("%s",value);   // This entry procedure is
	  d = atof(value);     // more immune to invalid inputs
	  astrounits = d;      // than simple scanf().
	  printf("\nEntered %e AU.\n", astrounits);
	  
	  dmeters = astrounits*au;
	  parallax = au/dmeters;  // calc. parallax in radians.
	  parallax = parallax*(180.0/3.14159);  // and in degrees.
          parallax = parallax*(60.0*60.0); // and in seconds of arc.
	  printf("        %e meters,\n", dmeters);
	  ly = (1.581395349e-5)*astrounits;   // use lightyears per AU.
          lightyears[i] = ly;
	  printf("        %e light-years,\n", ly);
          pc = ly/3.26;
	  printf("        %e parsecs for model=%d.\n", pc,i);
	  printf("        %e arc-seconds of parallax.\n",parallax);
       }
       if(select == 5){
	  okflag = 0;
	  while(okflag == 0){
	     printf("Enter nominal floating-point distance in light-years: ");
	     scanf("%s",value);  // entry procedure more immune to
	     ly = atof(value);   // invalid inputs than simple scanf
	     if(ly <= 0.0)
		printf("ly = %s is an illegal value. Try again.\n",
		value);
	     else
	       okflag = 1;
	  }
	  lightyears[i] = ly;
	  printf("\nEntered %e light-years nominal distance,\n", ly);
	  pc = ly/3.26;
	  printf("          %e parsecs,\n",pc);
	  dmeters = ly*scale;
	  printf("          %e meters,\n", dmeters);
	  parallax = au/(ly*scale);  // cal parallax in radians.
	  parallax = (180.0/3.14159)*parallax; // parallax in degrees.
	  parallax = parallax*(60.0)*(60.0); // parallax in arc-seconds.
	  printf("          %e arc-seconds of parallax for model=%d.\n",
	  parallax,i);
	  scanscale = 2.0; // Use plus-minus 10 percent scan scale.
	  percent = 10.0;      //  +-10% of nominal distance for scan range
       }
       if(select == 6){
	  okflag = 0;
	  while(okflag == 0){
	     printf("Enter nominal floating-point distance in light-years: ");
	     scanf("%s",value);  // entry procedure more immune to
	     ly = atof(value);   // invalid inputs than simple scanf
	     if(ly <= 0.0)
		printf("ly = %s is an illegal value. Try again.\n",
		value);
	     else
	       okflag = 1;
	  }
	  lightyears[i] = ly;
	  printf("\nEntered %e light-years nominal distance,\n", ly);
	  pc = ly/3.26;
	  printf("          %e parsecs,\n",pc);
	  dmeters = ly*scale;
	  printf("          %e meters,\n", dmeters);
	  parallax = au/(ly*scale);  // cal parallax in radians.
	  parallax = (180.0/3.14159)*parallax; // parallax in degrees.
	  parallax = parallax*(60.0)*(60.0); // parallax in arc-seconds.
	  printf("          %e arc-seconds of parallax for model=%d.\n",
	  parallax,i);
	  scanscale = 4.0; // Use plus-minus 20 percent scan scale.
	  percent = 20.0;  //  +-20% of nominal distance for scan range
       }
       if(select == 7){	  
	  okflag = 0;
	  while(okflag == 0){
	     printf("Enter nominal floating-point distance in light-years: ");
	     scanf("%s",value);  // entry procedure more immune to
	     ly = atof(value);   // invalid inputs than simple scanf
	     if(ly <= 0.0)
		printf("ly = %s is an illegal value. Try again.\n",
		value);
	     else
	       okflag = 1;
	  }
	  lightyears[i] = ly;
	  printf("\nEntered %e light-years nominal distance,\n", ly);
	  pc = ly/3.26;
	  printf("          %e parsecs,\n",pc);
	  dmeters = ly*scale;
	  printf("          %e meters,\n", dmeters);
	  parallax = au/(ly*scale);  // cal parallax in radians.
	  parallax = (180.0/3.14159)*parallax; // parallax in degrees.
	  parallax = parallax*(60.0)*(60.0); // parallax in arc-seconds.
	  printf("          %e arc-seconds of parallax for model=%d.\n",
	  parallax,i);
	  scanscale = 8.0; // Use plus-minus 1 percent scan scale.
	  percent = 40.0;      //  +-1% of nominal distance for scan range
       }
       if(select == 8){	  
	  okflag = 0;
	  while(okflag == 0){
	     printf("Enter nominal floating-point distance in light-years: ");
	     scanf("%s",value);  // entry procedure more immune to
	     ly = atof(value);   // invalid inputs than simple scanf
	     if(ly <= 0.0)
		printf("ly = %s is an illegal value. Try again.\n",
		value);
	     else
	       okflag = 1;
	  }
	  lightyears[i] = ly;
	  printf("\nEntered %e light-years nominal distance,\n", ly);
	  pc = ly/3.26;
	  printf("          %e parsecs,\n",pc);
	  dmeters = ly*scale;
	  printf("          %e meters,\n", dmeters);
	  parallax = au/(ly*scale);  // cal parallax in radians.
	  parallax = (180.0/3.14159)*parallax; // parallax in degrees.
	  parallax = parallax*(60.0)*(60.0); // parallax in arc-seconds.
	  printf("          %e arc-seconds of parallax for model=%d.\n",
	  parallax,i);
	  scanscale = 0.2; // Use plus-minus 1 percent scan scale.
	  percent = 1.0;   //  +- 1% of nominal distance for scan range
       }
       if(select == 9){
	  okflag = 0;
	  while(okflag == 0){
	     printf("Enter nominal floating-point distance in light-years: ");
	     scanf("%s",value);  // entry procedure more immune to
	     ly = atof(value);   // invalid inputs than simple scanf
	     if(ly <= 0.0)
		printf("ly = %s is an illegal value. Try again.\n",
		value);
	     else
	       okflag = 1;
	  }
	  lightyears[i] = ly;
	  printf("\nEntered %e light-years nominal distance,\n", ly);
	  pc = ly/3.26;
	  printf("          %e parsecs,\n",pc);
	  dmeters = ly*scale;
	  printf("          %e meters,\n", dmeters);
	  parallax = au/(ly*scale);  // cal parallax in radians.
	  parallax = (180.0/3.14159)*parallax; // parallax in degrees.
	  parallax = parallax*(60.0)*(60.0); // parallax in arc-seconds.
	  printf("          %e arc-seconds of parallax for model=%d.\n",
	  parallax,i);
	  scanscale = 0.02; // Use plus-minus 0.1 percent scan scale.
	  percent = 0.10;      //  +-0.1% of nominal distance for scan range
       }
//  
// if entry == 1, you can keep entering, otherwise, discontinue.
// 
       if( entry == 1){
	
	   printf("\nSTAR NAME ENTRY - Including notes, ray & declination, ect.\n");
	   printf("NO SPACES. Use underscores instead, as in Sirius_CMa.\n");
	   printf("Enter star name < 50 chars: ");
           scanf("%s",sp[i].starname);
	   printf("Entered starname = %s.\n",sp[i].starname);

	   okflag = 0;
           while(okflag == 0){
             printf("\nSTRUCTURAL ASSUMPTIONS - POLYTROPIC STARS.\n");
	     printf("Enter 0 n=3 Eddington Standard Model Star Structural Assumptions,\n");
	     printf("        [n=3, gamma=4/3 polytrope.][brigher than 1, physical.]\n");
	     printf("        [Central density = 54.1825 x avg_density.]\n");
	     printf("        This is the best assumption for small stars like the Sun,\n");
	     printf("        including both gas & radiation pressure for sizable stars,\n");
	     printf("        & modeling hydrostatic equilibrium. As Main Sequence stars\n");
	     printf("        age, their H-burning cores change their molecular weight\n");
	     printf("        in the T > 10**7 K core until hydrogen is exhausted and the\n");
	     printf("        core collapses, igniting the next fuel or spawning a white\n");
	     printf("        dwarf or a red giant with a highly degenerate He core.\n");
	     printf("        Note convective mixing little modifies mu(r) stability.\n");
	     printf("        By calculating the mass of the H-burning core & normalizing\n");
	     printf("        L_nuclear/L_star to unity by adjusting X(H), we can\n");
	     printf("        compute the mass of hydrogen burned since star formation,\n");
	     printf("        calculate the energy released, and divide by the luminosity\n");
	     printf("        to estimate the age of Main Sequence stars. These n=3 \n");
	     printf("        polytrope models may also well-approximate new stars during\n");
	     printf("        their Helmholtz-Kelvin contraction to the Main Sequence,\n");
	     printf("        although the mass-luminosity law may not then apply.\n");
	     printf("      1 n=1.5, gamma=5/3 polytrope [dimmer than 0, physical.] \n");
	     printf("        This is the best assumption for radiationless gas objects.\n");
	     printf("        [Central concentration of matter and brightness increase with n.\n");
	     printf("        [Central density = 5.99071 x avg_density.]: ");
//
//           Higher-order polytropes presently generate exceptions, domain errors, ect.
//
//	     printf("      2 n = 3.5, gamma=9/7, density_central = 152.884 x avg_density.\n");
//	     printf("        Models higher central concentration of matter, but\n");
//	     printf("        not so well physically motovated via equations of state. \n");
//	     printf("        Note that heavier, denser cores produce hotter burning,\n");
//	     printf("        and that many stars finally hatch a dense white dwarf\n");
//	     printf("      3 n = 4.0, gamma=5/4,density_central = 622.408 x avg_density.\n");
//	     printf("        Models higher central concentation of matter, but\n");
//	     printf("        not so well physically motovated via equations of state: ");

	     scanf("%s",value);   // This entry procedure is
	     d = atof(value);     // more immune to invalid inputs
	     starbrand = d;      // than simple scanf().
	     printf("\nEntered %d.\n",starbrand);
	     if(starbrand != 0 && starbrand != 1 && starbrand != 2 )
		printf("Error: Re-enter the structural assumptions, 0-2.\n");
	     else{
	        starform[i] = starbrand;
	        okflag = 1;
	     }
	   }

	   printf("\nVISUAL MAGNITUDE ENTRY - model=%d. -26.74 is Sun at 1 AU, T=5770 K.\n",i);
	   printf("                                  -1.42 is Sirius_A at 8.7 LY, Type=A1.\n");
	   printf("                                  8.65 is Sirius_B at 8.7 LY, Type=A5.\n");
	   printf("Enter floating-point visual magnitude m : ");
	   scanf("%s",value);   // This entry procedure is
	   d = atof(value);     // more immune to invalid inputs
	   vm = d;              // than simple scanf().
	   printf("Entered visual magnitude = %e.\n\n",vm);
	   visualmag[i] = vm;
	   printf("SURFACE TEMPERATURE OR SPECTRAL TYPE ENTRY - model=%d.\n",i);
	   if(tempentry == 1){ // tempentry = 1 is a compile-time option.
//                                Use 0 for convenience.
	       printf("Enter 1 to enter the surface temp T_surface in degrees K, or\n");
	       printf("      2 to enter the spectral type specifying T_surface: ");
	       scanf("%s",value);   // This entry procedure is
	       d = atof(value);     // more immune to invalid inputs
	       select = d;          // than simple scanf().
	       printf("Entered %d.\n\n",select);
           }
// 	   else
// 	     select = 2;  // Mod to force spectral type entry only.
//   For user-friendly simplification. Skips select=1 code for now.
	   if(select == 1){
	       printf("Enter floating-point surface temp in degrees K: ");
	       scanf("%s",value);   // This entry procedure is
	       d = atof(value);     // more immune to invalid inputs
	       tempsurf = d;      // than simple scanf().
	       printf("Entered %e degrees Kelvin for model=%d.\n", tempsurf,i);
	       surfacetemp[i] = tempsurf;
	       sp[i].surftemp = tempsurf;
	       sp[i].equiv = 1;  // spectral type equivilant
//                                  must be deduced from surf. temp.
	       setspectype(tempsurf,i);  // lookup spectral type
//                                          OBAFGKM 0-10 from tempsurf
//                                        and set sp[i].spectcode,
//                                        sp[i].spectnumber.
               g = sp[i].spectnumber;
	       if(g == 0.0 || g == 1.0 || g == 2.0
	       || g == 3.0 || g == 4.0 || g == 5.0 || g == 6.0
	       || g == 7.0 || g == 8.0 || g == 9.0 || g == 10.0){
		     gk = g; // subsitute integrer for fractional type.
		     printf("Equivilant to temp of spectral type %c%d V.\n\n",
		     sp[i].spectcode, gk);
	       }
               else
		 printf("Equivilant to temp of spectral type %c%2.1f V.\n\n",
		 sp[i].spectcode, sp[i].spectnumber);
	   }
	   else{          // if select code != 1, assume
//                           spectral type entry, select == 2.
	       okflag = 0;
               while(okflag == 0){
		 printf("OBAFGKM Spectral types like O3 V [regular] or O3 III [giant] accepted.\n");
		 printf("Exotic types: W parallels O, R parallels K, N parallels M\n");
		 printf("              S parallels M.\n");
		 printf("For calibration only, type V specifies the Sun at 5770 deg K.\n");
                 printf("V-type calibration entries ignore the following digit.\n");
		 printf("Enter single-char code value for spectral type O,B,A,F,G,K or M:");
		 scanf("%s",spectstring);
		 spect = spectstring[0];
		 if(spect == 'o')
		    spect = 'O';
		 if(spect == 'b')
		    spect = 'B';
		 if(spect == 'a')
		    spect = 'A';
		 if(spect == 'f')
		    spect = 'F';
		 if(spect == 'g')
		    spect = 'G';
		 if(spect == 'k')
		    spect = 'K';
		 if(spect == 'm')
		    spect = 'M';
		 if(spect == 'v')  // special sun symbol for calibration.
                    spect = 'V';

		 printf("\nInitial spectral type = %c.\n",spect);
		 
		 if(spect != 'O' && spect != 'B' && spect != 'A'
		     && spect != 'F' && spect != 'G'  && spect != 'K'
		     && spect != 'M' && spect != 'V'){
		     printf("Spectral type not defined in DEPTHSTA. Use OBAFGKM.\n");
		     printf("Exotic types: W parallels O, R parallels K, N parallels M\n");
		     printf("S parallels M.\n");
                     printf("V used for the Sun in calibration checks only.\n");
                 }
		 else{
		   okflag = 1;
		   sp[i].spectcode = spect;
                 }
	       }
	       okflag = 0;
               while(okflag == 0){
		  printf("Enter floating-point value for the type [3.0 for A3, ect.]: ");
		  scanf("%s",value);   // This entry procedure is
	          d = atof(value);     // is more immune to invalid inputs
	          dspectnum = d;      // than simple scanf().

		  printf("\nEntered %2.1f.\n",dspectnum);
		  if(dspectnum <= 10.0 && dspectnum >= 0.0){
		      okflag = 1;
		      sp[i].spectnumber = dspectnum;
                  }
                  else printf("Number must be in the range 0.0 through 10.0.\n");
	       }

	       g = dspectnum;
	       if(g == 0.0 || g == 1.0 || g == 2.0 || g == 3.0
	       || g == 4.0 || g == 5.0 || g == 6.0 || g == 7.0
	       || g == 8.0 || g == 9.0 || g == 10.0){
		     gk = g; // subsitute integrer for fractional type.
 		     printf("\nNow spectral type = %c%d.\n", spect, gk);
	       }
               else
		 printf("\nNow spectral type %c%2.1f.\n",
		 spect,dspectnum);
	       
	       okflag = 0;
	       while(okflag == 0){
		  printf("\nCLASSIFICATION - model=%d. These are just used as tags\n",
		   i);
		  printf("by DEPTHSTA, except that giants [III] cause surface temp\n");
		  printf("to be selected diferently than for dwarf [V], the default.\n");
	          printf("Enter 1 for supergiants Ia, Ib, I.\n");
		  printf("      2 for bright giants II.\n");
		  printf("      3 for giants III.\n");
		  printf("      4 for subgiants IV.\n");
		  printf("      5 for ZAMS V zero-age main sequence.\n");
		  printf("      6 for subdwarfs VI.\n");
		  printf("      7 for white dwarfs VII.\n");
		  printf("      8 for regular [dwarf V] class or if class is unknown: ");
		  scanf("%s",value);   // This entry procedure is
	          d = atof(value);     // more immune to invalid inputs
	          xspectclass = d;      // than simple scanf().
		  
		  if(xspectclass >= 1 && xspectclass <= 8){
		       g = sp[i].spectnumber;
		       if(g == 0.0 || g == 1.0 || g == 2.0 || g == 3.0
		       || g == 4.0 || g == 5.0 || g == 6.0 || g == 7.0
		       || g == 8.0 || g == 9.0 || g == 10.0){
			    gk = g; // subsitute integrer
//                                     for fractional type.
			    printf("\nNow spectral type = %c%d",
			    sp[i].spectcode, gk);
	               }
                       else
			   printf("\nNow spectral type = %c%2.1f",
			   sp[i].spectcode, sp[i].spectnumber);
                  }             // Due to accidents of history, the old 
		  if(xspectclass == 1){ // internal code is
//                                      different from user-friendly.
		      printf(" I, supergiant.\n"); //  1 -> III giants.
		      okflag = 1; spectclass = 3;  //  2 -> regular dwarf V
                  }                            //  3 -> supergiants Ia,Ib,I
	          if(xspectclass == 2){            //  4 -> subgiants IV.
		     printf(" II, bright giant.\n"); //  5 -> ZAMS V
		     okflag = 1; spectclass = 8;     //  6 -> VI subdwarfs
		  }                            //  7 -> VII white dwarf
		  if(xspectclass == 3){        //  8 -> bright giants II.
		     printf(" III, giant.\n");     // This section
//                                             translates user-friendly
		     okflag = 1;  spectclass = 1;  // code to the
//                                              old format as a temp fix.
		  }
		  if(xspectclass == 4){
		     printf(" IV subgiant.\n");
		     okflag = 1;  spectclass= 4;
		  }
		  if(xspectclass == 5){
		     printf(" ZAMS V, zero-age main seq.\n");
		     okflag = 1;   spectclass = 5;
		  }
		  if(xspectclass == 6){
		     printf(" VI, subdwarf.\n");
		     okflag = 1;  spectclass = 6;
		  }
		  if(xspectclass == 7){
		     printf(" VII, whitedwarf\n");
		     okflag = 1;  spectclass = 7;
		  }
		  if(xspectclass == 8){
		     printf(" V, dwarf or regular.\n");
		     okflag = 1;  spectclass = 2;
                  }                     
		  if(xspectclass < 1 || xspectclass > 8)
		      printf("\n Domain Error: Spectral class must be 1-8 above.\n");
               }	       
	       sp[i].classification = spectclass;
	       sp[i].equiv = 0;   // spectral not deduced, but specified.
	       sp[i].surftemp = templookup2(spect,dspectnum,spectclass);
	       surfacetemp[i] = sp[i].surftemp;
	       if(sp[i].surftemp == 0.0){    // default to regular temp assignments.
		     printf("This temp undefined for giants [III]. Use default [2 for V].\n");
		     sp[i].surftemp = templookup2(spect,dspectnum,2);
		     surfacetemp[i] = sp[i].surftemp;
	       }
	       printf("Surface_temp[spectype] of star = %e Kelvin.\n",
	       sp[i].surftemp);
	   }  // select == 2 is over with.
//
// Enter chemistry of star model i.
//
       varymu[i] = 0;  // Set flag to vary mu in nuclear luminosity
//                        calculations to zero
//                        until  the user requests option 4 below.
       printf("\nMOLECULAR WEIGHT & CHEMISTRY ENTRY - model=%d.\n",i);
       printf("model = %d.  [First model = 0.]\n",i);
       printf("Enter 1 for Schwarzschild chemistry, X=0.75,Y=0.23,Z=0.02,\n");
       printf("        for Old Population I [galactic disk] stars.\n");
       printf("      2 for Green's helium-rich chemistry, X = 0.63,Y=0.35,Z=0.02.\n");
       printf("      3 to specify a custom X,Y,Z abundance scheme.\n");
       printf("        Note that Schwarzschild gives heavy element abundances as\n");
       printf("        Population      Abundance Z of Heavy Elements\n");
       printf("         Young Pop.I         0.04   galactic disk\n");
       printf("         Intermed. Pop.I     0.03   galactic disk\n");
       printf("         Old Pop.I           0.02   galactic disk, includes Sun.\n");
       printf("         Mild Pop.II         0.01   galactic halo\n");
       printf("         Extreme Pop.II      0.003  galactic halo. \n");
//       printf("       4 to vary the molecular weight with 0 <= X <= 1-Z\n");
//       printf("         in order to obtain a best fit to the luminosity L.\n");
//       printf("         This should be used with n=3, gamma=4/3 structure, the\n");
//       printf("         Eddington Standard Structure. Then the equation of\n");
//       printf("         radiative equilibrium can be used to figure the partition\n");
//       printf("         of L_total = L_radiative + L_conductive + L_convective\n");
//       printf("       5 Same as above, but assume that Z pertains to iron\n");
//       printf("         only, with Z=26 and A =56. The effect on on the molecular\n");
//       printf("         weight is carefully computed & a wee bit more impacting.\n");
       printf("       4 Experimental Chemistry X=0.61, Y=.37, Z=.02. \n");
       printf("       5 Clayton Sun chemistry X=0.5976, Y=.3825, Z=.0199 : ");
       scanf("%s",value);   // This is more immune 
       d = atof(value);     // to invalid inputs
       select = d;      // than simple scanf().
       
       printf("\nEntered %d.\n",select);
         if(select == 1){
	    chemx[i]  = 0.75;  chemy[i] = 0.23, chemz[i] = 0.02;
	    chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.5*chemz[i]));
            real_xcno[i] = xcno;
	 }
	 if(select == 4){
	    chemx[i]  = 0.61;  chemy[i] = 0.37, chemz[i] = 0.02;
	    chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.5*chemz[i]));
            real_xcno[i] = xcno;
	 }
	 if(select == 5){
	    chemx[i]  = 0.5976;  chemy[i] = 0.3825, chemz[i] = 0.0199;
	    chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.5*chemz[i]));
            real_xcno[i] = xcno;
	 }
         if(select == 2){
            chemx[i]  = 0.63;  chemy[i] = 0.35, chemz[i] = 0.02;
	    chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.5*chemz[i]));
            real_xcno[i] = xcno;
	 }

//	 if(select == 4 || select == 5){   // old variation of abundances routine
//	   varymu[i] = 1;  // set flag to vary mu in
//                           nuclear luminoisty calculations.
//	   if(select == 5)
//	     varymu[i] = 2; // specify pure iron as heavy elements.
//
//	   okflag = 0;
//	   while(okflag == 0){
//	     printf("Enter Z, default = 0.02 : ");
//	     scanf("%s",value);
//	     d = atof(value);
//	     if(d >= 1.0){
//		printf("Z >= 1.0. Try again.\n");
//	     }
//	     else{
//		chemz[i] = d;
//		chemy[i] = 0.001;   // keep He non-zero initially.
//		chemx[i] = 1.0 - chemz[i] - chemy[i];    // Set X,Y,mu
//                                                   to initial values.
//		if(varymu[i] == 1) // Start with max concentration
//                                    of H, keep He non-zero.
//		    chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.5*chemz[i]));
//		if(varymu[i] == 2)
//		    chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.48214*chemz[i]));
//		if(varymu[i] == 3)
//		    chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.391*chemz[i]));
//		real_xcno[i] = xcno;
//		okflag = 1;
//	     }
//	   }
//	 }
//            end of old variation of abundances routine

	 if(select == 3){
	   okflag = 0;
           while(okflag == 0){
	      printf("Enter 0 < X < 1.0 abundance by weight of hydrogen, X: ");
	      scanf("%s",value);   // This entry procedure is
	      d = atof(value);     // more immune to invalid inputs
	      xx = d;      // than simple scanf().
	      chemx[i] = xx;
	      printf("\nEntered X = %f.\n",xx);
	      if(xx > 1.0 || xx < 0.0){
	         printf("Domain error: X must be <= 1.0.\n");
	      }
	      else okflag = 1;
	   }
	   okflag = 0;
           while(okflag == 0){
	      printf("Enter 0.0 < Y < %f abundance by weight of helium, Y: ",
	      1.0 - xx);
	      scanf("%s",value);   // This entry procedure is
	      d = atof(value);     // more immune to invalid inputs
	      yy = d;              // than simple scanf().
	      chemy[i] = yy;
	      printf("Heavy element abundance Z = 1.0 - X - Y = %f.\n",
	      1.0 - xx - yy);
	      chemz[i] = 1.0 - xx - yy;
	      chemmu[i] = 1.0/((2.0*chemx[i]) + (0.75*chemy[i]) + (0.5*chemz[i]));
	      if(1.0 - xx - yy < 0.0 || 1.0 - xx - yy > 1.0){
		 printf("Domain error: must have 0 <= 1.0 - X - Y <= 1.0.\n");
//        Note that compiler problem gets lower bound wrong
//         if naively 1.0-X-Y = 0.0.
	      }
	      else okflag = 1;
	   }
           okflag = 0;
	   while(okflag == 0){  // Now enter CNO abundance.
	       printf("Enter CNO abundance, typically 0.0089087 < Z = %f : ",
	       chemz[i]);
	       scanf("%s",value);   // This entry procedure is
	       d = atof(value);     // more immune to invalid inputs;
	       real_xcno[i] = d;
	       if(real_xcno[i] <= chemz[i])
		  okflag = 1;
	       else
		  printf("CNO abunduance %f > Z = %f, so try again.\n",
		  real_xcno[i],chemz[i]);
           }
	 }
	 printf("\n\nINPUT VARIABLES - model=%d, starname = %s.\n",
	 i,sp[i].starname);
	 printf("Distance = %e LY, visual magnitude = %f\n",
	 lightyears[i], visualmag[i]);
	 if(sp[i].equiv == 1)         // Was spectral type
//                                       deduced from temp?
	    printf("Temp same as for ");  // Spectral type was deduced.

            g = sp[i].spectnumber;
	    if(g == 0.0 || g == 1.0 || g == 2.0 || g == 3.0
	    || g == 4.0 || g == 5.0 || g == 6.0 || g == 7.0
	    || g == 8.0 || g == 9.0 || g == 10.0){
	         gk = g; // subsitute integrer for fractional type.
 		 printf("Spectral type %c%d", sp[i].spectcode, gk);
            }
            else
		 printf("Spectral type %c%2.1f",
		 sp[i].spectcode, sp[i].spectnumber);

	 if(sp[i].classification == 1)
	    printf(" III [giant].\n");
	 if(sp[i].classification == 2)
	    printf(" V [dwarf, regular].\n");
	 if(sp[i].classification == 3)
	    printf(" I [supergiant].\n");
	 if(sp[i].classification == 4)
	    printf(" IV [subgiant].\n");
	 if(sp[i].classification == 5)
	    printf(" ZAMS V [zero-age main sequence.\n");
	 if(sp[i].classification == 6)
	    printf(" VI [subdwarfs].\n");
	 if(sp[i].classification == 7)
	    printf(" VII [white dwarf].\n");
	 if(sp[i].classification == 8)
	    printf(" II [bright giant].\n");
	 printf("Surface temperature = %e degrees Kelvin.\n",
	 sp[i].surftemp);
	 printf("Model=%d chemistry:",i);
         abuncode[i] = select;
	 if(select == 1)
	    printf(" Schwarzschild abundances for population I:\n");
	 if(select == 2)
	    printf(" Green's helium-rich abundances for population I:\n");
	 if(select == 3)
	    printf(" User-specified abundances:\n");
	 if(select == 4)
            printf(" Mu-varied initial abundances:\n");
	 printf("X = %f, Y = %f, Z = %f, mu = %f.\n",
	 chemx[i],chemy[i],chemz[i],chemmu[i]);
	 ilast = i;  // last model number.
	 msmodels = ilast + 1; // total number of
//                               main sequence models to process.
       }
       ++i;
       if( i == 1){
	  printf("\nStellar model entered.\n");
//	  printf("Enter 1 to start processing models 0-24: ");
//	  scanf("%s",value);   // This entry procedure is
//	  d = atof(value);     // more immune to invalid inputs
	  select = 1;          // than simple scanf().
       }

    }
    if(msmodels != 0){       // Any models to process?

	fp = fopen("depthsta.txt","w");
	processmsmodels();  // process the list of main sequence models
//	reportms();   // prepare report file from mem.vars.
	status = fclose(fp);   // close starexp.txt

	printf("ReportFile=depthsta.txt in DEPTHSTA directory. ");
    }
    printf("Enter 1 for new run:");
    scanf("%s",value);   // This
    d = atof(value);     // entry procedure is more
//                          immune to invalid inputs
    select = d;          // than simple scanf().

}
/**************************************************
*
*  processmsmodels - process main sequence models.  <SCREEN>
*                                             Reports to screen.
*   This edition modified to function as a depthscanner.
*
*   msmodels = number of models to process, starting with 0.
*              Computes 20 models for any choice of resolution
*              window around the nominal distance.
*
****************************************************/
void processmsmodels(void)
{
    int j;
    int sustain;
     double r;		// distance to the star in LY.
     double rpc;        // distance to the star in parsecs.
     double m;          // visual magnitude m.
     double absmag;     // absolute magnitude M
//                         with bolometric corrections.
     double absmagnbc;  // absolute magnitude M without
//                          bolometric corrections.
     double lratio;     // L/Lsun luminosity.
     double lrationbc;  // L/Lsun luminosity without
//                         bolometric corrections.
     double expo;       // exponent, L/Lsun = 10**exponent.
     double exponbc;    // exponent, L/Lsun = 10**exponent, w/o BC.
     double mratio;     // M/Msun mass ratio.
     double mrationbc;  // M/Msun mass ratio without
//                         bolometric corrections.
     double mratiolow;  // M/Msun mass ratio.
     double mratiolownbc; // M/Msun mass ratio w/o B.C.
     double mratiobest; // M/Msun for x = 3.409 best fit
//                         to mass-luminosity data.
     double mratiobestnbc;  // M/Msun best fit with x=3.409
//                             w/o bolometric corrections.
     double rratio;     // R/Rsun.
     double rrationbc;  // R/Rsun without bolometric corrections.
     double x = 3.0;    // L/Lsun = (M/Msun)**x.   3 <= x < 4.
     double tsurface;   // suface temperature of star for BC,
//                         bolometric correction.
     double bc;	// M.Schwarzschild bolometric correction, pg. 9. [8].
     double xt;		// xt = log10(Tsurface).
     double rstar;	// stellar radius in meters.
     
     double mstar;      // mass of the star in kg.
     
     double avgdensity; // average density of the star.
     double densitycentral; // central density of the star.
     double pressurecentral; // central pressure of the star.
     double factor;
     double fbeta;       // (1-b)/b**4 = (P**3/ro**4)*[(mu*mh)**4](a/3).
     double mu;          // molecular weight chemmu[j].
 
 
     double beta;       // b.
     double pgas;       // gas pressure.
     double pradiation; // radiation pressure.
     double pratio;     // Pradiation/Pgas
     double tc;        // central temperature
//                        tc=(mu*mh/(k*density_c))*beta*Ptotal_c
     double omega;     // binding energy of n=3
//                        polytropic model star.
     double centradpress; // central radiation
//                           pressure, gamma = 5/3 model.
     double age; // main sequence lifetime,
//                  see THERMONUCLEAR FUSION IN STARS, pg.232.
     double ageschw; // main sequence lifetime,
//                      1/M**2.409 formulae with Schw. sun age.
     double fmsmodels;
     double fj;
     double deltad;
     double distance[20];
     double lstarratio[20];
     double nucstarratio[20];
     double errorsec[20];
     int kk;  // scan core index for H-burning core measures.
     double rr;  // scan core radius for H-burning core measurements.
     int looplim;  // scan core index limit based on polytrope type.
     double masscorekg, masscoresolar, masscoreratio;
     double starage;       // starage in years
     double staroldie[30]; // starage array for depthscan in years.
     double lifetimes[30]; // star lifetime array for depthscan in years.
     double chemmucore; // H-burning core chemistry: molecular weight.
     double tt;        // temp in H-burning core
     double delrr;    // differential step variable in core integration.
     double roin, roout,romid; // core density variables
     double corelifetime;  // maximum H-burning core lifetime, yrs
     double phi;
     int redgiant;     // Red giant detected flag. based on HR diagram patch
//                     0.5 < log10(L/Lsun) < 3.0 && 3.4 < log10(Tsurface) < 3.7
//                         Relativistically degenerate object variables.
     int wdwarfflag; // White dwarf detected -> 1, else -> 0.
//                         H-R diagram bounds for white dwarf stars.
     int reddwarf;   // Oh-Oh: some white dwarfs are red, after all.
//                         Red dwarf detected -> 1, else -> 0.
//          These red white dwarfs are in the last H-R diagram patch below.
     double tub[5] = {4.6,4.2,4.0,3.8,3.4};  // log10(Tsurface)  upper bounds
     double tlb[5] = {4.2,4.0,3.8,3.4,3.0};  // log10(Tsurface) lower bounds
     double ltop[5] = {0.0,-1.5,-2.0,-3.0,-4.0}; // log10(L/Lsun) upper bounds.
     int kwd; // index for H-R diagram patches.
     double tptr; //  log10(tsurface)


    j = 0;
    msmodels = 20;
    deltad  =  lightyears[0]/200.0;
    printf("\nSTELLAR DEPTH SCAN: StarName = %s, Range  = +- %2.2f percent\n", sp[0].starname, percent);
    printf("Nominal distance = %e LY = %f pc = %f AU.\n",
	  lightyears[0], lightyears[0]/3.2600, lightyears[0]/1.581395349e-5 );
    printf("Spectral type = %c%2.1f, visual_magnitude = %f .\n",
	sp[0].spectcode, sp[0].spectnumber, visualmag[0]);
    printf("Each Step = %f LY = %f parsec.  Sec of Error from Nominal d.\n",
	  deltad, deltad/3.2600, 3.26/deltad );

    if(varymu[j] == 3)  // set up chemmu[j] accorinding
//                         if variance scheme declared.
      chemmu[j] = 1.0/(2.0*chemx[j] + (0.75*chemy[j]) + 0.391*chemz[j]);
//                         Z is U_238.
    if(varymu[j] == 2)
      chemmu[j] = 1.0/(2.0*chemx[j] + (0.75*chemy[j]) + 0.48214*chemz[j]);
//                         Z is iron.
    if(varymu[j] == 1)
      chemmu[j] = 1.0/(2.0*chemx[j] + (0.75*chemy[j]) + 0.5*chemz[j]);
//                       Z is light-hvy.

    fprintf(fp,"DEPTHSTA Stellar Distance Scanner\n");
    fprintf(fp,"Starname=%s, models=%d, Range: +-%2.2f percent.\n\n",
       sp[0].starname, msmodels, percent);
    fprintf(fp,"Lnuclear/Lobserved = 1 gives the ZAMS distance,\n");
    fprintf(fp,"Lnuclear/Lobserved > 1 gives the distance corresponding to\n");
    fprintf(fp,"greater stellar ages when the star age < Main Sequence lifetime.\n");
    fmsmodels = msmodels;
    j = 0;
    while(j < msmodels){
	fj =  j;
	
	x = 3.0;
	fprintf(fp,"\nDEPTHSTA Model=%d: Starname=%s. ",
	  j, sp[0].starname);
	if(starform[0] == 0)
	   fprintf(fp,"Polytrope: n=3.\n");
	if(starform[0] == 1)
	   fprintf(fp,"Polytrope: n=1.5.\n");
	if(starform[0] == 2)
	   fprintf(fp,"Polytrope: n=3.5.\n");
	if(starform[0] == 3)
	   fprintf(fp,"Polytrope: n=4.0.\n");
	fprintf(fp,"Spectral type = %c%2.1f. ",
	sp[0].spectcode, sp[0].spectnumber);
	if(sp[0].spectcode == 'V')
	  fprintf(fp,"Sun. ");
	fprintf(fp,"Surface temp = %e deg Kelvin.\n",
	sp[0].surftemp);
	r = lightyears[0]; // get distance in light-years.
	r = (r - (fj*deltad*scanscale)) + ((msmodels*deltad*scanscale)/2.0);
	errorsec[j] =  (3.26/lightyears[0]) - (3.26/r);
	distance[j] = r;
	m = visualmag[0]; // get visual magnitude m.
	tsurface = surfacetemp[0];
	xt = log10(tsurface);  // find the look-up value for BCstellar.
	bc = bolometric(xt);  // interpolate B.C.
//                               from a table as a function of xt.
//
//  Derivations start from
//     (L1/L2)(r2/r1)**2 = I1/I2 = 10**[(2/5)(m2 - m1)],
//
//  so that  m2 = m1 + (5/2)*log10(L1/L2) + 5*log10(r2/r1).
//
//  Uncorrected, L/Lsun = 10**((2/5)*(4.83 - M_v),
//      where 4.83 is the uncorrected
//      absolute visual magnitude of the Sun.
//  M_bol = M_v + B.C.  After Zombeck,[9], page 25.
//                     HANDBOOK OF SPACE ASTRONOMY & ASTROPHYSICS.
//  Corrected,  L/Lsun = 10**((2/5)*(4.83 - M_bol),
//                 where 4.83 is the uncorrected
//                 absolute magnitude of the Sun.
//  For the Sun, M_v = +4.83 (visual magnitude at 10 pc)
//               m_v = -26.74 (visual magnitude at 1 AU)
//               M_bol = +4.75 = M_v + B.C.  absolute mag
//                                          at 10 pc, corrected for
//                                           atmospheric absorption.
//               m_bol = -26.82  visual magnitude of the sun above
//                               the atmosphere at 1 AU.
//
//  Note that our ideal type G2-star is slightly hotter
//             at 5780 deg K. than the sun,
//    which has a surface temp of 5770 deg.K.
//
//  Lsun_corrected/Lsun_uncorrected = 10**[(-2/5)*B.C.]
//                                  = exp[-0.921034*B.C.]
//                                  = exp[-2.3025851*(2/5)*B.C.].
//
//  (L/Lsun)_both_corrected
//        = 10**[(2/5)(Mvsun + B.C.sun - (Mv + B.C.star))]
//        = 10**[(2/5)(M_bol_sun - M_bol)].
//
// We must use L/Lsun_corrected, whether or not L
//     is corrected or anomolies result.
//
//   L/Lsun = 10**((2/5)(M_bol - M_v)) x 10**(-2/5)(B.C.),
//            second term prepares after B.C.
//
// Note L/Lsun
// = [(r/r_sun)**2]*[10**(2/5)(m_sun_bol - m_v)] x 10**(-2/5)B.C.,
//
// where the 2nd term gives the effect of introducing
// the bolometric correction to m_v.
//
	rpc = r/3.2600;  // compute distance in parsecs.
	absmag = m + 5.0 - (5.0*log10(rpc));   // cal absolute
//                                                magnitude M.
	absmagnbc = absmag;       // save pre-corrected w/o BC.
        rawabsmag[0] = absmag;   // save to report variable.
	absmag = absmag + bc;    // include stellar bolometric BC.
	bcabsmag[0] = absmag;    // save to report variable.
        bolo[0] = bc;            // save B.C.
	expo = (4.83 - 0.068824 - absmag)/2.5; // Must use program's
//                                      4.83 + B.C.= 4.83 - 0.068072
//         m_bol_sun     or use 0.068824 for Sun at exact temp 5770.0
//         4.761176      for calibration, because our G2 is not
//                       precisely the same as the Sun, which is
//                      just a little cooler.  We must have the
//                      ratio for both corrected for astro research.
//                     Based on Zombeck's table of corrections,
//  m_bol_sun=4.761176        M_bol_sun = 4.83 - 0.068824 = 4.761176
	exponbc = (4.83 - 0.068824 - absmagnbc)/2.5;
//                                     exponent without bolometric corr.
	lratio = exp(2.302585*expo); // e**Ax = 10**x, A = 2.302585
        bclratio[0] = lratio;   // save to report variable.
	lrationbc = exp(2.302585*exponbc); // same for no B.C.
        rawlratio[0] = lrationbc;
	fprintf(fp,"L/Lsun = %e, with bolometric correction BC = %f\n",
	lratio,bc);
	fprintf(fp,"L/Lsun = %e, without bolometric correction.\n",
	lrationbc);
	if(lratio > 1.0e7){
	   printf("!Error!\nL/Lsun = %e >> Max = 10**6 for stable physical stars.\n",
	   lratio);
	   printf("Probable cause: parameter entry invalid.\n");
	   printf("  Distance = %e LY\n",r);
	   printf("  Visual magnitude = %f \n",m);
	   printf("  Spectral type = %c%2.1f. \n",sp[j].spectcode, sp[j].spectnumber);
	   if(lratio > 3.0e9 && lratio < 5.0e10){
	      printf("The object is in the supernova luminosity range.\n");
	   }
	   else if(lratio > 5.0e5 &&  lratio < 3.0e9){
	      printf("The object is brighter than a nova, dimmer than a supernova.\n");
	   }
	   else if(lratio > 2.0e4 && lratio < 5.0e5){
	      printf("The object is in the  classical nova luminosity range.\n");
	   }	   
	   else if(lratio > 5.0e10){
	      printf("The object is brighter than a typical supernova.\n");
	   }
	   else if(lratio < 2.0e4){
	      printf("The object is brighter than a star, dimmer than a classical nova.\n");
	   }
	   printf("May cause numeric exception! -> EXIT DEPTHSTA.\n");
           exit(0);
	}
//
// L/Lsun = (M/Msun)**x, where x = about 3, so M/Msun = (L/Lsun)**(1/x).
// Then log10(M/Msun) = (1/x)log10[L/Lsun],
// so M/Msun = 10**[(1/x)*log10[L/Lsun] = exp[(2.302585/x)*log10[L/Lsun].
//
	mratio = exp((2.302585/x)*log10(lratio));
        mrationbc = exp((2.302585/x)*log10(lrationbc));
	x = 4.0;
	mratiolow =  exp((2.302585/x)*log10(lratio));
	mratiolownbc = exp((2.302585/x)*log10(lrationbc));
//                                            again w/o B.C.
	fprintf(fp,"%f < M/Msun < %f,  3 < x < 4., with B.C.\n",
	mratiolow, mratio);
	fprintf(fp,"%f < M/Msun < %f,  3 < x < 4., without B.C.\n",
	mratiolownbc, mrationbc);
	x = 3.409; // empirical best guess from chart measurements.
	mratiobest =  exp((2.302585/x)*log10(lratio));
        bcmratiobest[0] = mratiobest;
	mratiobestnbc =  exp((2.302585/x)*log10(lrationbc));
	rawmratiobest[0] = mratiobestnbc;

	fprintf(fp,"M/Msun = %f , x = %f  best mass-lum fit with B.C..\n",
	mratiobest, x);
	fprintf(fp,"M/Msun = %f , x = %f  best mass-lum fit without B.C..\n",
	mratiobestnbc, x);
//
//  To do nuclear luminosity boundaries, re-run
//   nuclearluminosity with mratiolow & mratio.
//              to pick the central density and mass.
//
// Mass-Luminosity Chart:
//   See THERMONUCLEAR FUSION IN STARS, by James A.Green,
//   Figure 1.25, page 34.

	rratio = (5770.0/tsurface)*(5770.0/tsurface)*sqrt(lratio);
	bcrratio[0] = rratio;
	fprintf(fp,"R/Rsun = %e with bolometric corrections.\n",
	rratio);

//
// A white dwarf detector. Also detects red dwarfs.
//    White dwarfs include both white dwarfs and red white dwarfs (Green).
//    For a real White Dwarf, R/Rsun < 1/50.118, from Chandra table.
//    This code uses a fuzzy radius check & fuzzy mass criterion in
//    addition to good and reasonable H-R diagram coverage checks.
//    Note that the actual Mstar/Msun ratio is unknown in this case.
//
        wdwarfflag = 0;
        if(rratio < 0.03 && mratiobest >= 0.22)  // Roughly, from Chandrasekhar
	    wdwarfflag = 1;          // mass-radius table. Fuzzy check
	    
        kwd = 0;         // check 4 vertical patches in H-R diagram.
        while(kwd < 4){  //  Log(L/L_sun) vs. Log(T_surface) H-R diagram testing
          tptr = log10(tsurface);
	  if(tptr <= tub[kwd] && tptr >= tlb[kwd] && log10(lratio) <= ltop[kwd]){
	     wdwarfflag = 1;
	  }
	  kwd++;
	}

	reddwarf = 0;  // A red dwarf detector. Red dwarfs are red white dwarfs.
	if(tptr <= tub[4] && tptr >= tlb[4] && log10(lratio) <= ltop[4]){
	     reddwarf = 1;
	}
// End, relativistically degenerate object detector - white & red dwarfs

//
//  Red giant detector.     0.5 < log10(L/Lsun) < 3.0
//                       AND 3.4 < log10(Tsurface) < 3.7  is H-R diagram criterion.
//
	     redgiant = 0;
	     tptr = log10(tsurface);
	     if(log10(lratio) < 3.0 && log10(lratio) > 0.5 && tptr < 3.7 && tptr > 3.4)
		redgiant = 1;
// End, red giant detector.

	rrationbc = (5770.0/tsurface)*(5770.0/tsurface)*sqrt(lrationbc);
        rawrratio[0] = rrationbc;
	fprintf(fp,"R/Rsun = %e without bolometric corrections.\n",
	rrationbc);

	fprintf(fp,"Visual magnitude   m = %f,", m);
	age = 9.92e9*(1.0/mratiobest)*(1.0/mratiobest);
//                                              t0(Msun/M)**2 lifetime.
//
//  t_ms = delta_X*M*Ecc/L,
//  t_sun = delta_X*M*Ecc/Lsun,
//          with delta_X = 0.13 (Schwarzschild)
//  Therefore,
//   t_ms = t_sun(M/Msun)(Lsun/L) = t_sun(Msun/M)**2.409.
//                                = t_sun*exp(2.409*log(Msun/M)).
//
	ageschw = 12.88e9*exp( 2.409*log( 1.0/mratiobest ));
//                      tsun_Schw*(Msun/M)**2.409 lifetime.
	fprintf(fp,"  Schw. Lifetime = %e years.\n",ageschw);

	fprintf(fp,"Absolute magnitude with bolometric cor. M + B.C. = %f\n",
	absmag);
	fprintf(fp,"Absolute magnitude without bolometric cor. M = %f\n",
	absmagnbc);
	fprintf(fp,"r = %e light-years = %e parsecs.\n",
	r, rpc);

//
// Compute the average and the central density for n=3 polytrope.
//    
	     rstar = rratio*rsun;
	     xrstar[0] = rstar;    // save in report
//                                    radius of star in meters.
	     mstar = mratiobest*msun;
	     xmstar[0] = mstar;   // save in report mass of star in kg.

	     fprintf(fp,"Radius = %e meters. ",rstar);
	     fprintf(fp,"Mass = %e kg.\n", mstar);      
     
	     avgdensity = mstar/((4.0/3.0)*3.14159*(rstar*rstar*rstar));
	     xavgdensity[0] = avgdensity;
	     
             if(starform[0] == 0){   // gamma=4/3 central density
	         densitycentral = xavgdensity[0]*54.1825;
		 xdensitycentral[0] = densitycentral;		 
	     }
	     else if(starform[0] == 1){
		 densitycentral = avgdensity*5.99071;
//                                     see pg.84, THERMONUCLEAR
		 xdensitycentral[0] = densitycentral;
//                                      FUSION IN STARS, by J.A.Green
	     }
	     else if(starform[0] == 2){
	         densitycentral = avgdensity*152.884; // see p.427, Chiu,
		 xdensitycentral[0] = densitycentral; // STELLAR PHYSICS.
	     }
	     else if(starform[0] == 3){
		 densitycentral = avgdensity*622.408; // see p.427, Chiu,
		 xdensitycentral[0] = densitycentral; // STELLAR PHYSICS.
             }
//
// Compute the central total pressure for polytrope of index n
//             = (GM**2/R**4)*[(n+1)4pi(dtheta/dx)**2)]**-1
//
	     pressurecentral =   mstar*mstar*6.67e-11/(rstar*rstar*rstar*rstar);
             if(starform[0] == 0){       // gamma=4/3, n=3 model
		 factor = 4.0*3.14159*4.0*(0.04231)*(0.04231);
	     }
	     else if(starform[0] == 1){ // gamma=5/3, n=1.5 model
		 factor = 2.5*3.14159*4.0*(2.71406/(3.65375*3.65375))*(2.71406/(3.65375*3.6375));
	     }
	     else if(starform[0] == 2){ // gamma=9/7 n=3.5 model
		 factor = 4.5*3.14159*4.0*(1.89056/(9.53581*9.53581))*(1.89056/(9.53581*9.53581));
	     }
	     else if(starform[0] == 3){ //  gamma=5/4, n=4.0 model
		 factor = 5.0*3.14159*4.0*(1.79723/(14.97155*14.97155))*(1.79723/(14.97155*14.97155));
	     }

	     factor = 1.0/factor;
	     pressurecentral = pressurecentral*factor;
             xpressurecentral[0] = pressurecentral;

	     fprintf(fp,"Average density=%e kg/m**3. ",
	     avgdensity);
	     fprintf(fp,"Central density=%e kg/m**3.\n",
	     densitycentral);
	     fprintf(fp,"Central total pressure = %e Newtons/m**2.\n",
	     pressurecentral);
//
// For gamma = 4/3 models,
// find (1 - beta)/beta**4 = ( (Pc**3)/ro_c**4 )*[(mu*mh/k)**4]*(a/3).
//
	     mu = chemmu[0];
	     fbeta = pressurecentral*pressurecentral*pressurecentral;
	     fbeta = fbeta/(densitycentral*densitycentral*densitycentral*densitycentral);
	     fbeta = fbeta*(mu*mh/k)*(mu*mh/k)*(mu*mh/k)*(mu*mh/k);
	     fbeta = (a/3.0)*fbeta;
	     xfbeta[0] = fbeta; // report model variable.
             if(starform[0] == 0)
		fprintf(fp,"[1 - beta]/beta**4 = %e. ", fbeta);

	     if( mratiobest > 2.0){
		fprintf(fp,"Carbon Cycle ON: M > 2*M_sun\n");  // Test for carbon cycle on.
	     }
	     else{
		fprintf(fp,"\n");
	     }

	     beta = findbeta(fbeta);
             if(starform[0] == 0){
		  xbeta[0] = beta;
	     }
	     else if(starform[0] == 1 || starform[0] == 2 || starform[0] == 3){
		  xbeta[0] = 1.0;
                  beta = 1.0;       // radiation pressure neglected.
	     }
             if(starform[0] == 0) // if gamma=4/3
		  fprintf(fp,"Beta = %e. Pgas = beta*Ptotal, Pradiation = (1-beta)*Ptotal.\n",
		   beta);
			  
	     pgas = beta*pressurecentral;
             xpgas[0] = pgas;
	     pradiation = (1.0 - beta)*pressurecentral;
             xpradiation[0] = pradiation;
	     pratio = pradiation/pgas;
             xpratio[0] = pratio;

             if(starform[0] == 0){
	         fprintf(fp,"Pradiation/Ptotal = %e. ", 1.0 - beta);
	         fprintf(fp,"Pgas/Ptotal = %e.\n", beta);
	         fprintf(fp,"Pradiation/Pgas = %e. ", pratio );
             }
             if(starform[0] == 0){      // gamma = 4/3, n = 3 model
		  tc = (mu*mh/(k*densitycentral))*beta*pressurecentral;
             }
	     else if(starform[0] == 1 || starform[0] == 2 || starform[0] == 3){  
                  tc = pressurecentral*mu*mh/(k*densitycentral);
             }
             xtc[0] = tc;
	     fprintf(fp,"Tcentral = %e deg Kelvin.\n",tc);
	     if(starform[0] == 1 || starform[0] == 2 || starform[0] == 3){
                centradpress = (1.0/3.0)*a*tc*tc*tc*tc;
		fprintf(fp,"Central Radiation pressure = %e Newtons/m**2.\n",
		centradpress);
	     }

             if(starform[0] == 0)
		  omega = (3.0/(5.0 - 3.0))*(6.67e-11)*mstar*mstar/(rstar);
	     else if(starform[0] == 1)
		  omega = (3.0/(5.0 - 1.5))*(6.67e-11)*mstar*mstar/(rstar);
	     else if(starform[0] == 2)
		  omega = (3.0/(5.0 - 3.5))*(6.67e-11)*mstar*mstar/(rstar);
	     else if(starform[0] == 3)
		  omega = (3.0/(5.0 - 4.0))*(6.67e-11)*mstar*mstar/(rstar);
	     xomega[0] = omega;
	     fprintf(fp,"Grav binding energy = %e J. ",omega);
	     if(bcmratiobest[0] > 6.23)
		 fprintf(fp,"Pre-Supernova M/Msun > 6.23.\n");
	     else
		 fprintf(fp,"Not Pre-Supernova: M/Msun < 6.23.\n");
//
//    Pre-renormalize the normalization constants so stellar age calc can proceed
//       for any choice of the initial abundances of the elements.
//
	     nuclearluminosity(0); // Pre-compute normalizeclayton
//				      for our specific present abundances X(i) = X,Y,...	     
	     normalizeclayton = scclaytonluminosity[0]/lratio;
//                                    Screened p-p fusion clayton to pre-renormalize
//
	     
             nuclearluminosity(0);  // Compute the nuclear
//               			       luminosity of the star,
     
	     fprintf(fp,"P-P chain Lnuclear/Lstar=%e. ",scclaytonluminosity[0]/lratio);
	     fprintf(fp,"Clayton screened & unnormalized.\n");

	     fprintf(fp,"P-P + CNO + He-burning Lnuclear/Lstar = %e. \n", totalluminosity/lratio);
	     nucstarratio[j] = totalluminosity/lratio;

	     fprintf(fp," %d) %e LY, ", j, distance[j]);
	     fprintf(fp,"Lnuc/Lstar = %e,", nucstarratio[j]);
	     if( nucstarratio[j]< 1.0){
	        fprintf(fp,"NegAge or Contract to M.S.\n");
		fprintf(fp,"      %f percent difference from nominal %e LY.\n",
		   ((distance[j] - lightyears[0])/lightyears[0])*100.0, lightyears[0]);
             }
             else{
		fprintf(fp," PosAge, %f sec error.\n", errorsec[j]);
	        fprintf(fp,"      %f percent error from nominal %e LY.\n",
		   ((distance[j] - lightyears[0])/lightyears[0])*100.0, lightyears[0]);
	     }
	     if(wdwarfflag == 1 && reddwarf == 0)
	       fprintf(fp,"  White dwarf detected.\n");
	     if(wdwarfflag == 1 && reddwarf == 1)
	       fprintf(fp,"  Red White dwarf detected! - Red Dwarf.\n");
	     if(redgiant == 1)
	       fprintf(fp,"  Red giant detected.\n");

	     printf("%d) ", j);
	     printf(" %e LY, ", r);
	     printf("Lnuc/Lstar = %f, ", totalluminosity/lratio);
	     if(totalluminosity/lratio < 1.0){
	       printf("NegAge|Contract->M.S. ");
	     }
	     else{
	       printf("PosAge, %f sec error. ", errorsec[j]);
	     }
	     if(wdwarfflag == 1 && reddwarf == 0 )
	       printf("White Dwarf detected.\n");
	     else
	       printf("\n");
	     if(wdwarfflag == 1 && reddwarf == 1 )
	       printf("Red White Dwarf!\n");
	     if(redgiant == 1)
	       printf("Red giant detected.\n");


//  H-burning core measurements and Stellar Age calculation

            kk=0;
	    rhcore  = 0.0;
	    looplim = looplimset2(0);	    
	    while(kk < looplim){
               if(starform[0] == 0){
	          tt = xtc[0]*theta[kk];
		  rr = (xrstar[0]/rsun)*(xr[kk]/6.9011);
	       }
	       else if(starform[0] == 1){
		  tt = xtc[0]*xtheta[kk];
		  rr = (xrstar[0]/rsun)*(xxr[kk]/3.65375);
	       }
	       else if(starform[0] == 2){
		  tt = xtc[0]*yxtheta[kk];
		  rr = (xrstar[0]/rsun)*(yxr[kk]/9.53581);
	       }
	       else if(starform[0] == 3){
		  tt = xtc[0]*zxtheta[kk];
		  rr = (xrstar[0]/rsun)*(zxr[kk]/14.97155);
	       }	       
	       if(tt > 1.0e7){		  
		  rhcore = rr;   // Finds hydrogen-burning core maximum radius r/R_sun.
	       }      
       
               kk++;
	    }

//    Find H-burning core density at edge, and compute the coremass

	    kk = 1;
	    looplim = looplimset2(0);
	    if(starform[0] == 0){   //  n = 3
	      delrr = (xrstar[0]/rsun)*(xr[1]/6.9011);
	      ro = xdensitycentral[0];
	      masscorekg =  (4.0/3.0)*3.24159*(delrr*rsun)*(delrr*rsun)*(delrr*rsun)*ro;
	    }
	    if(starform[0] == 1){  //  n = 1.5.
	      delrr = (xrstar[0]/rsun)*(xr[1]/3.63375);
	      ro = xdensitycentral[0];
	      masscorekg =  (4.0/3.0)*3.24159*(delrr*rsun)*(delrr*rsun)*(delrr*rsun)*ro;
	    }
	    while(kk < looplim - 1){
	      if(starform[0] == 0){ // gamma = 4/3 model
		  roin = xdensitycentral[0]*theta[kk]*theta[kk]*theta[kk];
		  roout = xdensitycentral[0]*theta[kk+1]*theta[kk+1]*theta[kk+1];
		  romid = (roin + roout)/2.0;
		  tt = xtc[0]*theta[kk];
		  delrr = (xrstar[0]/rsun)*(xr[kk+1]/6.9011) - (xrstar[0]/rsun)*(xr[kk]/6.9011);
		  rr = (xrstar[0]/rsun)*(xr[kk]/6.9011);
		  rr = (rr + (xrstar[0]/rsun)*(xr[kk+1]/6.9011))/2.0;
		  if(tt > 1.0e7){
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		     masscorekg = masscorekg + 4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid;
		  }
	      }
	      else if(starform[0] == 1){
		  phi = sqrt(xtheta[kk]);
		  tt = xtc[0]*xtheta[kk];
		  roin = xdensitycentral[0]*phi*phi*phi;
		  phi = sqrt(xtheta[kk+1]);
		  roout = xdensitycentral[0]*phi*phi*phi;  //  n = 1.5.
		  romid = (roin + roout)/2.0;
		  delrr = (xrstar[0]/rsun)*(xr[kk+1]/3.63375) - (xrstar[0]/rsun)*(xr[kk]/3.63375);
		  rr = (xrstar[0]/rsun)*(xr[kk]/3.63375);
		  rr = (rr + (xrstar[0]/rsun)*(xr[kk+1]/3.63375))/2.0;
		  if(t > 1.0e7){
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		     masscorekg = masscorekg + 4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid;
		  }	 
	      }
	      kk++;
	    }

//    Find X in H-burning core such that Lnuc/Lstar = 1 when p-p fusion & CNO are both
//    accounted for.  Assume helium fusion is negligible in the Main Sequence stars.
//    Note that then Lnuc/Lstar = totalluminosity/lratio.

	    claynormx = chemx[0]/sqrt((totalluminosity/lratio));

	    if(totalluminosity/bclratio[0] <= 1.0)  // Watch out for Lnuc < Lstar,
	       delh = 0.0;                         // when core abundances remain constant
	    else                                  //  and the star is unborn.
	       delh = chemx[0] - claynormx;
	    
	    starage  = (1.0/(4.0*mh))*delh*masscorekg;
	    starage = starage*(4.19737e-12)/(lsun*bclratio[0]);  // 4.19737e-12 J is 26.2 MeV.
	    starage = starage/3.156e7;  // Find years age from seconds.

	    if( nucstarratio[j] < 1.0){
	      fprintf(fp,"    Clayton Star age < 0, invalid or contracting to Main Sequence.\n",starage);
	    }
            else{
	      fprintf(fp,"    Clayton Star age = %e yr based on the H-burning core\n",starage);
	      fprintf(fp,"    depletion required to normalize L(R) for Clayton p-p fusion.\n");
	    }

	    if(totalhluminosity < bclratio[0])
	       fprintf(fp,"    L_nuc < L_observed, Age invalid or contracting to Main Sequence.\n");

	    fprintf(fp,"    10**7 K H-burning core radius = %1.2f solar radii", rhcore);
	    fprintf(fp,"   = %1.2f star radius.\n", rhcore*rsun/xrstar[0]);
	    fprintf(fp,"    Mantle: X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0],chemy[0],chemz[0],chemmu[0]);
	    chemmucore = 1.0/(2.0*(chemx[0] - delh) + (0.75*(chemy[0] + delh)) + 0.5*chemz[0]);
	    fprintf(fp,"    Core:   X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0] - delh, chemy[0] + delh, chemz[0], chemmucore);
//
//  DEBUG probe
//
//	    printf("DEBUG: rhcore = %f, solar radii.\n", rhcore);
//	    printf("       rhcore*rsun/xrstar[0] = %f, .\n", (rhcore*rsun)/xrstar[0]);
//	    printf("       xdensitycentral[0] = %e kg per m**3\n", xdensitycentral[0]);
//	    printf("       rocoremin = %e kg per m**3\n", rocoremin);
//	    printf("       rocoreavg = %e kg per m**3\n", rocoreavg);
//	    probe(0);
//
//  END DEBUG probe
//
	    masscoresolar = masscorekg/msun;   // mass of the H-burning core, solar units.
	    masscoreratio = masscorekg/xmstar[0];   // M_core/M_star .
	    fprintf(fp,"    H-burningCoreMass = %e kg , = %f Msun , Mcore/Mstar = %f.\n",
	    masscorekg, masscoresolar, masscoreratio);
	    staroldie[j] = starage;
	    
	    delh = chemx[0];  //   Compute H-burning core lifetime
	    corelifetime  = (1.0/(4.0*mh))*delh*masscorekg;
	    corelifetime = corelifetime*(4.19737e-12)/(lsun*bclratio[0]);  // 4.19737e-12 J is 26.2 MeV.
	    corelifetime = corelifetime/3.156e7;  // Find years age from seconds.
	    lifetimes[j] = corelifetime;
	    fprintf(fp,"    Max H-burning Core Lifetime = %e yr.\n",corelifetime);

// end of calc with bolometric corrections.
	j++; // on to next model.
    }
    fprintf(fp,"\n DEPTH SCAN SUMMARY\n\n");
    j = 0;
    while(j < msmodels){
       fprintf(fp," %d) %e LY ", j, distance[j]);
       fprintf(fp,"Lnuc/Lstar = %e ", nucstarratio[j]);
       if( nucstarratio[j]< 1.0){
	  fprintf(fp,"NegAge or Contraction to Main Seq.\n");
	  fprintf(fp,"      %f percent difference from nominal %e LY.\n",
	     ((distance[j] - lightyears[0])/lightyears[0])*100.0, lightyears[0]);
       }
       else{
	  fprintf(fp," PosAge, %f sec error.\n", errorsec[j]);
	  fprintf(fp,"      %f percent error from nominal %e LY.\n",
	     ((distance[j] - lightyears[0])/lightyears[0])*100.0, lightyears[0]);
	  fprintf(fp,"      Star Age = %e years,\n", staroldie[j]);
	  fprintf(fp,"      Max H-burning Core Lifetime = %e yr,", lifetimes[j]);
	  if(lifetimes[j] < staroldie[j]){
	      fprintf(fp," Invalid.\n");
	  }
	  else{
	      fprintf(fp," Possible.\n");
	  }
       }
       j++;
    }

}
/***********************************************************
*
*   nuclearluminosity() - Compute the nuclear luminosity for p-p fusion
*                         using formulae from various astrophysicists,
*                         for comparison purposes.
*                         Also computes CNO carbon cycle, Helium burning,
*                         the total hydrogen-burning luminosity, and
*                         the total nuclear luminosity.
*                <NUC>
*           
*   out: Nuclear luminosities in L_sun units.
*                                 
*        scclaytonluminosity[0] - Screened Clayton p-p fusion luminosity works for the Sun!
*                         If normalization goes by X(H) reduction, we find the solar age!
*        claytonluminosity[0] - Non-normalized, un-electrostatically screened Clayton p-p fusion.
*        nscclaytonluminosity[0] - Normalized, screened Clayton p-p fusion. 
*        cnoluminosity[0] - Carbon cyle luminosity ala Martin Schw.
*        totalhluminosity[0] - Total Hydrogen-burning luminosity.
*        heliumluminosity[0] - Helium-burning luminosity, triple-alpha process.
*        totalluminosity[0] - Total nuclear luminosity of the star,
*                             P-P fusion chain + CNO + Helium-burning.
*     
*
*                            Later we discover that the normalizing factor can be
*                            set to unity by reducing the abundance by weight of
*                            hydrogen X(H) in the star core until L_nuclear is L(R).
*                            If we use Clayton p-p fusion, then the age of the Sun
*                            comes out if we use the mass of hydrogen consumed to
*                            compute the energy radiated and divide by L(R).
*            ect.
*     Search key:            
*                 F) - Carbon cycle energy production,
*                      after M.Schwarzschild.
*                 H) - Triple-alpha process, after M.Schwarzschild.              
*               
*                 K) - Donald D. Clayton p-p fusion chain formulae.
*
*   The acid test - Do nuclear luminosites match observed luminosities?
*   These calculations assume the stars have uniformly mixed composition,
*   and that they obey the equation of state of a perfect gas throughout.
*   The standard model n=3, gamma=4/3 polytropes of Eddington are used
*   for the density and temperature structure.  Nuclear energy generation
*   formulae are integrated from the center to the surface of the star
*   to obtain the theoretical nuclear luminosity.   Then the Chiu p-p fusion
*   luminosity is normalized for the Sun using 1.310466, to get our best guess
*   of what p-p fusion amounts to in other stars. For comparison of p-p fusion
*   formulae, see STAREXP.  STARZ does not compare p-p fusion formulae
*   from different astrophysicists, which is the task of STAREXP.
*
*******************************************************************/
void nuclearluminosity(int j)
{
	double epsilon;  // energy generation in ergs/gram-sec.
        double x;       //  X = chemx[j], hydrogen fraction by weight.
	double epsilonmks; // energy generation in Joules/sec per Kg.
	double epsilonmks0;  // energy generation at inner shell edge r0.
	double epsilonmks1;  // energy generation at outter shell edge r1.

	double scepsilonmks0; // screened energy generation at inner shell edge r0.
	double scepsilonmks1; // screened energy generation at inner shell edge r1.
	double sclcentral0; // screened energy generation in central ball.
	double sctluminosity; // total screened energy generation in star.
	double zeta;      // clayton weak screening factor sub-term.
	double zetasum, zetatrailer;  // clayton weak screening factor sub-terms.
	double zetasun = 1.855713; // clayton zeta for sun, see zeta.cpp.
	double scrn0,scrn1; // screening factors at the shell edges, weak screening.

	double m = 1.6725e-27;   // mass of the proton in Kg.	
	double e = 4.802919e-10;  // charge of the proton in esu.       
	
	double ke, ne, ni, zbar;      // heat conduction variables.        
	double dv;     // volume of shell.
	double tluminosity; // total screened luminosity
//                             so far in integral.
	double unscreentot; // total unscreened luminosity
//                             so far in integral.
	double dluminosity; // differential luminosity for this shell.
	double dluminosity0; // inner shell edge quantity.
	double dluminosity1; // outter shell edge quantity.
	double lcentral0; // luminosity in Joules/sec
//                           of central shell sphere.
	double tau;        // rate factor.
	double tau0;       // tau at shell inner edge.
	double tau1;       // tau at shell outter edge.
	double q = 2.099359e-12; // Joules per proton-pair
//                               for pp fusion chain, 13.1 MeV/pair.
	double t9;  // temp in degrees kelvin/10**9.
	double troot; // t9**(1/3).
	double eg; // E_G = E_gamow
//                    = [2mc**2][(pi*ec**2)/(h_bar*c/(4pi*ep0))]
	double en;    // E = (3/2)kT.
	double v;     //  sqrt(3kT/m);
	double p;    // pressure at r	
	double phi;
	double root10;  // t**0.1
	double tc199;   //  t**19.9.
	double vo0, vo1;  // [(10**6)/T]**(1/3) in the
//                        Schwarzschild [8] formula for pp-fusion.

	int i;	// shell counter for integration.
	double y;  // floating-point i.
	double cy; // chemy[i] helium abundance.
	double f3a; // the triple-alpha screening
//                     factor, perhaps near 1.0.
	double screen; // the pp-fusion electrostatic
//                        screening factor after Chiu, pg.385.
	double zzv0,zzv1;  // fusion factors.
	double kappa, difft;  // factors in radiative
//                               equilibrium calculations.
	
	int looplim;  // loop limit.

//
//  F) The carbon cycle -  See THERMONUCLEAR FUSION IN STARS, pg.191-192.
// epsilon = (8.24e-24)*density*X*(X_CNO)*(T_6)**19.9  ergs/gram-sec
// when the density is given in grams/cm**3. If the density is given
// in kg/m**3,  epsilon* = (8.24e-27)*density*X*X_CNO*(T_6)**19.9
// ergs/gr-sec.
// so epsilonmks = (8.24e-31)*density*X*X_CNO*(T_6)**19.9
//  Joules/Kg per second.
//
       limset0();     // set central density, temperature limits.
	
	x = chemx[j];  // set hydrogen fraction by weight	
        tc = tc/1.0e6;
	root10 = nthroot(tc,10);
        tc199 = nthpower(root10,199);
	epsilonmks0 = (8.24e-31)*x*real_xcno[j]*ro*tc199;
        tc1 = tc1/1.0e6;
	root10 = nthroot(tc1,10);
        tc199 = nthpower(root10,199);
	epsilonmks1 = (8.24e-31)*x*real_xcno[j]*ro1*tc199;

	limset2(j);  // find r for exterior of central ball.
        lcentral0 = ((epsilonmks0 + epsilonmks1)/2.0)*((ro + ro1)/2.0)*(4.0/3.0)*pi*r*r*r; 
	i = 1;
        tluminosity = lcentral0;  // set total luminosity 
	looplim = looplimset();  // set the loop limit according to number table entries.
	while( i < looplim){
	    limset1(j,i);  // set shell density, temp limits ro,ro1,t,t1.
            t = t/1.0e6;
	    root10 = nthroot(t,10);
            tc199 = nthpower(root10,199);
	    epsilonmks0 = (8.24e-31)*x*real_xcno[j]*ro*tc199;
	    t1 = t1/1.0e6;
	    root10 = nthroot(t1,10);
            tc199 = nthpower(root10,199);
	    epsilonmks1 = (8.24e-31)*x*real_xcno[j]*ro1*tc199;	    
	    limsetr(j,i);  // set shell r limits r0,r1.
	    tluminosity = lumshell(r0,r1,epsilonmks0,epsilonmks1,ro,ro1,tluminosity);    
	    i++;
	}
	cnoluminosity[j] = tluminosity/lsun;  // CNO luminosity L/Lsun.


//
//  H) Compute the Helium-burning triple-alpha process in the star.
//     See THERMONUCLEAR FUSION IN STARS, PG. 192.
//     In MKS units,
// epsilonmks
//    = 50.8(Y**3)(density**2)[(1/T_8)**3]*f3a*exp[-44.02/T_8]
//      Joules/Kg-sec.
//
	limset0();     // set central density, temperature limits.

	cy = chemy[j];  // set helium fraction by weight
        f3a = 1.0;      // Guess triple-alpha screening factor = 1.
        tc = tc/1.0e8;
	epsilonmks0 = (50.8)*cy*cy*cy*ro*ro*(1.0/tc)*(1.0/tc)*(1.0/tc)*f3a*exp(-44.02/tc);
        tc1 = tc1/1.0e8;
	epsilonmks1 = (50.8)*cy*cy*cy*ro1*ro1*(1.0/tc1)*(1.0/tc1)*(1.0/tc1)*f3a*exp(-44.02/tc1);

	limset2(j);  // find r for exterior of central ball.
        lcentral0 = ((epsilonmks0 + epsilonmks1)/2.0)*((ro + ro1)/2.0)*(4.0/3.0)*pi*r*r*r; 
	i = 1;
        tluminosity = lcentral0;  // set total luminosity 
	looplim = looplimset();  // set the loop limit
//                              according to number table entries.
	while( i < looplim){
	    limset1(j,i);  // set shell density, temp limits ro,ro1,t,t1.    
	    t = t/1.0e8;
	    epsilonmks0 = (50.8)*cy*cy*cy*ro*ro*(1.0/t)*(1.0/t)*(1.0/t)*f3a*exp(-44.02/t);
            t1 = t1/1.0e8;
	    epsilonmks1 = (50.8)*cy*cy*cy*ro1*ro1*(1.0/t1)*(1.0/t1)*(1.0/t1)*f3a*exp(-44.02/t1);
	    limsetr(j,i);  // set shell r limits r0,r1.
	    tluminosity = lumshell(r0,r1,epsilonmks0,epsilonmks1,ro,ro1,tluminosity);    
	    i++;
	}
	heliumluminosity[j] = tluminosity/lsun;
//                      Triple-alpha process luminosity L/Lsun.
//
//
//  K) The p-p fusion chain energy generation is given
//     by Donald D. Clayton as
//
//  epsilon = (2.36x10**6)(density)(X**2)
//             x [((10**6)/T)**(2/3)]exp[-33.81((10**6)/T)**(1/3))]
//             x psi(alpha)
//             x [1 + 0.0123(T/10**6)**(1/3) + 0.0109(T/10**6)**(2/3)
//                          + 0.00095(T/10**6)] ergs/gram-sec,
//
// where  psi(alpha) = phi(alpha)[0.981F_PPI + 0.961FPPII + 0.727FPPIII],
//
// and    alpha = [1.2x10**17][(Y/X)**2]exp[-100*((10**6)/T)**(1/3))].
//
// Here  1 <= phi(alpha) <= 2, and phi(alpha) -> 1 as alpha -> 0,
//                                 phi(alpha) -> 2 as alpha -> infinity,
//
//       and phi(alpha) = 1 - alpha + alpha*[(1 + 2/alpha)**(1/2)].
//
// F_PPI = {[1 + (2/alpha)]**(1/2) - 1}/{((1 + (2/alpha))**(1/2)) + 3 },
//
// F_PPII = (1 - F_PPI)*[ tau_p(Be7)/(tau_p(Be7) + tau_e(Be7) ],
//
// and  F_PPIII = 1 - F_PPI - F_PPII,
//
//  where 1/(tau_e(Be7)) = [7.05x10**-33]*n_e*[((10**6)/T))**(1/2)], &
//
//  1/(tau_p(Be7)) = [6.3x10**(-17)]
//             *n_p*[((10**6)/T))**(2/3)]*exp[-102.65*(10**6/T)**(1/3)].
//
//  Here n_p is the number density of protons per unit volume, and
//       n_e is the number density of electrons per unit volume.
//
//  Thus  n_p = [(density)*X]/mh, where mh is the mass
//                                of the hydrogen atom, and
//
//        n_e = ([(density)*X]/mh)*1.0 + ([(density)*Y]/(4*mh))*2.0
//               + ([(density)*Z]/(2.0*Zp*mh))*Zp
//            = ([(density)*X]/mh)*1.0 + ([(density)*Y]/(4*mh))*2.0
//               + ([(density)*Z]/(2.0*mh)),  pretty nearly.
//
//
//  Here F_PPI is the fraction of alpha particles produced by PPI,
//       F_PPII is the fraction of alpha particles produced by PPII,
// and   F_PPIII is the fraction of alpha particles produced by PPIII.
//
//   Thus, in MKS units,
//
// epsilonmks = 0.236(density)(X**2)
//              [((10**6)/T)**(2/3)]exp[-33.81((10**6)/T)**(1/3))]
//             x psi(alpha)
//          x [1 + 0.0123(T/10**6)**(1/3) + 0.0109(T/10**6)**(2/3)
//                               + 0.00095(T/10**6)] ergs/gram-sec,
//                                             Joules/Kg-sec, with
//                                   the density given in Kg/m**3.
//   The sub-components are defined as given above.
//
//  See Donald D. Clayton, PRINCIPLES OF STELLAR EVOLUTION
//  AND NUCLEOSYNTHESIS, pp 383-387.
//  Note that the function plays well for the Sun, like
//  Hong-Yee Chiu's, but includes
//  factors that can magnify it more as the temperature goes up.
//
//  STAREXP4 introduces weak screening into the Clayton calculation
//  picture, using formulae from Claytons book above, pp.359-361 on
//  electrostatic screening. Weak screening is defined on page 360.
//  The expression for the fusion rate is multiplied by
//  exp(-U/kT), where  -U/kT = 0.188*Z1*Z2*sqrt(ro)*sqrt(zeta)*((T/10**6)**(-3/2)),
//  and T is given in degrees Kelvin. Zeta comes in connection with the
//  definition of the Debye radius in the screening theory (Clayton, pg.360),
//  where zeta = SUM(over Z) of ((Z*Z) + Z)*(X(Z)/A(Z)), and X(Z) is
//  the mass fraction of the element with charge Ze, A(Z) being the atomic
//  weight number, say an integer equal to the number of nucleons in the atom.
//  Calculation shows that zeta computed with Clayton abundances for the Sun
//  is zetasun = 1.855713. (See zeta.cpp).
//
	limset0();     // set central density, temperature limits.

	x = chemx[j];  // set helium fraction by weight

	vo0 = cuberoot(1.0e6/tc);
	vo1 = cuberoot(1.0e6/tc1);

	zeta = zetacal();  // Calculate zeta from detailed aboundances with x(i) Z-shaped
//                              to the Clayton (Aller 1961) heavy element abundance
//                              profile so that X + Y + Sum(3 thru 29)_of x(i) is unity.

	scrn0 = 0.188*sqrt(ro)*sqrt(zeta)*(sqrt(1.0e6/tc));
	scrn0 = scrn0*(sqrt(1.0e6/tc))*(sqrt(1.0e6/tc));
	scrn0 = exp(scrn0);
	scrn1 = 0.188*sqrt(ro)*sqrt(zeta)*(sqrt(1.0e6/tc1));
	scrn1 = scrn1*(sqrt(1.0e6/tc1))*(sqrt(1.0e6/tc1));
	scrn1 = exp(scrn1);

	epsilonmks0 = (0.236)*x*x*ro*vo0*vo0*(exp(-33.81*vo0))*clay(tc,ro,j);
	epsilonmks1 = (0.236)*x*x*ro1*vo1*vo1*(exp(-33.81*vo1))*clay(tc1,ro1,j);
//
//  Compute analogous electron-screened fusion rates
//
	scepsilonmks0 = (0.236)*x*x*ro*vo0*vo0*(exp(-33.81*vo0))*clay(tc,ro,j);
	scepsilonmks0 = scepsilonmks0*scrn0;
	scepsilonmks1 = (0.236)*x*x*ro1*vo1*vo1*(exp(-33.81*vo1))*clay(tc1,ro1,j);
	scepsilonmks1 = scepsilonmks1*scrn1;

	limset2(j);  // find r for exterior of central ball.
	lcentral0 = ((epsilonmks0 + epsilonmks1)/2.0)*((ro + ro1)/2.0)*(4.0/3.0)*pi*r*r*r;

	sclcentral0 = ((scepsilonmks0 + scepsilonmks1)/2.0)*((ro + ro1)/2.0)*(4.0/3.0)*pi*r*r*r;
	
	i = 1;
	tluminosity = lcentral0;  // set total unscreened luminosity
	unclaylum[0][0] = lcentral0/lsun;  // save clayton unscreened p-p fusion for center-to-surface
	sctluminosity = sclcentral0;  // set total screened luminosity
	claylum[0][0] = sclcentral0/lsun;  // save clayton screened p-p fusion for center-to-surface
	//                     This section does the luminosity partition.
        kappa = opacity( (ro + ro1)/2.0, (tc + tc1)/2.0 , j);
	difft = (tc1 - tc)/r; // Set up equation for dT/dr
//                               based on polytrope properties.
//               Set up radiative equilibrium luminosity = chi*L_nuc.
	radequilum[j][0] = -4.0*3.14159*r*r*((4.0*a*c)/3.0)*(tc*tc*tc/(kappa*ro))*difft/lsun;
	ne = (((ro + ro1)/2.0)/m)*(chemx[j] + 0.5*chemy[j] + 0.5*chemz[j]);
	ni = (((ro + ro1)/2.0)/m)*(chemx[j] + 0.25*chemy[j] + (1.0/12.0)*chemz[j]);
	zbar = chemx[j] + 2.0*chemy[j] + 12.0*chemz[j];
//             assume heavy elements carbon.
	ke = ( k/(2.0*3.14159))*sqrt((3.0*k*(tc + tc1)/2.0)/me);
	ke = ke*(4.0*3.14159)*ep0*k*((tc + tc1)/2.0)/(zbar*e*e);
	ke = ke*(4.0*3.14159)*ep0*k*((tc + tc1)/2.0)/(zbar*e*e);
	ke = ke*(ne/ni);
	conduclum[j][0] = -(4.0*3.14159*r*r*difft*ke)/lsun;
//                    approximate by zero at first.
	conveclum[j][0] = claylum[0][0]/normalizeclayton - radequilum[j][0] - conduclum[j][0];
	if( conveclum[j][0]  < 0.0 ) // screen for < 0 error.
	    conveclum[j][0] = 0.0;
	looplim = looplimset();  // set the loop limit
//                              according to number table entries.
	while( i < looplim){
	    limset1(j,i);  // set shell density,
//                            temp limits ro,ro1,t,t1.
	    vo0 = cuberoot(1.0e6/t);
	    vo1 = cuberoot(1.0e6/t1);
	    epsilonmks0 = (0.236)*x*x*ro*vo0*vo0*(exp(-33.81*vo0))*clay(t,ro,j);
	    epsilonmks1 = (0.236)*x*x*ro1*vo1*vo1*(exp(-33.81*vo1))*clay(t1,ro1,j);

	    scrn0 = 0.188*sqrt(ro)*sqrt(zeta)*(sqrt(1.0e6/t));
	    scrn0 = scrn0*(sqrt(1.0e6/t))*(sqrt(1.0e6/t));
	    scrn0 = exp(scrn0);
	    scrn1 = 0.188*sqrt(ro)*sqrt(zeta)*(sqrt(1.0e6/t1));
	    scrn1 = scrn1*(sqrt(1.0e6/t1))*(sqrt(1.0e6/t1));
	    scrn1 = exp(scrn1);
	    scepsilonmks0 = (0.236)*x*x*ro*vo0*vo0*(exp(-33.81*vo0))*clay(t,ro,j);
	    scepsilonmks0 = scrn0*scepsilonmks0;
	    scepsilonmks1 = (0.236)*x*x*ro1*vo1*vo1*(exp(-33.81*vo1))*clay(t1,ro1,j);
	    scepsilonmks1 = scrn1*scepsilonmks1;

	    limsetr(j,i);  // set shell r limits r0,r1.
	    tluminosity = lumshell(r0,r1,epsilonmks0,epsilonmks1,ro,ro1,tluminosity);
	    unclaylum[0][i] = tluminosity/lsun;  // save unscreened clayton luminosity shell-by-shell
	    sctluminosity = lumshell(r0,r1,scepsilonmks0,scepsilonmks1,ro,ro1,sctluminosity);
	    claylum[0][i] = sctluminosity/lsun; // save screened clayton luminosity shell-by-shell

	    	    r = (r0 + r1)/2.0; // Radiative equilibrium
//                              component of luminosity L_radiative.
	    kappa = opacity( (ro + ro1)/2.0, (t + t1)/2.0 ,j);
	    difft = (t1 - t)/((r0 + r1)/2.0); // Set up equation
//                            for dT/dr based on polytrope properties.
//                Set up radiative equilibrium luminosity = chi*L_nuc.
	    radequilum[j][i] = -4.0*3.14159*r*r*((4.0*a*c)/3.0)*(tc*tc*tc/(kappa*ro))*difft/lsun;
	    ne = (((ro + ro1)/2.0)/m)*(chemx[j] + 0.5*chemy[j] + 0.5*chemz[j]);
	    ni = (((ro + ro1)/2.0)/m)*(chemx[j] + 0.25*chemy[j] + (1.0/12.0)*chemz[j]);
	    zbar = chemx[j] + 2.0*chemy[j] + 12.0*chemz[j];   // assume heavy elements carbon.
	    ke = ( k/(2.0*3.14159))*sqrt((3.0*k*(t + t1)/2.0)/me);
            ke = ke*(4.0*3.14159)*ep0*k*((t + t1)/2.0)/(zbar*e*e);
	    ke = ke*(4.0*3.14159)*ep0*k*((t + t1)/2.0)/(zbar*e*e);
	    ke = ke*(ne/ni);
	    conduclum[j][i] = -(4.0*3.14159*r*r*difft*ke)/lsun;
//                               heat conduction luminosity.
	    conveclum[j][i] = claylum[0][i]/normalizeclayton - radequilum[j][i] - conduclum[j][i];
	if( conveclum[j][i]  < 0.0 ) // screen for < 0 error.
	    conveclum[j][i] = 0.0;

	    i++;
	}
	claytonluminosity[j] = tluminosity/lsun;  // Clayton process
//                                                  luminosity L/Lsun.
	scclaytonluminosity[j] = sctluminosity/lsun;  // Clayton screened L/Lsun.

	nscclaytonluminosity[j] = scclaytonluminosity[j]/normalizeclayton; // Normalized,screened.
//
//  Total Nuclear Luminosity & Total H-burning luminosity  -
//      1) Clayton P-P fusion chain - best fidelity to the age of the Sun.
//      2) CNO - carbon cycle. Still being worked on by Green. Not bright enough for Sirius.
//      3) He-burning - Schwarzschild formula.
//
	totalhluminosity  = scclaytonluminosity[0] + cnoluminosity[0]; // total H-burning
//                                                         // luminosity in solar units.
	totalluminosity = totalhluminosity + heliumluminosity[0];
//                     P-P plus CNO plus helium-burning luminosity in solar units.
//                     Check for appropriate electrostatic screening for each reaction.
}
/*****************************
*
*  opacity -  Compute the absorption coefficients & their
*         sum in MKS units. Need factor
*         of 1/10 to convert to MKS units from Schwarzschild cgs.
*
*  k_bf = (4.34e24)*Z*(1+X)*(gtf)*[ro/T**3.5]*gft,
*         approximate bound-free part,
*         gft for minimizing k_bf and maximizing L_radiative
*      = 10.0*sqrt( (1.0 + X)*ro ). Scharzschild, pg. 69.
*            k_ff = (3.68e21)*(X + Y)*[ro/T**3.5], approximate
*                   free-free part,
*  k_E = 0.02*(1 + X)  electron scattering,  See Schwarzschild, pp 62-73.
*                                                                    
*   so   return k_total = k_bf + k_ff + k_E
*
*************************************/
double opacity(double ro, double t, int i)
{
	 double  v, kbf, kff, ke, ktotal;

	 t = sqrt(t);
	 v = t*t*t*t*t*t*t;
	 kbf = 4.34e24*chemz[i]*(1.0 + chemx[i])*(ro/v);
         kbf = kbf/( 10.0*sqrt( (1.0 + chemx[i])*ro ) );
	 kff = 3.68e21*(chemx[i] + chemy[i])*(ro/v);
	 ke = 0.02*(1.0 + chemx[i]);
	 ktotal = kbf + kff + ke;   // Default opacity
	 if(opacityflag == 1){     // 1 ->  bound-free, free-free, electron scattering
	    ktotal = kbf + kff + ke;  
	 }
	 if(opacityflag == 2){     // 2 ->  free-free, electron scattering only
	    ktotal = kff + ke;  
	 }
	 if(opacityflag == 3){     // 3 ->  electron scattering only. 
	    ktotal = ke;  
	 }
         return(ktotal);

}            
/***************************************************
*
*   looplimset - Set looplim according the the number
*                of table entries in the associated
*                table of polytropic coefficients.
*
****************************************************/
int looplimset(void)
{
	int looplim;

        if(starform[j] == 0)
	    looplim = 26;
	else if(starform[j] == 1)
	    looplim = 21;         // n=1.5 case.
	else if(starform[j] == 2)    // n=3.5
	    looplim = 33;
	else if(starform[j] == 3)    // n=4.0
	    looplim = 32;  // back off two extra steps.

        return(looplim);
}
/***************************************************
*
*  lumshell - compute the luminosity of a shell.
*
****************************************************/
double lumshell(double r0, double r1, double epsilonmks0,
   double epsilonmks1, double ro, double ro1, double tluminosity)
{
	    double r,dv,dluminosity0,dluminosity1,dluminosity;

            r  =( r0 + r1)/2.0;   // use shell midpoint.
	    dv = 4.0*pi*r*r*(r1 - r0); // compute shell volume.
	    dluminosity0 = epsilonmks0*ro*dv;  // use trapezoidal rule.
	    dluminosity1 = epsilonmks1*ro1*dv;
            dluminosity = (dluminosity0 + dluminosity1)/2.0;
	    tluminosity = tluminosity + dluminosity;
            return(tluminosity);
}
/****************************************************
*
*  limset0() - Set central ball density, temp limits.
*
*****************************************************/
void limset0(void)
{
	double phi;

        ro = xdensitycentral[j];   // get central density in kg/m**3
	tc = xtc[j];               // get temp in deg. Kelvin at center.

	if(starform[j] == 0){     // gamma = 4/3, n=3 Eddington
//                                   standard model case.
	    tc1 = xtc[j]*theta[1];     // get temp at 1st step.
	    ro1 = (xdensitycentral[j])*theta[1]*theta[1]*theta[1];
	    
	}
	else if(starform[j] == 1){  // gamma=5/3, n=1.5, 
	    tc1 = xtc[j]*xtheta[1];     // get temp at 1st step.
            phi = sqrt(xtheta[1]);
	    ro1 = (xdensitycentral[j])*phi*phi*phi;
	    
	}
	else if(starform[j] == 2){  // n=3.5,
	    tc1 = xtc[j]*yxtheta[1];     // get temp at 1st step.
            phi = sqrt(yxtheta[1]);
	    ro1 = (xdensitycentral[j])*phi*phi*phi*phi*phi*phi*phi;
	    
	}
	else if(starform[j] == 3){  //  n=4.0,
	    tc1 = xtc[j]*zxtheta[1];     // get temp at 1st step.
	    ro1 = (xdensitycentral[j])*zxtheta[1]*zxtheta[1]*zxtheta[1]*zxtheta[1];
	    
	}

}
/*****************************************
*
*   limset1() - Set shell ro,ro1,t,t1 density & temp bounds.
*
******************************************/
void limset1(int j, int i)
{
	    double phi0,phi1;

	    if(starform[j] == 0){  // gamma=4/3,
//                                    n=3 star form assumption.
	       ro = xdensitycentral[j]*theta[i]*theta[i]*theta[i];
	       ro1 = xdensitycentral[j]*theta[i+1]*theta[i+1]*theta[i+1];
	       t = xtc[j]*theta[i];
	       t1 = xtc[j]*theta[i+1];
	   }
	   else if(starform[j] == 1){
               phi0 = sqrt(xtheta[i]);
	       ro = xdensitycentral[j]*phi0*phi0*phi0;
	       phi1 = sqrt(xtheta[i+1]);
	       ro1 = xdensitycentral[j]*phi1*phi1*phi1;
	       t = xtc[j]*xtheta[i];
	       t1 = xtc[j]*xtheta[i+1];
	   }
	   else if(starform[j] == 2){  // n=3.5
               phi0 = sqrt(yxtheta[i]);
	       ro = xdensitycentral[j]*phi0*phi0*phi0*phi0*phi0*phi0*phi0;
	       phi1 = sqrt(yxtheta[i+1]);
	       ro1 = xdensitycentral[j]*phi1*phi1*phi1*phi0*phi0*phi0*phi0;
	       t = xtc[j]*yxtheta[i];
	       t1 = xtc[j]*yxtheta[i+1];
	   }
	   else if(starform[j] == 3){    // n=4
	       ro = xdensitycentral[j]*zxtheta[i]*zxtheta[i]*zxtheta[i]*zxtheta[i];
	       ro1 = xdensitycentral[j]*zxtheta[i+1]*zxtheta[i+1]*zxtheta[i+1]*zxtheta[i+1];
	       t = xtc[j]*zxtheta[i];
	       t1 = xtc[j]*zxtheta[i+1];
	   }

}
/*****************************************
*
*   limset2(j) - pick central ball exterior radius r.
*
******************************************/
void limset2(int j)
{
        if(starform[j] == 0){      // n=3 is polytropic index.
	     r = (xr[1]/6.9001)*xrstar[j];  // get r in meters.
	}
	else if(starform[j] == 1){  // n=1.5
	     r = (xxr[1]/3.65375)*xrstar[j];  // get r in meters.
	}
	else if(starform[j] == 2){ // n=3.5
	     r = (yxr[1]/9.53581)*xrstar[j];  // get r in meters.
	}
	else if(starform[j] == 3){  // n=4
	     r = (zxr[1]/14.97155)*xrstar[j];  // get r in meters.
	}
}
/*********************************************
*
*  limsetr(j,i) - Set shell r-limits r0,r1
*
**********************************************/
void limsetr(int j, int i)
{
	   double y;

           if(starform[j] == 0){    // n=3, gamma=4/3.
              r0 = (xr[i]/6.9001)*xrstar[j];  // get r in meters.
	      r1 = (xr[i+1]/6.9001)*xrstar[j];  // get r in meters,
//                                                 next shell.
	   }
	   else if(starform[j] == 1){   // n=1.5, gamma=5/3.
	      r0 = (xr[i]/3.65375)*xrstar[j];  // get r in meters.
	      r1 = (xr[i+1]/3.65375)*xrstar[j];  // get r in meters,
//                                                  next shell.
	   }
	   else if(starform[j] == 2){   // n=3.5.
	      r0 = (xr[i]/9.53581)*xrstar[j];  // get r in meters.
	      r1 = (xr[i+1]/9.53581)*xrstar[j];  // get r in meters,
//                                                   next shell.
	   }
	   else if(starform[j] == 3){   // n=4.0.
	      r0 = (xr[i]/14.97155)*xrstar[j];  // get r in meters.
	      r1 = (xr[i+1]/14.97155)*xrstar[j];  // get r in meters,
//                                                   next shell.
	   }
}
/***************************************************
*
*   clay() - Compute the clayton factor for pp-fusion.
*
****************************************************/
//  epsilon = (2.36x10**6)(density)
//            (X**2)[((10**6)/T)**(2/3)]
//            *exp[-33.81((10**6)/T)**(1/3))]
//             x psi(alpha)
//         x [1 + 0.0123(T/10**6)**(1/3) + 0.0109(T/10**6)**(2/3)
//                       + 0.00095(T/10**6)] ergs/gram-sec,
//
// where  psi(alpha) = phi(alpha)[0.981F_PPI + 0.961FPPII + 0.727FPPIII],
//
// and    alpha = [1.2x10**17][(Y/X)**2]exp[-100*((10**6)/T)**(1/3))].
//
// Here  1 <= phi(alpha) <= 2, and phi(alpha) -> 1 as alpha -> 0,
//                               phi(alpha) -> 2 as alpha -> infinity,
//
// and phi(alpha) = 1 - alpha + alpha*[(1 + 2/alpha)**(1/2)].
//
//  F_PPI = {[1 + (2/alpha)]**(1/2) - 1}/{((1 + (2/alpha))**(1/2)) + 3 },
//
//  F_PPII = (1 - F_PPI)*[ tau_p(Be7)/(tau_p(Be7) + tau_e(Be7) ],
//
//  and  F_PPIII = 1 - F_PPI - F_PPII,
//
//  where 1/(tau_e(Be7))
//      = [7.05x10**-33]*n_e*[((10**6)/T))**(1/2)],  and
//
//  1/(tau_p(Be7)) = [6.3x10**(-17)]
//      *n_p*[((10**6)/T))**(2/3)]*exp[-102.65*(10**6/T)**(1/3)].
//
//  Here n_p is the number density of protons per unit volume, and
//       n_e is the number density of electrons per unit volume.
//
//  Thus  n_p = [(density)*X]/mh, where mh is the mass of the
//       hydrogen atom, and
//
//        n_e = ([(density)*X]/mh)*1.0 + ([(density)*Y]/(4*mh))*2.0
//               + ([(density)*Z]/(2.0*Zp*mh))*Zp
//            = ([(density)*X]/mh)*1.0 + ([(density)*Y]/(4*mh))*2.0
//               + ([(density)*Z]/(2.0*mh)),  pretty nearly.
//
//
// Here F_PPI is the fraction of alpha particles produced by PPI,
//     F_PPII is the fraction of alpha particles produced by PPII,
// and   F_PPIII is the fraction of alpha particles produced by PPIII.
//
//   Thus, in MKS units,
//
// epsilonmks = 0.236(density)(X**2)
//      *[((10**6)/T)**(2/3)]exp[-33.81((10**6)/T)**(1/3))]
// x psi(alpha) x [1 + 0.0123(T/10**6)**(1/3) + 0.0109(T/10**6)**(2/3)
//                               + 0.00095(T/10**6)] ergs/gram-sec,
//                                               Joules/Kg-sec, with
//                                       the density given in Kg/m**3.
//   The sub-components are defined as given above.
//
double clay(double t, double ro, int j)
{
	double v;
	double alpha;
	double v2;
        double t6;       //  t/10**6
	double root3t6;  // (t/10**6)**(1/3)
	double vt;
	double x;
	double y;
        
	x = chemx[j];
        y = chemy[j];
        vt = cuberoot(1.0e6/t);
	alpha = (1.2e17)*(y/x)*(y/x)*exp( -100.0*(vt) );
	v = psi(alpha,ro,t,j);
        t6 = t/1.0e6;
        root3t6 = cuberoot(t6);
	v2 = 1.0;
	v2 = v2  + 0.0123*root3t6;
	v2 = v2  + 0.0109*root3t6*root3t6;
	v2 = v2  + 0.00095*t6;
        v = v2*v;
       return(v);  
}
double psi(double alpha, double ro, double t, int j)
{
	double phi;
	double v;
	double np;     // proton number density.
	double ne;     // electron number density.	
	double x,y,z;  // abundances H,He,Heavy.
	double q;
	double taup;
	double taue;
	double fppi;
	double fppii;
	double fppiii;
	double root3t6;
	double root2t6;

       phi = 1.0 - alpha;
       phi = phi + (alpha*( sqrt(1.0 + (2.0/alpha)) ));

       x = chemx[j];
       y = chemy[j];
       z = chemz[j];
       np = (ro*x)/mh;
       ne = ((ro*x)/mh)*1.0 + ((ro*y)/(40*mh))*2.0 + ((ro*z)/(2.0*mh));

       root2t6 = sqrt(1.0e6/t);
       q = (7.05e-33)*ne*root2t6;
       taue = 1.0/q;

       root3t6 = cuberoot((1.0e6/t));

       q = (6.3e-17)*np*(root3t6*root3t6)*exp( -102.65*root3t6 );
       taup = 1.0/q;

       fppi = (sqrt(1.0 + (2.0/alpha)) - 1.0)/( sqrt( 1.0 + (2.0/alpha) ) + 3.0 );

       fppii = (1.0 - fppi)*( taup/(taup + taue) );

       fppiii = 1.0 - fppi - fppii,

       v =  phi*(0.981*fppi + 0.961*fppii + 0.727*fppiii);

       return(v);  
}
/****************************************************
*
*   reportms() - Report results of main sequence run
*                in "depthsta.txt" for the entire set
*                of models from stored memory variables.  <R>
*
****************************************************/
void reportms(void)
{	
	int i;       // model index.
	int gk;    // integer spectral type number.
	int g;   // g = sp[i].spectnumber.
	char feedfwd = 0x0C;  // Feed forward character.
//                              Seems not to work.
	double modelrr;   // idealized uniform
//                           core compaction ratio r/R for
//         Lstar = [(4pi/3)*r**3]*epsilonmks[ro_central,T_central].

	double tfreefall;  //  Free Fall time scale is ((R**3)/(GM))**(1/2) seconds.

	double deltastar;   // [L_star - L_Claton]/L_star.
        double imoonsun; // Imoon/Isun = fnovasun/fnovamoon.
	double snlum; // supernova luminosity in solar units on day gd.
	double distratio; // disance ration r_Sun/r_SN.
	double mvis;      // m_visual(gd) of supernova on day gd.
	double deltamag;  // delta visual magnitude.
	double dwave;       // distance the blast wave has
//                             propagated from the SN star.
	double blastyear;  // blastwave arrival time, years
//                            into the future.
        double blastv;    // blastwave velocity, km/sec.
	int gd;  // day count of supernova burst.
	double fnovamoon; // SN_intensity/moonlight_intensity
	double fnovasun; // SN brilliance/solar brilliance.
	double mvis0;  // initial supernova magnitude
	double lamda; // wavelength at emission maximum in spectrum.
        double y;    // floating-point i.
	double pc;  // distance in parsecs.

	double dmeters;  // distance in meters.
	
	double parallax;  // distance in arc-sec of parallax.
	double p,t,ro,r;  // pressure, temperature, density, radius.

	double tem[36];   // temperature array from center to surface for convection cal.
	double pres[36];   // pressure array from center to surface for convection cal.
	double rdis[36];   // radial position in meters for convection cal.
	double dtdx;       // (T(i + 1) - T(i))/deltar .
	double fgammapt;   // ((gamma - 1)/gamma)*((T(i + 1) + T(i))/2.0)
//                            *( 2.0/(T(i + 1) + T(i)) )*((P(i + 1) + P(i))/deltar)
	double delstep;	  // delta_r, radial step delstep.

	double ti;        // Phillips typical internal temperature, pg.20.
	double photonmfp; // Photon mean free path, ((T_surface/T_typ_internal)**4)*R
	
	double photonnum;  // Central Photon number density  = (2.03e7)*(T**3), Phil.p.60.
	double photonen;  // Central Photon energy density  = (7.565e-16)*(T**4), Phil.p.61.
	int k;           // polytrope step radius 0-17.
	double kboltz = 1.3805e-23;  // Boltzmann's constant in J/K.
	double m43; 	// M**(4/3)
	double mf0;     // m_agv is mu*m_hydrogen
	double mf;
	double tphil; // Phillips electron degeneracy temperature A*(M**(4/3))
//                          should approximate central temperature.
	double phi;
	double lgro;   // log10(density) - working var, degen. tests.
	int degencode; // 0->no electron degeneracy.
//                        1->relativistic electron degeneracy.
//                        2->non-relativisitic electron degeneracy.
	double mue;  // mu_e = (1 + X)/2, pretty nearly.
	double chemmucore;  // the molecular weight of the core of the star. A perturbation.
	double k1;   // k1 = 1.415e10*[(2/(1 + X))**(5/3)],
//                      P = K1*(ro)**(5/3) non-rel.degen.electrons
	double zv, ro3,ro43;
	double rn1, rn2;  // Two ratios of ionization of species in the stellar photosphere.
	double vx,minlum,maxlum;
	double rotcrit;    // critical rotational angular velocity for breakup.
	double centradpress;  // central radiation pressure.
	double sn;  // supernova probability factor.
	double snf1, snf2; // other supernova probability factors.

	double rinitialratio;  // R/R_solar initial ionized radius during star formation.
	double aratio; // idealized core compaction factor a/R.
	double dau; // distance to the star in AU.
	double ps; // SN blast wave pressure on atmosphere.
	double delt; // temperature change induced by SN.
	double events; // neutrino events per cm**3 in human body.
	double v2;  // SN amplitude in ergs.
	double m_moon;  // magnitude of the moon.
	double degenrelativistic;  // relativistic
//                                    degeneracy seen = 1 flag.
	double underluminous;  // model underluminous = 1, else 0.
	double nqarg, nq;    //  crit. quantum concentration for elec. degen. at T.
	int newclass;  // reclassification code.
//                0->undetermined.
//	          1  -> class Ia-O. 11 -> class Ia.
//                12 -> class Ib.
//                2 -> class II, bright giants.
//                3 -> class III, giants.
//                4 -> class IV.
//                5 -> class V, dwarfs & regular
//                7 -> class VII, white dwarfs.
	int looplim;

	int ij;

	
	fprintf(fp,"\n\nDEPTHSTA: Main Sequence Modeling. Report_File=depthsta.txt.\n");
	time(&secs_now);
	str_now = ctime(&secs_now);
	fprintf(fp,"%s\n",str_now);
	fprintf(fp,"%d models processed in this run, 0-%d.\n",
	msmodels,msmodels - 1);
	i = 0;
	while(i < msmodels){
	    fprintf(fp,"\nMODEL = %d INPUT VARIABLES:\n",i);
	    fprintf(fp,"Starname = %s.\n",sp[i].starname);
	    if(starform[i] == 0)
	       fprintf(fp,"Assumed polytropic structure: n=3, gamma=4/3.\n");
	    if(starform[i] == 1)
	       fprintf(fp,"Assumed polytropic structure: n=1.5, gamma=5/3.\n");
	    if(starform[i] == 2)
	       fprintf(fp,"Assumed polytropic structure: n=3.5, gamma=9/7.\n");
	    if(starform[i] == 3)
	       fprintf(fp,"Assumed polytropic structure: n=4.0, gamma=5/4.\n");
	    fprintf(fp,"Distance = %e LY,\n",lightyears[i]);
            pc = (lightyears[i]/3.26);
	    fprintf(fp,"         = %e parsecs,\n",pc);
            dmeters = scale*lightyears[i];
	    fprintf(fp,"         = %e meters,\n",dmeters);
	    parallax = au/dmeters;   // parallax in radians
	    parallax = parallax*(180.0/3.14159); // parallax in degrees.
	    parallax = parallax*(60.0)*(60.0); // parallax in arc-sec.
	    fprintf(fp,"         = %e arc-seconds of parallax.\n",
	    parallax);
	    fprintf(fp,"Visual magnitude = %f. \n",
	    visualmag[i]);
	   
            if(sp[i].equiv == 0)
	       fprintf(fp,"Surface temperature[spect.type] = %e degrees Kelvin.\n",
	       surfacetemp[i]);
	    if(sp[i].equiv == 1)
	       fprintf(fp,"Surface temperature was specified as = %e deg.Kelvin.\n",
	       surfacetemp[i]);
	    if(sp[i].equiv == 1)
	       fprintf(fp,"Spectral type deduced from surface temp entry: \n");
	    if(sp[i].equiv == 0)
	       fprintf(fp,"Spectral type specified at entry time: \n");

//	       g = sp[i].spectnumber;
//	       if(g == 0.0 || g == 1.0 || g == 2.0
//             || g == 3.0 || g == 4.0 ){
//		     gk = g; // subsitute integrer for fractional type.
//		     fprintf(fp,"Spectral type = %c%d ",
//                   sp[i].spectcode, gk);
//	       }
//	       else if( g == 5.0 || g == 6.0 || g == 7.0
//             || g == 8.0 || g == 9.0 || g == 10.0){
//		     gk = g; // subsitute integrer for fractional type.
//		     fprintf(fp,"Spectral type = %c%d ",
//                   sp[i].spectcode, gk);
//               }
//               else
		 fprintf(fp,"Henry Draper OBAFGKM Spectral type = %c%2.1f ",
		 sp[i].spectcode, sp[i].spectnumber);
	    if(sp[i].classification == 1)
		 fprintf(fp,"III [giant].\n");
	    if(sp[i].classification == 2)
		 fprintf(fp,"V [dwarf, regular].\n");
	    if(sp[i].classification == 3)
		 fprintf(fp,"I [supergiant].\n");
	    if(sp[i].classification == 4)
		 fprintf(fp,"IV [subgiant]\n");
	    if(sp[i].classification == 5)
		 fprintf(fp,"ZAMS V [zero-age main sequence].\n");
	    if(sp[i].classification == 6)
		 fprintf(fp,"VI [subdwarf].\n");
	    if(sp[i].classification == 7)
		 fprintf(fp,"VII [white dwarf].\n");
	    if(sp[i].classification == 8)
		 fprintf(fp,"II [bright giant].\n");

            if(sp[i].spectcode == 'O')
	      fprintf(fp,"Spectrum: Hot star with He II absorption.\n");
	    if(sp[i].spectcode == 'B')
	      fprintf(fp,"Spectrum: He I absorption, H developing later.\n");
	    if(sp[i].spectcode == 'A')
	      fprintf(fp,"Spectrum: Very strong H, decreasing later, Ca II increasing.\n");
	    if(sp[i].spectcode == 'F')
	      fprintf(fp,"Spectrum: Ca II stronger, H weaker, metals developing.\n");
	    if(sp[i].spectcode == 'G')
	      fprintf(fp,"Spectrum: Ca II strong, Fe & other metals strong, H weaker.\n");
	    if(sp[i].spectcode == 'K')
	      fprintf(fp,"Spectrum: Strong metallic lines, CH & CN bands developing.\n");
	    if(sp[i].spectcode == 'M')
	      fprintf(fp,"Spectrum: Very red, TiO bands developing strongly.\n");
	    if(sp[i].spectcode == 'V') // V specifies the Sun
//                                          for Test Purposes.
	      fprintf(fp,"The Sun: Ca II strong, Fe & other metals strong, H weaker.\n");
	    lamda = 2.9e6/surfacetemp[i];
	    fprintf(fp,"Wavelength at emission maximum = %f nanometers.\n",lamda);
	    if(abuncode[i] == 1)
	       fprintf(fp,"Schwarzschild abundances for Population I were used:\n");
	    if(abuncode[i] == 2)
	       fprintf(fp,"Green helium-rich abundances for Population I were used:\n");
	    if(abuncode[i] == 3)
	       fprintf(fp,"User-Specified abundances were used:\n");
	    if(abuncode[i] == 4)
	       fprintf(fp,"Mu was varied to optimize L_nuclear:\n");
	    if(abuncode[i] == 5)
	       fprintf(fp,"Mu was varied with core of iron + H + He to optimize L_nuc:\n");
	    if(abuncode[i] == 6)
	       fprintf(fp,"Mu was varied with core of U238 + H + He to optimize L_nuc:\n");
	    fprintf(fp,"X[H] = %f, Y[He] = %f, Z[hvy] = %f.\n",
	    chemx[i],chemy[i], chemz[i]);
	    fprintf(fp,"Clayton L/L_sun = %e, .\n", normalizeclayton);	    
	    claynormx = chemx[0]/sqrt(normalizeclayton);
	    fprintf(fp,"Approximate X[H] to normalize Clayton by core exhaustion = %f, .\n",
	    claynormx);
	    fprintf(fp,"Detailed model abundances X(Z), by weight, for screening,\n");
	    fprintf(fp,"with heavy element abundance shaped like Aller,1961 - Clayton.\n");
	    fprintf(fp," 1->H,                                                        2->He,\n");
	    fprintf(fp," 3->Li, 4->Be, 5->B,  6->C, 7->N,  8->O, 9->F,               10->Ne,\n");
	    fprintf(fp,"11->Na,12->Mg,13->Al,14->Si,15->P,16->S, 17->Cl,             18->Ar,\n");
	    fprintf(fp,"19->K ,20->Ca,21->Sc,22->Ti,23->V,24->Cr,25->Mn,26->Fe,27->Co,28->Ni\n");
	    fprintf(fp,"   0 <=  X(Z) <=  1,  and  Parts by weight\n");
	    for(ij = 1; ij < 30; ij++){
	      fprintf(fp," chem[%d] = %f      %f\n",ij, chem[ij], 1000.0*(chem[ij]/chem[1]));
	    }
            fprintf(fp,"X_CNO = %f.\n",real_xcno[i]);
	    fprintf(fp,"Molecular weight mu = %f.\n\n",chemmu[i]);

	    fprintf(fp,"Stellar Photosphere Ionization Ratios from the Saha Equation:\n");
	    rn1 = exp(98000.0/surfacetemp[i]);   // Phillips, pg.70.
	    rn2 = exp(-128000.0/surfacetemp[i]);
	    fprintf(fp,"     n(Na+)/n(Na) = %e n(H+)/n(H),\n",rn1);
	    fprintf(fp,"     n(He+)/n(He) = %e n(H+)/n(H)\n\n",rn2);

	    fprintf(fp,"MODEL = %d OUTPUT VARIABLES, %s:\n",
	    i,sp[i].starname);
	    if(bcflag == 0)
	      fprintf(fp,"Zombeck bolometric corrections used.\n");
	    if(bcflag == 1)
	      fprintf(fp,"Schwarzschild bolometric corrections used.\n");
	    fprintf(fp,"Absolute magnitude before bolometric correction: M = %f.\n",
	    rawabsmag[i]);
	    fprintf(fp,"Absolute magnitude after correction: M + B.C = %f.\n",
	    bcabsmag[i]);
	    fprintf(fp,"Bolometric Correction = %f.\n\n",bolo[i]);
//	    fprintf(fp,"Visual magnitude above atmosphere
//                         = m_bol = %f\n", visualmag[i] + bolo[i]);
	    fprintf(fp,"L/Lsun, M/Msun, R/Rsun, prior to bolometric correction:\n");
	    fprintf(fp,"L/Lsun = %f, M/Msun = %f, R/Rsun = %f.\n\n",
	         rawlratio[i],rawmratiobest[i],rawrratio[i]);
	    fprintf(fp,"After bolometric corrections, these quantities are:\n");
	    fprintf(fp,"L/Lsun = %f, M/Msun = %f, R/Rsun = %f.\n\n",
	         bclratio[i], bcmratiobest[i], bcrratio[i]);   
	    fprintf(fp,"Radius = %e meters.\n",xrstar[i]);
	    fprintf(fp,"Mass   = %e kilograms.\n",xmstar[i]);
	    fprintf(fp,"Average density = %e kg/m**3.\n",
	        xavgdensity[i]);
	    fprintf(fp,"Central density = %e kg/m**3.\n",
		xdensitycentral[i]);
	    fprintf(fp,"   Central number density = %e particles/m**3.\n",
		xdensitycentral[i]/(chemmu[i]*mh));

	    nqarg =  2.0*3.14159*xtc[i]*kboltz*chemmu[i]*mh/(h*h);
	    nq = 2.0*(sqrt(nqarg))*(sqrt(nqarg))*(sqrt(nqarg));
	    fprintf(fp,"   Crit central g_s*N_Q quantum concentration = %e particles/m**3.\n",
		nq);
	    fprintf(fp,"   g_s*N_Q/n = %e, number_for_elec.degen(Tc)/actual_n, Phillips,p50.\n",
		nq/(xdensitycentral[i]/(chemmu[i]*mh)));
	    fprintf(fp,"   Compare with competing criteria electron degeneracy code results below.\n",
		nq/(xdensitycentral[i]/(chemmu[i]*mh)));

	    fprintf(fp,"Central pressure = %e Newtons/m**3.\n",
	        xpressurecentral[i]);
	    if(starform[i] == 0){     // gamma=4/3
	        fprintf(fp,"fbeta = [1-b]/b**4 = %e.  ",xfbeta[i]);
		fprintf(fp,"beta = %e.\n",xbeta[i]);
            }
	    fprintf(fp,"Central gas pressure = %e Newtons/m**2.\n",
	         xpgas[i]);
	    if(starform[i] == 0){   // gamma = 4/3
	       fprintf(fp,"Central radiation pressure = %e Newtons/m**2.\n",
	            xpradiation[i]);
	       fprintf(fp,"Ratio radiation_pressure/gas_pressure = %e.\n",
	            xpratio[i]);
	    }
	    else if(starform[i] == 1 || starform[i] == 2
	    || starform[i] == 3){
                centradpress = (1.0/3.0)*a*tc*tc*tc*tc;
		fprintf(fp,"Central Radiation pressure = %e Newtons/m**2.\n",
		   centradpress);
		fprintf(fp,"Central Rad_Pressure/Central_gas_pressure = %e.\n",
		   centradpress/xpressurecentral[i]);
	    }

	    fprintf(fp,"Central temperature = %e degrees Kelvin.\n",
		xtc[i]);
	    fprintf(fp,"    Central photon density = %e photons/m**3.\n",
		2.03e7*xtc[i]*xtc[i]*xtc[i]);  // Phillips,pg.60-61.
	    fprintf(fp,"    Central photon energy density = %e J/m**3.\n",
		7.565e-16*xtc[i]*xtc[i]*xtc[i]*xtc[i]);
	    ti = gnewton*xmstar[i]*chemmu[i]*mh/(kboltz*3.0*xrstar[i]);
	    fprintf(fp,"Typical Phillips internal temp T_i = %e degrees K.\n",
		ti);
	    fprintf(fp,"    Typical photon density = %e photons/m**3.\n",
		2.03e7*ti*ti*ti);  // Phillips,pg.60-61.
	    fprintf(fp,"    Typical photon energy density = %e J/m**3.\n",
		7.565e-16*ti*ti*ti*ti);

//	    m43 = cuberoot(xmstar[i]); // T = (1/k)((G*G*((mu*m_H)**(8/3))*m_e/h*h)*(M**4/3)
//	    m43 = nthpower(m43,4);   // This is the temp at which electrons become degenerate.
//	    mf0 = 0.5*mproton;       // Pure hydrogen case, average mass.
//	    mf = cuberoot(mf0);      // Phillips, eqn 1.28, pg.18.
//	    mf = ((mf*mf)*(mf*mf))*((mf*mf)*(mf*mf));
//	    tphil = (1.0/kboltz)*((gnewton*gnewton*mf*me)/(h*h))*m43;
//	    fprintf(fp,"Phillips electron degen temp A*M**(4/3) = %e deg K.\n",
//		tphil);
//	    fprintf(fp,"Probe: M = %e, mavg = %e, m_e = %e\n",
//		xmstar[i],mf0,me);
//	    fprintf(fp,"Probe: M**(4/3) = %e, mavg**(8/3) = %e, h = %e, k = %e\n",
//		m43,mf,h,kboltz);

	    fprintf(fp,"Stellar gravitational Binding Energy = %e Joules.\n",
		xomega[i]);
	    fprintf(fp," \n");    // issue top of page - feed forward.
//
//  Stars live about [9.92x10**9](Msun/M)**2 years.
//  Therefore the maximum lifetime
//  of a supernova progenitor is about
//                 [9.92x10**9](1/6.23)**2 years.
//  Thus, the relative likelihood that the star
//   will supernova sometime soon is
//
//   1 - {[9.92x10**9](Msun/M)**2}/[9.92x10**9](1/6.23)**2
//        = 1.0 - 38.8(Msun/M)**2, M > 6.23.
//
//
	    if(bcmratiobest[i] > 6.23){
	       fprintf(fp,"Star Type II pre-supernova, M/Msun = %e > 6.23 Msun.\n",
	         bcmratiobest[i]);
	       sn = 38.8*(msun/xmstar[i])*(msun/xmstar[i]);
//                               old formula ala Bowers&Deeming.
	       sn = 1.0 - sn;
	       fprintf(fp,"Relative likelihood of SN soon in progenitor class = %f,\n\n",
	       sn);
       
	    }
	    else{
	       fprintf(fp,"The star %s,\n",sp[i].starname);
	       fprintf(fp,"   M/Msun = %e < 6.23 Msun,\n",
		  bcmratiobest[i]);
	       fprintf(fp,"   is not a Type II supernova progenitor.\n");
	    }
	    age = (1.0/bcmratiobest[i])*(1.0/bcmratiobest[i])*(9.92e9);
//                                       Bowers&Deeming Style
	    ageschw = 12.88e9*exp( 2.409*log( 1.0/bcmratiobest[i] ));
//                                       1/M**2.409 lifetime.
	    fprintf(fp,"Stellar M.S. lifetime = 12.88x10**9[Msun/M]**2.409 = %e yrs.\n",
	    ageschw);
	    vv = sqrt(1.0/bcmratiobest[i]);
	    zage = vv*vv*vv*vv*vv*13.0e9;
	    fprintf(fp,"Zombeck lifetime = 13.0x10**9[Msun/M]**2.5 = %e yrs.\n",
	       zage);
	    fprintf(fp,"Green's 1st M.S.lifetime = 9.92x10**9[Msun/M]**2 = %e yrs.\n",
	       age);

	    hkage  = ((4.10428805e-45)*(xmstar[i])*xmstar[i])/((bclratio[i])*xrstar[i]);
	    fprintf(fp,"Helmholtz-Kelvin gamma=4/3 contraction lifetime = %e yrs,\n",
	       hkage);
	    fprintf(fp,"   an upper limit to the star formation time.\n");

	    rinitialratio = ((43.2)*(bcmratiobest[i]))/(1.0 - 0.2*chemx[i]);
	    fprintf(fp,"   Initial star formation ionized radius R/R_solar = %e \n",
	       rinitialratio);
	    fprintf(fp,"   T_average at initial ionized radius = 30,000 deg. K.\n");

	    tfreefall = xrstar[i]*xrstar[i]*xrstar[i]/((6.67e-11)*xmstar[i]);
	    tfreefall = 1.11072*sqrt(tfreefall); // The Physics of Stars, A.C.Phillips.
	    tfreefall = tfreefall/60.0;  // A.C.Phillips free-fall time, pg.7.
	    fprintf(fp,"Phillips freefall pulse time scale = %e min \n", tfreefall);
	    fprintf(fp,"   = %e hrs = %e days = %e years.\n",
	      tfreefall/60.0, tfreefall/1440.0, tfreefall/526176.0 );
	    fprintf(fp,"   Cepheid - typ 10 days, RR Lyrae - typ 0.5 to 1 day,\n");
	    fprintf(fp,"   both in F-G range, Red supergiants 100 days to several years.\n");

	    rotcrit = (6.67e-11)*xmstar[i]/(xrstar[i]*xrstar[i]*xrstar[i]);
	    rotcrit = sqrt(rotcrit);
	    fprintf(fp,"Critical breakup angular rotation = %e radians/sec.\n", rotcrit);
	    fprintf(fp,"    or %e rev/sec = %e rev/min\n", rotcrit/6.28318, rotcrit*9.549305);
	    fprintf(fp,"    = %e rev/day, Surface velocity = %e km/sec.\n",
	      rotcrit*13750.1, rotcrit*xrstar[i]/1000.0);

	    photonmfp =   surfacetemp[i]/ti;  // Compute photon mean free path.
	    photonmfp = nthpower(photonmfp,4);
	    photonmfp = photonmfp*xrstar[i];
	    photontime = (xrstar[i]*xrstar[i])/(c*photonmfp);
	    fprintf(fp,"Photon mean free path = %e meters ", photonmfp); // Phillips, p.23.
	    fprintf(fp,"= %e mm\n", photonmfp*1000.0);
	    fprintf(fp,"Photon random-walk escape time = %e sec ", photontime); // Phillips, p.23.
	    fprintf(fp,"= %e years.\n", photontime/3.156e7);
	    auxluminosities(); // L_solar_wind(R) & L_neutrino(R)

	    fprintf(fp,"\nVARIABLES FROM CENTER TO SURFACE I\n");
	    if(starform[i] == 0)
	      fprintf(fp,"n=3, gamma=4/3, %s.\n",
		 sp[i].starname);
	    if(starform[i] == 1)
	      fprintf(fp,"n=1.5, gamma=5/3, %s.\n",
	         sp[i].starname);
	    if(starform[i] == 2)
	      fprintf(fp,"n=3.5, gamma=9/7, %s.\n",
	         sp[i].starname);
	    if(starform[i] == 1)
	      fprintf(fp,"n=4.0, gamma=5/4, %s.\n",
	         sp[i].starname);
	    fprintf(fp,"Radius           Density       Temperature   Pressure   Degeneracy\n");
	    fprintf(fp," r/Rsun          kg/m**3       deg. Kelvin    Newtons/m**2  Code\n");
            degenrelativistic = 0;  // reset relativistic-dengeneracy seen flag.
	    k=0;
	    looplim = looplimset2(i);
	    while(k < looplim){
              if(starform[i] == 0){ // gamma = 4/3 model
	          p = xpressurecentral[i]*theta[k]*theta[k]*theta[k]*theta[k];
		  t = xtc[i]*theta[k];
		  ro = xdensitycentral[i]*theta[k]*theta[k]*theta[k];
		  if(t > 1.0e7)
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		  r = (xrstar[i]/rsun)*(xr[k]/6.9011);
		  lgro = 5.964 + log10( (1.0 + chemx[j])/2.0 );
		  degencode = 0;    // preset for no degeneracy.
		  if(ro > 0.003){
		     if(log10(ro) > lgro){
//                        See THERMONUCLEAR FUSION IN STARS, pg. 76.
		       degencode = 1;  // relativistic electron
//                                        degeneracy exists.
                       degenrelativistic = 1;  // and has been seen.
		     } // relativisitic electron degeneracy exists.
                     mue = (1.0 + chemx[i])/2.0;
		     zv = cuberoot(mue);
		     k1 = 1.615e10*zv*zv*zv*zv*zv;   // compute K1
//                                                 in P = K1(ro)**(5/3).
// non-relativistic electron degeneracy boundary is defined by
//   K1(density)**(5/3) = [density/(mue*mh)]kT,
//   or density**(4/3) > (1/(K1*mue*mh))kT,
//    while degencode != 1.  See pg. 74-75,
//                           THERMONUCLEAR FUSION IN STARS.
		     ro3 = cuberoot(ro);
		     ro43 = ro3*ro3*ro3;
		     if( ro43 > (1.0/(k1*mue*mh))*k*t  && degencode != 1)
			 degencode = 2;    // encode
//                                 non-relativistic electron dengeneracy.
                 }     
	      }
	      else if(starform[i] == 1){
                  phi = sqrt(xtheta[k]);
		  p = xpressurecentral[i]*phi*phi*phi*phi*phi;
	          t = xtc[i]*xtheta[k];
		  ro = xdensitycentral[i]*phi*phi*phi;
		  if(t > 1.0e7)
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		  r = (xrstar[i]/rsun)*(xxr[k]/3.63375);
		  lgro = 5.964 + log10((1.0 + chemx[j])/2.0);
		  degencode = 0;  // preset for no degeneracy.
                  if(ro > 0.003){
		     if(log10(ro) > lgro) // See THERMONUCLEAR FUSION
//                                          IN STARS, pg. 76.
			degencode = 1;  // relativisitic electron
//                                         degeneracy exists.	     
                     
		     mue = (1.0 + chemx[i])/2.0;
		     zv = cuberoot(mue);
		     k1 = 1.615e10*zv*zv*zv*zv*zv;   // compute K1 in
//                                                  P = K1(ro)**(5/3).
// non-relativistic electron degeneracy boundary is defined by
//   K1(density)**(5/3) = [density/(mue*mh)]kT,
//   or density**(4/3) > (1/(K1*mue*mh))kT,
//    while degencode != 1.
//    See pg. 74-75, THERMONUCLEAR FUSION IN STARS.
		     ro3 = cuberoot(ro);
		     ro43 = ro3*ro3*ro3;
		     if( ro43 > (1.0/(k1*mue*mh))*k*t && degencode != 1)
			 degencode = 2;    // encode
//                                non-relativistic electron dengeneracy.
		  }
	      }
	      else if(starform[i] == 2){   // n=3.5 polytrope
                  phi = sqrt(yxtheta[k]);
		  p = xpressurecentral[i]*phi*phi*phi*phi*phi*phi*phi*phi*phi;
	          t = xtc[i]*yxtheta[k];
		  ro = xdensitycentral[i]*phi*phi*phi*phi*phi*phi*phi;
		  if(t > 1.0e7)
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		  r = (xrstar[i]/rsun)*(yxr[k]/9.53581);
		  lgro = 5.964 + log10((1.0 + chemx[j])/2.0);
		  degencode = 0;  // preset for no degeneracy.
                  if(ro > 0.003){
		     if(log10(ro) > lgro) // See THERMONUCLEAR FUSION
//                                               IN STARS, pg. 76.
			degencode = 1;  // relativisitic
//                                        electron degeneracy exists.
                        
                  
		     mue = (1.0 + chemx[i])/2.0;
		     zv = cuberoot(mue);
		     k1 = 1.615e10*zv*zv*zv*zv*zv;   // compute
//                                              K1 in P = K1(ro)**(5/3).
// non-relativistic electron degeneracy boundary is defined by
//   K1(density)**(5/3) = [density/(mue*mh)]kT,
//     or density**(4/3) > (1/(K1*mue*mh))kT,
//    while degencode != 1.  See pg. 74-75,
//     THERMONUCLEAR FUSION IN STARS.
		     ro3 = cuberoot(ro);
		     ro43 = ro3*ro3*ro3;
		     if( ro43 > (1.0/(k1*mue*mh))*k*t && degencode != 1)
			 degencode = 2;    // encode
//                               non-relativistic electron dengeneracy.
		  }
	      }
	      else if(starform[i] == 3){  // n=4 polytrope
		  phi = zxtheta[k];
		  p = xpressurecentral[i]*phi*phi*phi*phi*phi;
	          t = xtc[i]*zxtheta[k];
		  ro = xdensitycentral[i]*phi*phi*phi*phi;
		  if(t > 1.0e7)
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		  r = (xrstar[i]/rsun)*(zxr[k]/14.97155);
		  lgro = 5.964 + log10((1.0 + chemx[j])/2.0);
		  degencode = 0;  // preset for no degeneracy.
                  if(ro > 0.003){
		     if(log10(ro) > lgro)      // See THERMONUCLEAR
//                                            FUSION IN STARS, pg. 76.
			degencode = 1;  // relativisitic
//                                         electron degeneracy exists.
		     mue = (1.0 + chemx[i])/2.0;
		     zv = cuberoot(mue);
		     k1 = 1.615e10*zv*zv*zv*zv*zv;   // compute K1
//                                              in P = K1(ro)**(5/3).
// non-relativistic electron degeneracy boundary is defined by
//   K1(density)**(5/3) = [density/(mue*mh)]kT,
//   or density**(4/3) > (1/(K1*mue*mh))kT,
//    while degencode != 1.  See pg. 74-75,
//    THERMONUCLEAR FUSION IN STARS.
		     ro3 = cuberoot(ro);
		     ro43 = ro3*ro3*ro3;
		     if( ro43 > (1.0/(k1*mue*mh))*k*t && degencode != 1)
			 degencode = 2;    // encode
//                        non-relativistic electron dengeneracy.
		  }
	      }
	       tem[k] = t;
	       pres[k] = p;
	       rdis[k] = r*rsun;
	       fprintf(fp," %e   %e   %e   %e  ",r,ro,t,p);
	       fprintf(fp,"%d\n",degencode);
	       if(degencode == 1){
		  degenrelativistic = 1;  // set relativisitic
//                                         degeneracy seen flag.
	       }
	       k++;
	    }
            fprintf(fp,"Degeneracy Code\n");
	    fprintf(fp,"  = 0 -> no electron degeneracy.\n");
	    fprintf(fp,"  = 1 -> relativistic electron degeneracy -> gamma=4/3.\n");
	    fprintf(fp,"  = 2 -> non-relativistic electron degeneracy -> gamma=5/3.\n\n");
	    fprintf(fp," \n");    // issue top of page - feed forward.
//
//
	    fprintf(fp,"\nCONVECTION: VARIABLES FROM CENTER TO SURFACE I-B\n");
	    if(opacityflag == 1)
	      fprintf(fp,"Opacity includes bound-free, free-free, & electron scattering .\n");
	    if(opacityflag == 2)
	      fprintf(fp,"Opacity includes free-free and electron scattering only.\n");
	    if(opacityflag == 3)
	      fprintf(fp,"Opacity includes electron scattering only.\n");
	    fprintf(fp,"   If condition true, then convection is allowed.\n");
	    fprintf(fp,"r/r_star,                dT/dr  <   ((gam - 1)/gam)*(T/P)*(dP/dr)");
	    fprintf(fp,"  true/false\n");
	    k=0;
	    looplim = looplimset2(i);
	    while(k < looplim -1 ){
	       delstep = (rdis[k + 1] - rdis[k])/2.0;
	       dtdx = (tem[k+1] - tem[k])/delstep;
	       fgammapt =  ((tem[k+1] + tem[k])/2.0);
	       fgammapt =  fgammapt*(2.0/(pres[k+1] + pres[k]));
	       fgammapt =  fgammapt*((pres[k+1] - pres[k])/delstep);
	       if(starform[i] == 0){    // n = 3, gamma = 4/3 model
		  fgammapt =  fgammapt*0.25;
	       }
	       if(starform[i] == 1){    // n = 1.5, gamma = 5/3 model
		  fgammapt =  fgammapt*0.4;
	       }
	       if(starform[i] == 2){    // n = 3.5, gamma = 9/7 model
		  fgammapt =  fgammapt*(2.0/9.0);
	       }
	       if(starform[i] == 3){    // n = 4, gamma = 5/4 model
		  fgammapt =  fgammapt*0.20;
	       }
	       fprintf(fp,"r/r_star = %e, ", ((rdis[k+1] + rdis[k])/2.0)/xrstar[i]);
	       fprintf(fp,"dT/dr = %e, %e",dtdx,fgammapt);
	       if(dtdx < fgammapt){
		   fprintf(fp," True,convection.\n");
	       }
	       else{
		   fprintf(fp," False,no convection.\n");
	       }
	       k++;
	    }
	    
	    fprintf(fp,"\n");
	    fprintf(fp,"\nVARIABLES FROM CENTER TO SURFACE II\n");
	    if(starform[i] == 0)
	      fprintf(fp,"n=3, gamma=4/3, %s.\n",sp[i].starname);
	    if(starform[i] == 1)
	      fprintf(fp,"n=1.5, gamma=5/3, %s.\n",sp[i].starname);
	    if(starform[i] == 2)
	      fprintf(fp,"n=3.5, gamma=9/7, %s.\n",sp[i].starname);
	    if(starform[i] == 1)
	      fprintf(fp,"n=4.0, gamma=5/4, %s.\n",sp[i].starname);

	    if(opacityflag == 1)
	      fprintf(fp,"Opacityflag 1, bound-free, free-free, electron scattering incl.\n");
	    if(opacityflag == 2)
	      fprintf(fp,"Opacityflag 2, free-free, electron scattering included.\n");
	    if(opacityflag == 3)
	      fprintf(fp,"Opacityflag 3, Thompson electron scattering opacity only. \n");

            fprintf(fp,"Luminosity Partition in units of solar luminosity Lsun\n");
	    fprintf(fp,"Radius    L_nuclear = L_radiative + L_conduction + L_convection\n");
	    fprintf(fp," r/Rsun                [Maximum]\n");
	    k=0;
	    looplim = looplimset2(i);
	    while(k < looplim - 2){
               if(starform[i] == 0){
	          t = xtc[i]*theta[k];
		  r = (xrstar[i]/rsun)*(xr[k]/6.9011);
	       }
	       else if(starform[i] == 1){
		  t = xtc[i]*xtheta[k];
		  r = (xrstar[i]/rsun)*(xxr[k]/3.65375);
	       }
	       else if(starform[i] == 2){
		  t = xtc[i]*yxtheta[k];
		  r = (xrstar[i]/rsun)*(yxr[k]/9.53581);
	       }
	       else if(starform[i] == 3){
		  t = xtc[i]*zxtheta[k];
		  r = (xrstar[i]/rsun)*(zxr[k]/14.97155);
	       }
	       fprintf(fp,"%f   %f  %e  ",
		 r, claylum[i][k]/normalizeclayton, radequilum[i][k]);
	       fprintf(fp," %e   %f\n",
	         conduclum[i][k], conveclum[i][k]);
       
               k++;
	    }
	    fprintf(fp,"Last 2 table entries omitted.\n");

	    fprintf(fp,"\nNUCLEAR REACTIONS MAP, %s\n",
	        sp[i].starname);
	    if(starform[i] == 0)
	      fprintf(fp,"Structure: n=3, gamma = 4/3.\n");
	    if(starform[i] == 1)
	      fprintf(fp,"Structure: n=1.5, gamma = 5/3.\n");
	    if(starform[i] == 2)
	      fprintf(fp,"Structure: n=3.5, gamma=9/7.\n");
	    if(starform[i] == 3)
	      fprintf(fp,"Structure: n=4.0, gamma=5/4.\n");
	    fprintf(fp,"The star is assumed homogeneously mixed, mu=%e.\n",
	    chemmu[i]);
	    fprintf(fp,"Threshold:10**7 K    10**8 K  5x10**8 K  10**9 2x10**9 3x10**9 \n");
	    fprintf(fp,"[approx.] Hydrogen   Helium   Carbon     Neon  Oxygen  Silicon\n");
	    fprintf(fp,"          burns to  to C & O  O,Ne,Na,   O,Mg  Mg-S    to Iron.\n");
	    fprintf(fp,"r/Rsun    Helium              Mg                    \n");
	    k=0;
	    rhcore  = 0.0;
	    looplim = looplimset2(i);
	    while(k < looplim){
               if(starform[i] == 0){
	          t = xtc[i]*theta[k];
		  r = (xrstar[i]/rsun)*(xr[k]/6.9011);
	       }
	       else if(starform[i] == 1){
		  t = xtc[i]*xtheta[k];
		  r = (xrstar[i]/rsun)*(xxr[k]/3.65375);
	       }
	       else if(starform[i] == 2){
		  t = xtc[i]*yxtheta[k];
		  r = (xrstar[i]/rsun)*(yxr[k]/9.53581);
	       }
	       else if(starform[i] == 3){
		  t = xtc[i]*zxtheta[k];
		  r = (xrstar[i]/rsun)*(zxr[k]/14.97155);
	       }
	       
	       if(t > 1.0e7){
		  fprintf(fp,"%e, H->He [T=%e K] ",r,t);
		  rhcore = r;   // Finds hydrogen-burning core maximum radius r/R_sun.
	       }
	       else
		  fprintf(fp,"%e, weak  [T=%e K] ",r,t);
	       if(t > 1.0e8)
		  fprintf(fp,"He->C&O ");
	       else
		  fprintf(fp,"  ");
	       if(t > 5.0e8)
		  fprintf(fp,"C->O,Ne,Na,Mg ");
	       else
		  fprintf(fp,"  ");
	       if(t > 1.0e9)
		  fprintf(fp,"Ne->O,Mg ");
	       else
		  fprintf(fp,"  ");
	       if(t > 2.0e9)
		  fprintf(fp,"O->Mg-S ");
	       else
		  fprintf(fp,"  ");
	       if(t > 3.0e9)
		  fprintf(fp,"Si->Fe\n");
	       else
		  fprintf(fp,"\n");

               k++;
	    }

	    evolver();     // compute the age of the star, stellar evolution vars, ect.
//

	    if(degenrelativistic != 1){
	       fprintf(fp,"\nIDEALIZED CORE DIMENSIONS\n");
	       modelrr = calaratio2(i); // compute the
//                                     idealized core compaction ratio,
//                                     uniform core.
               fprintf(fp,"For the screened Chiu pp-fusion chain epsilon in J/Kg-sec,\n");
	       fprintf(fp,"Model %d idealized uniform core r/R = %f, ",
	          i,modelrr);
	       if(modelrr > 0.99)
		  fprintf(fp,"FAILS.\n");
	       else
                  fprintf(fp,"\n");
	       fprintf(fp,"r such that Lstar = [(4pi/3)*r**3]*epsilon[ro,t]\n");
	       fprintf(fp,"for ro and t at their central values for this polytrope.\n");
	    }
	    else
	       fprintf(fp,"White dwarf relativistic dengeneracy detected.\n\n\n\n\n");

// issue top of page - feed forward.

	    fprintf(fp,"\nFUSION - NUCLEAR LUMINOSITY MEASURES, \n");
	    if(starform[i] == 0)
	      fprintf(fp,"gamma = 4/3, n=3, %s\n",sp[i].starname);
	    else if(starform[i] == 1)
	      fprintf(fp,"gamma = 5/3, n=1.5, %s.\n",sp[i].starname);
	    else if(starform[i] == 2)
	      fprintf(fp,"gamma = 9/7, n=3.5, %s\n", sp[i].starname);
	    else if(starform[i] == 3)
	      fprintf(fp,"gamma = 5/4, n=4, %s.\n",sp[i].starname);
	    fprintf(fp,"Non-Normalized, Official Fusion Functions integrated\n");
	    fprintf(fp,"over the polytrope from the center to the surface.\n");
	    fprintf(fp,"Chiu luminosity includes electrostatic screening.\n");
	    fprintf(fp,"Clayton luminosity also includes electrostatic screening.\n");
	    if(abuncode[i] == 4)
	      fprintf(fp,"Optimized via variation of molecular weight.\n");
	    vx = bcmratiobest[j];
	    maxlum = vx*vx*vx*vx;
	    minlum = vx*vx*vx;
	    
	    fprintf(fp,"In solar luminosity units.\n");
	    fprintf(fp,"   [M/Msun]**4 = Max[L/Lsun] = %e,[Upper bound].\n",
	    maxlum);
	    fprintf(fp,"Computed observed     L/Lsun = %e,[from distance & m]*.\n",
	    bclratio[i]);
	    fprintf(fp,"   [M/Msun]**3 = Min[L/Lsun] = %e,[Lower bound.]\n",minlum);
	    fprintf(fp,"P-P chain luminosity  L/Lsun = %e,[Clayton,scrnd&normd]\n",
	    nscclaytonluminosity[i]);
	    fprintf(fp,"P-P chain luminosity  L/Lsun = %e,[Clayton,unscreened.]\n",
	    claytonluminosity[i]);
	    fprintf(fp,"P-P chain luminosity  L/Lsun = %e,[Clayton,screened.]\n",
	    scclaytonluminosity[i]);	    
	    fprintf(fp,"Carbon cycle only L_CNO/Lsun = %e,[CNO cycle, M.Schw.,unscr.]\n",
	    cnoluminosity[i]);
	    fprintf(fp,"Triple-Alpha cyc.  L_He/Lsun = %e,[Helium burning,unscr.]\n",
	    heliumluminosity[i]);
	    fprintf(fp,"Total H-burning  L_Htot/Lsun = %e,[H-burning, PP + CNO.]\n",
	    totalhluminosity);
	    fprintf(fp,"Total nuclear L:  L_tot/Lsun = %e,[Total, PP+CNO+He.]\n\n",
	    totalhluminosity);
//
//
	    fprintf(fp,"In units of the luminosity of the star %s.\n", sp[i].starname);
	    fprintf(fp,"   [M/Msun]**4 = Max[L/Lstar] = %e,[Upper bound].\n",
	    maxlum/bclratio[0]);
	    fprintf(fp,"Computed observed     L/Lstar = %e,[from distance & m]*.\n",
	    bclratio[i]/bclratio[0]);
	    fprintf(fp,"   [M/Msun]**3 = Min[L/Lstar] = %e,[Lower bound.]\n",minlum/bclratio[0]);
	    fprintf(fp,"P-P chain luminosity  L/Lstar = %e,[Clayton,scrnd&normd]\n",
	    nscclaytonluminosity[i]/bclratio[0]);
	    fprintf(fp,"P-P chain luminosity  L/Lstar = %e,[Clayton,unscreened.]\n",
	    claytonluminosity[i]/bclratio[0]);
	    fprintf(fp,"P-P chain luminosity  L/Lstar = %e,[Clayton,screened.]\n",
	    scclaytonluminosity[i]/bclratio[0]);
  
	    fprintf(fp,"Carbon cycle only L_CNO/Lstar = %e,[CNO cycle, M.Schw.,unscr.]\n",
	    cnoluminosity[i]/bclratio[0]);
	    fprintf(fp,"Triple-Alpha cyc.  L_He/Lstar = %e,[Helium burning,unscr.]\n",
	    heliumluminosity[i]/bclratio[0]);
	    fprintf(fp,"Total H-burning  L_Htot/Lstar = %e,[H-burning, PP + CNO.]\n",
	    totalhluminosity/bclratio[0]);
	    fprintf(fp,"Total nuclear L:  L_tot/Lstar = %e,[Total, PP+CNO+He.]\n",
	    totalhluminosity/bclratio[0]);
//
//  * When I add screened Clayton p-p fusion, CNO, and Helium burning, I get 0.8 or so
//    of the luminosity of Sirius, probably because cnoluminosity[0], 
//    the CNO luminosity, is unscreened in the above calculation.
//    Presently, that is how I know it is unscreened. We need Ltotal over Lstar to
//    exceed unity before reducing H in the core can normalize the luminosity and
//    the age of Sirius can be computed from the reduction in X(H).
//
	    fprintf(fp,"\nFUSION PROCESS DIAGNOSIS:\n");
            underluminous = 0;  // preset for usually overluminous.
	    if(bclratio[i] < scclaytonluminosity[i])
		underluminous = 1;
	    else
                underluminous = 0;
            if(degenrelativistic == 1){
		fprintf(fp,"The star shows central relativistic degeneracy,\n");
		fprintf(fp,"and is a white dwarf, class VII or Vc.\n");
                
	    }
	    if(underluminous == 1 && degenrelativistic == 1){
		fprintf(fp,"This is the reason the star seems to be underluminous\n");
		fprintf(fp,"according to Clayton pp-fusion formulae.\n");
                
	    }
	    if(underluminous == 0 && degenrelativistic == 0){
                deltastar = (bclratio[i] - scclaytonluminosity[i])/bclratio[i];
		fprintf(fp,"[L_Star - L_Clayton]/L_star = %e,\n",deltastar);
		fprintf(fp,"[L_Star/L_Chiu] = %e.\n",bclratio[i]/scclaytonluminosity[i]);
		fprintf(fp,"The star is overluminous according to the\n");
                if(starform[i] == 0)
		    fprintf(fp,"n=3, gamma=4/3 polytropic structural hypothesis,\n");
                if(starform[i] == 1)
		    fprintf(fp,"n=1.5, gamma=5/3 polytropic structural hypothesis,\n");
                if(starform[i] == 2)
		    fprintf(fp,"n=3.5, gamma=9/7 polytropic structural hypothesis,\n");
		if(starform[i] == 3)
                    fprintf(fp,"n=4.0, gamma=5/4 polytropic structural hypothesis,\n");
		fprintf(fp,"because the model is not centrally dense enough.\n");
		fprintf(fp,"The star is actually denser at the center than the model.\n");
		fprintf(fp,"A model with a higher polytropic index n may be\n");
		fprintf(fp,"required to improve the stellar structure description.\n");
	    }
	    if(underluminous == 1 && degenrelativistic == 0){
	       fprintf(fp,"Perhaps the star is underluminous for the current\n");
	       fprintf(fp,"model star structural hypothesis. Perhaps the\n");
	       fprintf(fp,"model central density is too high. This means\n");
	       fprintf(fp,"that the polytropic index n is too high, so that\n");
	       fprintf(fp,"the model is denser in the center than in the star.\n\n");
            }
	    if(claytonluminosity[i] == bclratio[i])
	       fprintf(fp,"The model hypothesis fits the star exactly.\n");
//	        
	    fprintf(fp,"\n");
	    fprintf(fp,"4th power L/Lsun-Max/[L/Lsun-obs]     = %e,\n",
	    maxlum/bclratio[i]);
	    fprintf(fp,"L/Lsun-observed/[L/Lsun-obs]          = %e, *\n", 1.0);
	    fprintf(fp,"3rd power L/Lsun-Min/[L/Lsun-obs]     = %e,\n",
	    minlum/bclratio[i]);
	    fprintf(fp,"L/Lsun_Clayton_unscreened/[L/Lsun]    = %e,\n",
	    claytonluminosity[i]/bclratio[i]);
	    fprintf(fp,"L/Lsun_Clayton_screened/[L/Lsun]      = %e,\n",
	    scclaytonluminosity[i]/bclratio[i]);
	    fprintf(fp,"L/Lsun_Clayton_scr-normalized/[L/Lsun]= %e,\n",
	    nscclaytonluminosity[i]/bclratio[i]);	    
	    fprintf(fp,"L_CNO/Lsun/[L/Lsun-observed]          = %e.\n",
	    cnoluminosity[i]/bclratio[i]);
	    fprintf(fp,"L_He/Lsun/[L/Lsun-observed]           = %e.\n",
	    heliumluminosity[i]/bclratio[i]);
	    fprintf(fp," \n");       // feedfwd optional
//
//  SN - The CONDENSED supernova progenitor report
//

	    if(bcmratiobest[i] > 6.23){  // special supernova report.
	       fprintf(fp,"\n\nTYPE II SUPERNOVA PROGENITOR DETECTED: %s.\n",
	       sp[i].starname);
	       fprintf(fp,"Star is a pre-supernova M/Msun = %f > 6.23 Msun.\n",
	       bcmratiobest[i]);

	       sn = 38.8*(msun/xmstar[i])*(msun/xmstar[i]);
//                  old solution using 1/M**2 age.
	       sn = 1.0 - sn;  // older measure
//                  based on (Msun/M)**2 lifetime.
	       sn = 82.02*exp( 2.409*log( 1.0/bcmratiobest[i] ));
//                  12.88e9(Msun/M)**2.409 lifetime.
	       sn = 1.0 - sn; // Refined, based on Schw.
//                      solar lifetime & L/Lsun = (M/Msun)**3.409.

               fprintf(fp,"Over the class of Type II pre-supernova stars:\n");
	       fprintf(fp,"Relative likelihood of a supernova soon = %f,\n",
	          sn);

	       age = (1.0/bcmratiobest[i])*(1.0/bcmratiobest[i])*(9.92e9);
//                 Bowers&Deeming style.
	       ageschw = 12.88e9*exp( 2.409*log( 1.0/bcmratiobest[i] ));
//                 1/M**2.409 lifetime.
	       fprintf(fp,"Star max age = 12.88x10**9[Msun/M]**2.409 = %e yrs.\n",
	       ageschw);
	       fprintf(fp,"Likelihood of a supernova this year = %e.\n",
	       (1.0/ageschw));
	       fprintf(fp,"Likelihood of a supernova this century = %e.\n",
	       (100.0/ageschw));
	       
//
//  See SUPERNOVA VIGNETTES, by James A. Green, pg.75.  Chap.5. Supernova Vulnerability.
//
	       dau = lightyears[i]*(9.46e15/1.496e11);
//                              compute distance in AU to star.
	       ps = bcmratiobest[i]*(1.0/dau)*6050.0;
//                              compute pressure wave.
               fprintf(fp,"A typical Supernova features L/Lsun = 1.0x10**10.\n");
	       fprintf(fp,"Typical SN blastwave pressure here = %e Newtons/m**2.\n",
	       ps);

	       blastv = 2000.0;
	       fprintf(fp,"Blastwave assumed min velocity = %e km/sec\n",
	       blastv);
	       dwave = lightyears[i]*(3.156e7)*(blastv*1000.0);
//                           pos. of blastwave in meters
//                           from the star.
	       dwave = dwave/9.46e15;  // current position of
//                                        blastwave in lightyears.
               blastyear = ((lightyears[i] - dwave)*(9.46e12/blastv))/3.157e7;
	       fprintf(fp,"Estimated max blastwave arrival time > %e years.\n",
               blastyear);

	       blastv = 30000.0;  // Zwicky max velocity.
//                                  Supernova Vignettes, page 95.
	       fprintf(fp,"Blastwave Zwicky max velocity = %e km/sec\n",
	       blastv);
	       dwave = lightyears[i]*(3.156e7)*(blastv*1000.0);
//                      pos. of blastwave in meters
//                                 from the star.
	       dwave = dwave/9.46e15;  // current position of
//                                       blastwave in lightyears.
               blastyear = ((lightyears[i] - dwave)*(9.46e12/blastv))/3.157e7;
	       fprintf(fp,"Estimated min blastwave arrival time > %e years.\n",
	       blastyear);

//
// What is actually needed here is to compute the visual magnitude m of the supernova.
//  The eye works on a log scale.  See below, SN burst magnitude.
//  Try I_SN/I_moon = 10**((2/5)*(-12.0 - m_SN)), where m_SN is mvis.
//  and I_SN/I_sun = 10**((2/5)*(-26.72 - m_SN)).
//  Note 10**q = (exp(gamma))**q = exp(gamma*q) ,
//   where gamma = log_e(10) =  2.302585093.
//
//		mvis = -26.72 - 25.0 - ( 5.0*log10( 1.496e11/(lightyears[i]*9.46e15) ));
//		fnovamoon = exp(2.302585093*(0.4)*(-12.0 - mvis));
// fnovamoon = (2.434e-5)*(1.0e10)/(lightyears[i]*lightyears[i]);     old line
//	       fprintf(fp,"Typical intensity: I_supernova/I_moonlight = %e.\n",
//	       fnovamoon);
// fnovasun = (2.499/(lightyears[i]*lightyears[i]));     old line
//		fnovasun = exp(2.302585093*(0.4)*(-26.72 - mvis));
//	       fprintf(fp,"                   I_supernova/I_sunlight  = %e.\n",
//	       fnovasun);
//	       imoonsun = fnovasun/fnovamoon;  // find Imoon/Isun.
	       fprintf(fp,"Typically, Lsupernova/Lsun = 10**10, initially.\n");
		mvis = -26.72 - 25.0 - ( 5.0*log10( 1.496e11/(lightyears[i]*9.46e15) ) );
		fprintf(fp,"Then the initial visual magnitude of the SN = %f.\n",
		mvis);
               delt = 297.0*( sqrt(sqrt(fnovasun + 1.0)) - 1.0);
	       fprintf(fp,"Typical max Temperature Change here = %f degrees K.\n",
	       delt);
               delt = (9.0/5.0)*delt;
	       fprintf(fp,"                            = %f degrees F.\n",
	       delt);
	       events = 128.5/(lightyears[i]*lightyears[i]);
	       fprintf(fp,"Typical events per cm**3 in human body = %e.\n",
	       events);
//
//  See SUPERNOVA VIGNETTES, pg.173, Type II SN burst magnitude.
//
		v2 = (5.586*(1.77505)) - 4.535*(1.411);
		v2 = v2 + 0.00111*(bcmratiobest[i]*bcmratiobest[i]);
		v2 = v2 + 0.0279132*bcmratiobest[i];
		v2 = v2*1.0e52;  // burst amplitude in ergs.
		fprintf(fp,"Predicted total SN burst energy in ergs   = %e.\n",
		   v2);
		fprintf(fp,"                                in Joules = %e.\n",
		  (v2/1.0e7));	
	
		fprintf(fp,"EVOLUTION OF SUPERNOVA %s LUMINOSITY,\n",
		  sp[i].starname);
		fprintf(fp,"Day   Luminosity    Visual_Magnitude  \n");
		fprintf(fp,"       (L/Lsun)                 \n");
		
		mvis0 = -26.72 - 25.0 - ( 5.0*log10( 1.496e11/(lightyears[i]*9.46e15) ) );
                gd = 0;
		while(gd < 370){
		    snlum = (lightyears[i]*9.46e15)/1.496e11;
		    snlum = snlum*snlum;
		    snlum = snlum*exp( 0.921034*(-26.72 - (mvis0 + fsn(gd) )) );
		    mvis = -26.72 - ( 5.0*log10( 1.496e11/(lightyears[i]*9.46e15) ) );
		    mvis = mvis - 2.5*log10(snlum);
		    fprintf(fp,"%d     %e     %f  ",gd,snlum,mvis);
		    m_moon = -26.72 - 2.5*log10(imoonsun);
//                                  idealized moon magnitude
//                                  is -12, according to Kaler.
		    if(mvis < -12)
			fprintf(fp,"brighter than moon's -12.\n");
		    if(mvis >= -12)
			fprintf(fp,"moon brighter at -12.\n");
                    if(gd <= 80)
		       gd = gd + 10;
		    else
		       gd = gd + 30;
		}
		
	    }

	    fprintf(fp,"For Type I or Type II SN of %s.\n",
	        sp[i].starname);
	    fprintf(fp,"William K. Rose gives a classical nova maximum\n");
	    fprintf(fp,"luminosity that is higher, 3x10**5 Lsun. Then for ");
	    fprintf(fp,"%s\n", sp[i].starname);
	    mvis = -26.72 - 2.5*log10( 3.0e5*(1.496e11/(lightyears[i]*9.46e15))*(1.496e11/(lightyears[i]*9.46e15)) ) ;
	    fprintf(fp,"brightest m_classical_nova = %f.\n", mvis);	    
	    i++;
	}


}
//  end_reportms()    <ENDR>
/**********************************************************
*
*   fsn(gd) - Supernova delta_magnitude function on day gd.
*
*   fsn(gd) = alpharise*gd + A0  0 <= gd <= 6.65
*           = 0                  6.65 <= gd <= 12.5
*           = alpha*gd + A,     12.5 <= gd < 90,
*           = beta*gd + B,      90 <= gd < 420.
*
*   Note that fsn(gd) = m_SN(gd) - m_SN(0), 0 <= gd <= 370
*
*   alpharise = -1/7 = -0.142857 , A0 = 1, 
*   alpha = 0.0472244, A = 0, beta = 0.01143, B = 3.2215.
*
**********************************************************/
double fsn(int gd)
{
     double val;                    // Piecewise linear approximation.
     double realgd;
     double alpharise = -0.142857;  // SN luminosity rises for 7 days
     double aaa0 = 1.0;             // Then is solid for about 5 days.
     double alpha = 0.055197402; // Then falls off starting at 13 days
     double aaa = -0.7175662;  // until the 90-day point, after which
     double beta = 0.01143; // it falls until day 420 or so as predicted.
     double bbb = 3.2215;

     if(gd >= 0 && gd <= 7){
          realgd = gd;
	  val = alpharise*realgd + aaa0;
          return(val);
     }
     if(gd >=7 && gd <= 13){
	 val = 0.0;
         return(val);
     }
     if(gd >= 13 && gd <= 90){
	realgd = gd;
	val = alpha*realgd + aaa;
        return(val);
     }
     if(gd >= 90 && gd <= 420){
	realgd = gd;
	val = beta*realgd + bbb;
	return(val);
     }
     val = 8.0;     // give a steady glow after day 420.
     return(val);       

}
/***************************************************************
*
*   calaratio(i) - For model i, compute the compaction ratio a/R
*                  such that
*
* L = dentisy*[(4pi/3)(a**3)]*epsilonmks[density0(a,M,R), T0(a,M,R)].
*
*  Here  density0 = A[ 1 - exp(-(R/a)**3) ],
*
*  where
8   A = M/{ [(4pi/3)*(R**3 - a**3)*exp[-(R/a)**3]] + (4pi/3)(a**3) }.
*
*  and    T0 = K[(density0)**(1/3)](mu*mh/k). Use K for gamma=4/3.
*
* For epsilonmks, we use Phillip's simple pp-fusion chain luminosity.
*    for a given L, we have to find the best fit a/R.
*
*  The theory is based on a psuedo-polytropic representation in which
*
*     density(r) = A(exp[-(r/a)**3] - exp[-(R/a)**3]).
*
*   Note that for any value of a and constant M and R,
*   M = integral_0_to_R[ density(r)*(4pi/3)(r**2)dr ].
*
*  Also, T(r) = K*{[density(r)]**(1/3)}(mu*mh/k),
*
*     P(r) = (density(r)/(mu*mh))*k*T(r).
*
*  Find the a/R value giving the least squared
*   error  (L_star - L_model)**2.
*
*  start with a/R = 1/4.
*  Search a(n) = (R/4)*(n/1000), n=1,2,...,1000.
*
**************************************************************/
double calaratio(int i)
{
	 int kk;
         double rk;       // real value of k.
	 double rlim;     // rlim = R for this star in meters.
	 double lstar;    // L/Lsun, model i.
	 double ar;     // compact model core radius.
	 double aball; // volume of the model compact core.
	 double ro0;  // A = ro0. NOT the actual central
//                       density, but close.
         double ro;  // the actual model central density.
	 double div; // divisor for density.
	 double t;   // temperature
	 double kgamma = 1.0; // K-factor for gamma=4/3
//                            polytrope, set in routine.
	 double mu;  // molecular weight.
	 double fac1, fac2;  // temp. factors.
	 double epsilonmks;  // the fusion energy in Joules/Kg-second.
	 double x;  // X=hydrogen abundance.
	 double error;      //  L/Lsun(simulation-core) - L/Lsun-observed.
	 double sqerror;    // squared error.
	 double smallesterror; // smallest squared error.
         int errormark;   // index of least squared error.
	 double lstarsim;  // simulation core luminosity.
	 double aratio;  //  ar/R to be returned.

	 mu = chemmu[i];    // fetch the molecular weight for this model.
         x = chemx[i];     // fetch the abundance of hydrogen by weight.
	 rlim = xrstar[i];  // get radius of star in meters.
	 lstar = bclratio[i]; // get L/Lsun for this star.
	 smallesterror = 1.0e18; // make the first error huge.
	 kk=1;
	 while(kk <= 1000){
            rk = kk;
	    ar = ((rlim)/4.0)*(rk/1000.0);        // cal core radius
//                                                 under investigation.
	    aball = ((4.0/3.0)*3.14159)*ar*ar*ar;  // cal core volume.

//  ro0 = A = M/{ [(4pi/3)*(R**3 - ar**3)*exp[-(R/ar)**3]] + (4pi/3)(ar**3) }.

	    div = (4.0*3.14159/3.0)*(rlim*rlim*rlim - ar*ar*ar)*exp(-(rlim/ar)*(rlim/ar)*(rlim/ar));
            div = div + (4.0*3.14159/3.0)*ar*ar*ar;
	    ro0 = xmstar[i]/div;

//  ro = density(r) = A(exp[-(r/ar)**3] - exp[-(R/ar)**3]).

	    ro = ro0*( 1.0 - exp(-(rlim/ar)*(rlim/ar)*(rlim/ar)) );
//                                      cal central density.
            coredensity = ro;
	    fac1 = 2.0/(1.0 + chemx[i]);
	    fac1 = cuberoot(fac1);
	    fac2 = fac1*fac1*fac1*fac1*fac1;  // form (2/(1+X))**(5/3)
	    kgamma = 1.615e10*fac2;  // See THERMONUCLEAR
//                                      FUSION IN STARS, page 74.
	    t = (kgamma*cuberoot(ro))*(mu*mh/k); // cal temperature.
	    coretemp = t;
            corepress = (k/(mu*mh))*ro*t;  // core pressure.
	    epsilonmks = (9.5e-37)*x*x*ro*t*t*t*t;
	    lstarsim = aball*ro*epsilonmks;  // compute Phillips
//                                           luminosity of model core.
	    lstarsim = lstarsim/lsun;  //  and prepare in L/Lsun form.
	    error = lstarsim - lstar;  // find the error - compare
//                                         with observed L/Lsun.
	    sqerror = error*error;     // find the squared error.
	    if(sqerror < smallesterror){
		smallesterror = sqerror;
                errormark = kk;
            }
	    kk++;
	 }
         rk = errormark;
	 ar = (rlim/4.0)*(errormark/1000.0); // return a/R
//                                           with least squared error.
	 aratio = ar/rlim;
         aglobal = ar/1000.0;  // leave a in km in global variable.
         return(aratio);  
}
/********************************************
*
*  calaratio2(i) - Compute the uniform core compaction factor #2 = r/R
*        using L = ([4pi/3]r**3)*epsilonmks(xcentraldensity[i], xtc[i]).
*        Gives you a feel for size of central ball reqd to produce Lstar
*        at the computed central temperature and density for the
*        polytropic model under test.  Also compute r/Rsun.
* in: xtc[i] = central temperature,
*     xcentraldensity[i] = central density for the structural hypothesis
*                          we are examining.
* out: returns r/R compaction ratio,
*      coretemp2 = xtc[i] as global variable,
*      coredensity2 = xcentraldensity[i] as global variable.
*      corepress2 = central pressure for an ideal gas as global variable.
*
*********************************************/
double calaratio2(int i)
{
	 double rratio = 1.0;  // initial dummy code.
	 int kk;
         double rk;       // real value of k.
	 double rlim;     // rlim = R for this star in meters.
	 double lstar;    // L/Lsun, model i.
	 double ar;     // compact model core radius.
	 double aball; // volume of the model compact core.
	 double ro0;  // A = ro0. NOT the actual central density,
//                          but close.
         double ro;  // the actual model central density.
	 double div; // divisor for density.
	 double t;   // temperature
	 double mu;  // molecular weight.
	 double fac1, fac2;  // temp. factors.
	 double epsilonmks;  // the fusion energy in Joules/Kg-second.
	 double x;  // X=hydrogen abundance.
	 double error;  //  L/Lsun(simulation-core) - L/Lsun-observed.
	 double sqerror; // squared error.
	 double smallesterror; // smallest squared error.
         int errormark;   // index of least squared error.
	 double lstarsim;  // simulation core luminosity.
	 
	 double aratio;  //  ar/R to be returned.
	 
	 x = chemx[i]; // fetch the abundance of hydrogen by weight.
         mu = chemmu[i];   // fetch the molecular weight.
	 rlim = xrstar[i];  // get radius of star in meters.
	 lstar = bclratio[i]; // get L/Lsun for this star.
	 smallesterror = 1.0e55; // make the first error huge.
	 ro = xdensitycentral[i];
	 t = xtc[i];      // cal temperature.
	 coretemp2 = t;
	 coredensity2 = ro;
	 corepress2 = (ro/(mu*mh))*k*t;  // idealized perfect gas
//                                          law central pressure.

// Cal screened Chiu luminosity. 
	 epsilonmks = (0.413)*x*x*ro*cuberoot(1.0e6/t)*cuberoot(1.0e6/t)*exp(-33.81*cuberoot(1.0e6/t));
	 epsilonmks = epsilonmks*(1.0 + (0.25*sqrt(ro)*sqrt(1.0e6/t)*sqrt(1.0e6/t)*sqrt(1.0e6/t)));
   
	 
	 kk=1;
	 while(kk <= 1000){ // find the size of the idealized uniform core
            rk = kk;
	    ar = ((rlim)/1.0)*(rk/1000.0);  // cal core radius under investigation.
	    aball = ((4.0/3.0)*3.14159)*ar*ar*ar;  // cal core volume.  
   
	    lstarsim = aball*ro*epsilonmks;  // compute Phillips
//                                         luminosity of model core.
	    lstarsim = lstarsim/lsun;  //  and prepare in L/Lsun form.
	    error = lstarsim - lstar;  // find the error - compare
//                                        with observed L/Lsun.
	    sqerror = error*error;     // find the squared error.
	    if(sqerror < smallesterror){
		smallesterror = sqerror;
                errormark = kk;
            }
	    kk++;
	 }
         rk = errormark;
	 ar = (rlim/1.0)*(errormark/1000.0); // return a/R
//                                          with least squared error.
	 aratio = ar/rlim;
         rglobal = ar/1000.0;  // leave a in km in global variable.  
         return(aratio);
}
/***************************************
*
*  looplimset2() - set looplim for all-inclusive
*                  table output, rather than for
*                  shell integration algorithm.
*
*******************************************/
int looplimset2(int i)
{
		double looplim;

            if(starform[i] == 0)
		looplim = 28;
	    else if(starform[i] == 1)
		looplim = 23;
	    else if(starform[i] == 2)
		looplim = 35;
	    else if(starform[i] == 3)
		looplim = 36;

            return(looplim);
}
/***************************************
*
*  templookup2(spect, dspectnum, spectclass
*    - Look up spectral temperature, allow fractional spectnum.
*
*  spect = O,B,A,F,G,K, or M.
*          0,1,2,3,4,5, or 6 = corresponding table
*                             index.
*  dspectnum = floating-point fractional spectnum.
*  We allow types like M1.5, M3.2, ect.
*  spectclass = 1 for giant (III) or 2 for regular (V).
*        Any spectral class is associated with index 2.
*
*****************************************/
double templookup2(char spect, double dspectnum, int spectclass)
{
       int spectnum;
       int spectnumhigh, spectnumlow;
       double hightemp, lowtemp;
       double truetemp;

       if(dspectnum == 0.0){
//           if spectral number is an integer, use integer lookup.
           spectnum = 0;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 1.0){
           spectnum = 1;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 2.0){
           spectnum = 2;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 3.0){
           spectnum = 3;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 4.0){
           spectnum = 4;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 5.0){
           spectnum = 5;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 6.0){
           spectnum = 6;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 7.0){
           spectnum = 7;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 8.0){
           spectnum = 8;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 9.0){
           spectnum = 9;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if(dspectnum == 10.0){
           spectnum = 10;
	   truetemp = templookup(spect,spectnum,spectclass);
	   return(truetemp);
       }
       if( dspectnum > 0.0  &&  dspectnum < 1.0 ){
//                   if spectral type not integer, 
	  spectnumhigh = 1; spectnumlow = 0; // interpolate between
//                                              2 integer values.
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(1.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 1.0  &&  dspectnum < 2.0 ){
	  spectnumhigh = 2; spectnumlow = 1;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(2.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 2.0  &&  dspectnum < 3.0 ){
	  spectnumhigh = 3; spectnumlow = 2;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(3.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 3.0  &&  dspectnum < 4.0 ){
	  spectnumhigh = 4; spectnumlow = 3;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(4.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 4.0  &&  dspectnum < 5.0 ){
	  spectnumhigh = 5; spectnumlow = 4;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(1.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 5.0  &&  dspectnum < 6.0 ){
	  spectnumhigh = 6; spectnumlow = 5;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(6.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 6.0  &&  dspectnum < 7.0 ){
	  spectnumhigh = 7; spectnumlow = 6;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(7.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 7.0  &&  dspectnum < 8.0 ){
	  spectnumhigh = 8; spectnumlow = 7;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(8.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 8.0  &&  dspectnum < 9.0 ){
	  spectnumhigh = 9; spectnumlow = 8;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(9.0 - dspectnum));
	  return(truetemp);
       }
       if( dspectnum > 9.0  &&  dspectnum < 10.0 ){
	  spectnumhigh = 10; spectnumlow = 9;
	  hightemp = templookup(spect,spectnumlow,spectclass);
	  lowtemp =  templookup(spect,spectnumhigh,spectclass);
	  truetemp = lowtemp + ((hightemp - lowtemp)*(10.0 - dspectnum));
	  return(truetemp);
       }
}  
/****************************************
*
*  templookup(spect,spectnum,spectclass)
*      - Look up spectral type temperature.
*
*    spect = O,B,A,F,G,K,or M.  V = the Sun, for calibration.                                
*            0,1,2,3,4,5,or 6 = corresponing index for spect.
*    spectnum = integer number within OBAFGKM type, 0-10.
*    spectclass = 1 for giant (III) or 2 for regular (V).
*                 any other spectral class is assoicated with index 2
*                 for temperature look-up purposes.
*
*****************************************/
double templookup(char spect, int spectnum, int spectclass)
{
     double val;
     int k;   // index corresponding to spect code from OBAFGKM.    

     if(spect == 'O')  // first, map ascii type to index k.
	 k = 0;
     if(spect == 'B')
	 k = 1;
     if(spect == 'A')
	 k = 2;
     if(spect == 'F')
	 k = 3;
     if(spect == 'G')
	 k = 4;
     if(spect == 'K')
	 k = 5;
     if(spect == 'M')
	 k = 6;
     if(spect != 'V'){     // 'V' specifies the Sun
//                             for calibration purposes.
       if(spectclass == 1)
          val = spectable[spectclass][k][spectnum];  // lookup temp
       else if(spectclass == 2)
          val = spectable[spectclass][k][spectnum];  // lookup temp
       else  val = spectable[2][k][spectnum];  // lookup temp
     }
     else if(spect == 'V')
	val = 5770.0;   // for 'V', return surface temp. of the Sun.

     return(val);
}
/****************************************
*
*  initspectable - initialize table of temperatures
*                  connecting spectral type to temp in Kelvin.
*
*   Uses spectable[classification][alphacode][numeric-code]
*            classification = 1 -> giant (III) class.
*                           = 2 -> regular (dwarf V) class.
*            alphacode = O,B,A,F,G,K, or M.
*                       (0,1,2,3,4,5, or 6.)
*                        W,        R     S
*                                        N
*                    exotic types are parallel.
*    numeric-code = 0-10. Note fractional types
*                   like B3.4 are allowed by
*            higher-level routines which use the spectable
*           to fetch high and low values which are interpolated.
*
*   The array is dimensioned as spectable[3][7][11].
*
*  Classes are to use temperatures from table V (index 2),
*  except for the giant class.
*  Classes include: I,Ia,Ib = supergiants.  [index 3]
*                   II = bright giants.
*                   III = giants [index 1].
*                   IV = Subgiants.
*                    V = Main sequence dwarfs.
*                   ZAMS V = Zero Age Main Sequence
*                   VI = Sub-dwarfs.
*                   VII = White Dwarfs.
*     
*            
*
*****************************************/
void initspectable(void)
{
// set up giant star (class) III,  0.0 temperatures are undefined
//           type O.                   in Zombeck [9], p.68-71.
     spectable[1][0][0] = 0.0;
     spectable[1][0][1] = 0.0;
     spectable[1][0][2] = 0.0;
     spectable[1][0][3] = 0.0;
     spectable[1][0][4] = 0.0;
     spectable[1][0][5] = 0.0;
     spectable[1][0][6] = 0.0;
     spectable[1][0][7] = 0.0;
     spectable[1][0][8] = 0.0;
     spectable[1][0][9] = 0.0;
     spectable[1][0][10] = 0.0; 
// set up giant star (class) III,
//           type B.
     spectable[1][1][0] = 0.0;
     spectable[1][1][1] = 0.0;
     spectable[1][1][2] = 0.0;
     spectable[1][1][3] = 0.0;
     spectable[1][1][4] = 0.0;
     spectable[1][1][5] = 0.0;
     spectable[1][1][6] = 0.0;
     spectable[1][1][7] = 0.0;
     spectable[1][1][8] = 0.0;
     spectable[1][1][9] = 0.0;
     spectable[1][1][10] = 0.0;
// set up giant star (class) III,
//           type A.
     spectable[1][2][0] = 0.0;
     spectable[1][2][1] = 0.0;
     spectable[1][2][2] = 0.0;
     spectable[1][2][3] = 0.0;
     spectable[1][2][4] = 0.0;
     spectable[1][2][5] = 0.0;
     spectable[1][2][6] = 0.0;
     spectable[1][2][7] = 0.0;
     spectable[1][2][8] = 0.0;
     spectable[1][2][9] = 0.0;
     spectable[1][2][10] = 0.0;
// set up giant star (class) III,
//           type F.
     spectable[1][3][0] = 0.0;
     spectable[1][3][1] = 0.0;
     spectable[1][3][2] = 0.0;
     spectable[1][3][3] = 0.0;
     spectable[1][3][4] = 0.0;
     spectable[1][3][5] = 0.0;
     spectable[1][3][6] = 0.0;
     spectable[1][3][7] = 0.0;
     spectable[1][3][8] = 0.0;
     spectable[1][3][9] = 0.0;
     spectable[1][3][10] = 0.0;
// set up giant star (class) III,
//           type G.
     spectable[1][4][0] = 0.0;
     spectable[1][4][1] = 0.0;
     spectable[1][4][2] = 0.0;
     spectable[1][4][3] = 0.0;
     spectable[1][4][4] = 0.0;
     spectable[1][4][5] = 5010.0;
     spectable[1][4][6] = 4963.33;  // interp.
     spectable[1][4][7] = 4916.66;  // interp.
     spectable[1][4][8] = 4870.0;
     spectable[1][4][9] = 48200.0;   // interp.
     spectable[1][4][10] = 4770.0;  // interp.
// set up giant star (class) III,
//           type K.
     spectable[1][5][0] = 4720.0;
     spectable[1][5][1] = 4580.0;
     spectable[1][5][2] = 4460.0;
     spectable[1][5][3] = 4210.0;
     spectable[1][5][4] = 4010.0;
     spectable[1][5][5] = 3780.0;
     spectable[1][5][6] = 3760.0; // interp.
     spectable[1][5][7] = 3740.0; // interp.
     spectable[1][5][8] = 3720.0; // interp.
     spectable[1][5][9] = 3700.0; // interp.
     spectable[1][5][10] = 3680.0;  // interp.
// set up giant star (class) III,
//           type M.
     spectable[1][6][0] = 3660.0;
     spectable[1][6][1] = 3600.0;
     spectable[1][6][2] = 3500.0;
     spectable[1][6][3] = 3300.0;
     spectable[1][6][4] = 3100.0;
     spectable[1][6][5] = 2950.0;
     spectable[1][6][6] = 2800.0;
     spectable[1][6][7] = 2650.0; // interp.
     spectable[1][6][8] = 2500.0; // interp.
     spectable[1][6][9] = 2350.0; // interp.
     spectable[1][6][10] = 2200.0;  // interp.
// set up regular star (class) V,
//           type O.
     spectable[2][0][0] = 38000.0;  // interp.
     spectable[2][0][1] = 38000.0;   // interp.
     spectable[2][0][2] = 38000.0;  // interp.
     spectable[2][0][3] = 38000.0;  // interp.
     spectable[2][0][4] = 38000.0; // interpolated
     spectable[2][0][5] = 38000.0;
     spectable[2][0][6] = 38000.0;
     spectable[2][0][7] = 38000.0;
     spectable[2][0][8] = 35000.0;
     spectable[2][0][9] = 35000.0;
     spectable[2][0][10] = 31900.0; // val. for 9.5.
// set up regular star (class) V,
//           type B.
     spectable[2][1][0] = 30000.0;
     spectable[2][1][1] = 24200.0;
     spectable[2][1][2] = 22100.0;
     spectable[2][1][3] = 18800.0;
     spectable[2][1][4] = 17600.0; // interpolated fill
     spectable[2][1][5] = 16400.0;
     spectable[2][1][6] = 15400.0;
     spectable[2][1][7] = 14500.0;
     spectable[2][1][8] = 13400.0;
     spectable[2][1][9] = 12400.0;
     spectable[2][1][10] = 11600.0;  // interpolated fill
// set up regular star (class) V,
//           type A.
     spectable[2][2][0] = 10800.0;
     spectable[2][2][1] = 10265.0;  // interpolated fill.
     spectable[2][2][2] = 9730.0;
     spectable[2][2][3] = 9360.0;  // interpolated fill.
     spectable[2][2][4] = 8990.0;  // interpolated fill.
     spectable[2][2][5] = 8620.0;
     spectable[2][2][6] = 8405.0; // interpolated fill.
     spectable[2][2][7] = 8190.0;
     spectable[2][2][8] = 7952.5; // interpolated fill.
     spectable[2][2][9] = 7715.0; // interpolated fill.
     spectable[2][2][10] = 7477.5; // interpolated fill.
// set up regular star (class) V,
//           type F.
     spectable[2][3][0] = 7240.0;
     spectable[2][3][1] = 7105.0;  // interpolated fill.
     spectable[2][3][2] = 6930.0;
     spectable[2][3][3] = 6800.0; // interpolated fill.
     spectable[2][3][4] = 6670.0;  // interpolated fill.
     spectable[2][3][5] = 6540.0;
     spectable[2][3][6] = 6426.66; // interpolated fill.
     spectable[2][3][7] = 6313.33; // interpolated.
     spectable[2][3][8] = 6200.0;
     spectable[2][3][9] = 6060.0;  // interpolated.
     spectable[2][3][10] = 5990.0;    // interpolated.
// set up regular star (class) V,
//           type G.
     spectable[2][4][0] = 5920.0;
     spectable[2][4][1] = 5850.0;  // interpolated.
     spectable[2][4][2] = 5780.0;
     spectable[2][4][3] = 5723.33; // interpolated.
     spectable[2][4][4] = 5666.66; // interpolated.
     spectable[2][4][5] = 5610.0;
     spectable[2][4][6] = 5570.0; // interpolated.
     spectable[2][4][7] = 5530.0;  // interpolated.
     spectable[2][4][8] = 5490.0;
     spectable[2][4][9] = 5440.0;  // interp.
     spectable[2][4][10] = 5390.0; // interp.
// set up regular star (class) V,
//           type K.
     spectable[2][5][0] = 5340.0;
     spectable[2][5][1] = 5060.0; // interpolated.
     spectable[2][5][2] = 4780.0;
     spectable[2][5][3] = 4706.0; // interp.
     spectable[2][5][4] = 4632.0; // interp.
     spectable[2][5][5] = 4558.0; // interp.
     spectable[2][5][6] = 4484.0;  // interp.
     spectable[2][5][7] = 4410.0; 
     spectable[2][5][8] = 4287.5; // interp.
     spectable[2][5][9] = 4165.0;  // interp.
     spectable[2][5][10] = 4042.5; // interp.
// set up regular star (class) V,
//           type M.
     spectable[2][6][0] = 3920.0;
     spectable[2][6][1] = 3680.0;
     spectable[2][6][2] = 3500.0;
     spectable[2][6][3] = 3360.0;
     spectable[2][6][4] = 3230.0;
     spectable[2][6][5] = 3120.0;
     spectable[2][6][6] = 2966.66;  // interp.
     spectable[2][6][7] = 2813.33;   // interp.
     spectable[2][6][8] = 2660.0;
     spectable[2][6][9] = 2506.67; // interp.
     spectable[2][6][10] = 2353.34;  // interp.

     return;
}
/********************************************************
*
*    setspectype - Define spectral type equivilant from
*                  spectral temperature alone. (Estimate).
*                  Use spectclass = 2 (V) for temp lookup.
*   in: tempsurf, index for model number.
*  out: sp[i].spectcode = OBAFGKM
*       sp[i].spectnumber = 0.0-10.0.
*       sp[i].spectclass = 2 (equivilant to type V).
*
********************************************************/
void setspectype(double tempsurf, int i)
{
       int k;
       int goflag;

       sp[i].classification = 2;
//                   equivilant to the temperature for
//                   a regular dwarf (V). Sets equiv flag above.
       k = 0;
       goflag = 1;
       while( k < 11 && goflag == 1){
	  if( tempsurf >= spectable[2][0][k]){
	      sp[i].spectcode = 'O';
	      sp[i].spectnumber = k;
              goflag = 0;
	  }
          ++k;
       }
       k = 0;
       while( k < 11 && goflag == 1){
	  if( tempsurf >= spectable[2][1][k]){
	      sp[i].spectcode = 'B';
	      sp[i].spectnumber = k;
              goflag = 0;
	  }
          ++k;
       }
       k = 0;
       while( k < 11 && goflag == 1){
	  if( tempsurf >= spectable[2][2][k]){
	      sp[i].spectcode = 'A';
	      sp[i].spectnumber = k;
              goflag = 0;
	  }
          ++k;
       }
       k = 0;
       while( k < 11 && goflag == 1){
	  if( tempsurf >= spectable[2][3][k]){
	      sp[i].spectcode = 'F';
	      sp[i].spectnumber = k;
              goflag = 0;
	  }
          ++k;
       }
       k = 0;
       while( k < 11 && goflag == 1){
	  if( tempsurf >= spectable[2][4][k]){
	      sp[i].spectcode = 'G';
	      sp[i].spectnumber = k;
              goflag = 0;
	  }
          ++k;
       }
       k = 0;
       while( k < 11 && goflag == 1){
	  if( tempsurf >= spectable[2][5][k]){
	      sp[i].spectcode = 'K';
	      sp[i].spectnumber = k;
              goflag = 0;
	  }
          ++k;
       }
       k = 0;
       while( k < 11 && goflag == 1){
	  if( tempsurf >= spectable[2][6][k]){
	      sp[i].spectcode = 'M';
	      sp[i].spectnumber = k;
              goflag = 0;
	  }
          ++k;
       }
       if(tempsurf < 2353.34){
//            nominal cut off at lower end of range is M10.
	  sp[i].spectcode = 'M';
	  sp[i].spectnumber = 10;
       }
}
/***********************************************
*
*  initpolytrope - initialize xr[29], theta[29]
*                  for n=3 polytrope,
*                  and xxr[], xtheta[xxr]
*                  for n=1.5 polytrope
*
************************************************/
void initpolytrope(void)
{
// 29 rows of data for n=3, gamma=4/3.
// See Hong Yee Chiu, STELLAR PHYSICS, pg.429.

	xr[0] = 0.00; xr[1] = 0.1; xr[2] = 0.2;
	xr[3] = 0.3; xr[4] = 0.4;
	xr[5] = 0.5; xr[6] = 0.6; xr[7] = 0.7;
	xr[8] = 0.8; xr[9] = 0.9;
	xr[10] = 1.0; xr[11] = 1.2; xr[12] = 1.4;
	xr[13] = 1.6; xr[14] = 1.8;
	xr[15] = 2.0; xr[16] = 2.2; xr[17]=2.4;
	xr[18] = 2.6; xr[19]=2.8;
	xr[20] = 3.0; xr[21] = 3.5;  xr[22] = 4.0;
	xr[23] = 4.5; xr[24] = 5.0;
	xr[25] = 5.5; xr[26] = 6.0;  xr[27] = 6.5;
	xr[28] = 6.89685;

	theta[0] = 1.0;     theta[1] = 0.9983358;
	theta[2] = 0.9933731; theta[3] = 0.9851998;
	theta[4] = 0.9739582; theta[5] = 0.9598390;
	theta[6] = 0.9430731; theta[7] = 0.9239227;
	theta[8] = 0.9026720; theta[9] = 0.8796170;
	theta[10] = 0.8550574; theta[11] = 0.8025918;
	theta[12] = 0.7474647; theta[13] = 0.6915440;
	theta[14] = 0.6363094;
	theta[15] = 0.5828505; theta[16] = 0.5319070;
	theta[17] = 0.4839278;
	theta[18] = 0.4391363; theta[19] = 0.3975890;
	theta[20] = 0.3592269; theta[21] = 0.2762629;
	theta[22] = 0.2092804; theta[23] = 0.1550662;   // Minor errors
	theta[24] = 0.1108148;                          // fixed May 18,2002
	theta[25] = 0.07427935; theta[26] = 0.04372967; // theta(26) was 0.042967
	theta[27] = 0.01785659;
        theta[28] = 0.0;

// 23 rows of data for n=1.5, gamma = 5/3.
// See Hong Yee Chiu, STELLAR PHYSICS, pg.428.

	xxr[0] = 0.00; xxr[1] = 0.1; xxr[2] = 0.2;
	xxr[3] = 0.3; xxr[4] = 0.4;
	xxr[5] = 0.5; xxr[6] = 0.6; xxr[7] = 0.7;
	xxr[8] = 0.8; xxr[9] = 0.9;
	xxr[10] = 1.0; xxr[11] = 1.2; xxr[12] = 1.4;
	xxr[13] = 1.6; xxr[14] = 1.8;
	xxr[15] = 2.0; xxr[16] = 2.2; xxr[17] = 2.4;
	xxr[18] = 2.6;  xxr[19] = 2.8;
	xxr[20] = 3.0; xxr[21] = 3.5;  xxr[22] = 3.65375;

	xtheta[0] = 1.0;     xtheta[1] = 0.9983346;
	xtheta[2] = 0.9933533; xtheta[3] = 0.9851007;
	xtheta[4] = 0.9736505; xtheta[5] = 0.9591038;  // fixed May 18,2002 
	xtheta[6] = 0.9415880; xtheta[7] = 0.9212546; // theta(7) was 0.92122546
	xtheta[8] = 0.8982764; xtheta[9] = 0.8728454; // theta(8) was 0.8982746
	xtheta[10] = 0.8451696; xtheta[11] = 0.7839767;
	xtheta[12] = 0.7165629; xtheta[13] = 0.6448497;
	xtheta[14] = 0.5707200;
	xtheta[15] = 0.4959366; xtheta[16] = 0.4220768;
	xtheta[17] = 0.3504865;
	xtheta[18] = 0.2822523; xtheta[19] = 0.2181918;
	xtheta[20] = 0.1588576;
	xtheta[21] = 0.3261720; xtheta[22] = 0.0;

//  data for n=3.5 polytrope.
//  Chiu, STELLAR PHYSICS, pg.430.    35 values each.

	yxr[0] = 0.00; yxr[1] = 0.1; yxr[2] = 0.2;
	yxr[3] = 0.3; yxr[4] = 0.4;
	yxr[5] = 0.5; yxr[6] = 0.6; yxr[7] = 0.7;
	yxr[8] = 0.8; yxr[9] = 0.9;
	yxr[10] = 1.0; yxr[11] = 1.2; yxr[12] = 1.4;
	yxr[13] = 1.6; yxr[14] = 1.8;
	yxr[15] = 2.0; yxr[16] = 2.2; yxr[17] = 2.4;
	yxr[18] = 2.6;  yxr[19] = 2.8;
	yxr[20] = 3.0; yxr[21] = 3.5;  yxr[22] = 4.0;
	yxr[23] = 4.5; yxr[24] = 5.0;
	yxr[25] = 5.5; yxr[26] = 6.0;  yxr[27] = 6.5;
	yxr[28] = 7.0; yxr[29] = 7.5;
	yxr[30] = 8.0; yxr[31] = 8.5; yxr[32] = 9.0;
	yxr[33] = 9.5; yxr[34] = 9.53581;

	yxtheta[0] = 1.0;     yxtheta[1] = 0.9983362;
	yxtheta[2] = 0.9933796;
	yxtheta[3] = 0.9852324;
	yxtheta[4] = 0.9740588; yxtheta[5] = 0.9600767;
	yxtheta[6] = 0.9435471;
	yxtheta[7] = 0.9247621;
	yxtheta[8] = 0.9040325; yxtheta[9] = 0.8816767;
	yxtheta[10] = 0.8580095;
	yxtheta[11] = 0.8079341;
	yxtheta[12] = 0.7559766; yxtheta[13] = 0.7038914;
	yxtheta[14] = 0.6529871;
	yxtheta[15] = 0.6041621; yxtheta[16] = 0.5579738;
	yxtheta[17] = 0.5147163;
	yxtheta[18] = 0.4744922; yxtheta[19] = 0.4372729;
	yxtheta[20] = 0.4029450;
	yxtheta[21] = 0.3286420; yxtheta[22] = 0.2683473;
	yxtheta[23] = 0.2191654;
	yxtheta[24] = 0.1786788; yxtheta[25] = 0.1449887;
	yxtheta[26] = 0.1166389;
	yxtheta[27] = 0.09252102; yxtheta[28] = 0.07179022;
	yxtheta[29] = 0.05379917;
	yxtheta[30] = 0.03804794; yxtheta[31] = 0.02414698;
	yxtheta[32] = 0.01178996;
        yxtheta[33] = 0.007335955; yxtheta[34] = 0.0;

//  data for n=4.0 polytrope.
//  Chiu, STELLAR PHYSICS, pg. 431.   36 values per table.

	zxr[0] = 0.00; zxr[1] = 0.1; zxr[2] = 0.2;
	zxr[3] = 0.3; zxr[4] = 0.4;
	zxr[5] = 0.5; zxr[6] = 0.6; zxr[7] = 0.7;
	zxr[8] = 0.8; zxr[9] = 0.9;
	zxr[10] = 1.0; zxr[11] = 1.2; zxr[12] = 1.4;
	zxr[13] = 1.6; zxr[14] = 1.8;
	zxr[15] = 2.0; zxr[16] = 2.2; zxr[17] = 2.4;
	zxr[18] = 2.6;  zxr[19] = 2.8;
	zxr[20] = 3.0; zxr[21] = 3.5;  zxr[22] = 4.0;
	zxr[23] = 4.5; zxr[24] = 5.0;
	zxr[25] = 5.5; zxr[26] = 6.0;  zxr[27] = 6.5;
	zxr[28] = 7.0; zxr[29] = 7.5;
	zxr[30] = 8.0; zxr[31] = 8.5; zxr[32] = 9.0;
	zxr[33] = 9.5; zxr[34] = 10.0;
        zxr[35] = 14.97155;

	zxtheta[0] = 1.0;     zxtheta[1] = 0.9983366;
	zxtheta[2] = 0.9933862;
	zxtheta[3] = 0.9852649;
	zxtheta[4] = 0.9741584; zxtheta[5] = 0.9603108;
	zxtheta[6] = 0.9440112;
	zxtheta[7] = 0.9255782;
	zxtheta[8] = 0.9053458; zxtheta[9] = 0.8836492;
	zxtheta[10] = 0.8608137;
	xtheta[11] = 0.8129248;
	zxtheta[12] = 0.7637987; zxtheta[13] = 0.7150640;
	zxtheta[14] = 0.6678668;
	zxtheta[15] = 0.6229411; zxtheta[16] = 0.5806986;
	zxtheta[17] = 0.5413175;
	zxtheta[18] = 0.5048158; zxtheta[19] = 0.4711090;
	zxtheta[20] = 0.4400513;
	zxtheta[21] = 0.3728001; zxtheta[22] = 0.3180411;
	zxtheta[23] = 0.2731271;
	zxtheta[24] = 0.2359176; zxtheta[25] = 0.2047554;
	zxtheta[26] = 0.1783764;
	zxtheta[27] = 0.1558179; zxtheta[28] = 0.1363427;
	zxtheta[29] = 0.1193821;
	zxtheta[30] = 0.1044932; zxtheta[31] = 0.09132739;
	zxtheta[32] = 0.07960771;
	zxtheta[33] = 0.06911198; zxtheta[34] = 0.05966029;
	zxtheta[35] = 0.0;

}
// *******************************************************************************
//
//      zetacal(void) - Return zeta, used in electrostatic screening for fusion,
// 		   the sum over Z of (Z*Z + Z)(X(Z)/A(Z)), where
//                 X(Z) is the mass fraction of element Z. This version stripped
//                 and modified for inclusion into STAREXP4.cpp.
//                 Zeta is included in the formula for the Debye radius (Clayton,pg.360).
//                 We do this for Zeta_Population_I (galactic disk stars)
//	           and Zeta_Population_II (galactic halo stars).
//  		   For stellar models with variable xf,yf,zf,
//		   where xf = X(1), the hydrogen mass fraction,
//    		         yf = X(2), the helium mass fraction,
//  			 zf = 1 - xf - yf,   the heavy element mass fraction
//                 we compute zeta_I(xf,yf,zf) assuming a Population_I-like distribution
//                 for the heavy elements, so that the mass fraction distribution
//                 is shaped like Claytons Population I for heavy elements. This will turn 
//                 out to describe Population II, with its smaller z, tolerably well.
//                 Note we must have  Sum_of X(Z) =  1.0.
//                 That is, Sum_of(3 to 100) X(Z) =  1.0 - xf - yf = zf.
//		   Let f(Z) be the mass fraction distribution for Population_I.
//                 Then Sum_of(3 to 100) f(Z) =  1.0 - XI(1) - XI(2) = zf1.
//                 Thus Sum_of(3 to 100) (zf/zf1)*f(z) = zf.
//		   That is, X(Z) = (zf/zf1)*f(z) for Z = 3,4,5,....,100 is the
//		   shaped model distribution for heavy elements in X(Z).
//                 The new model distribution is saved in chem[Z] for reportms().
//                 For that model distribution, we compute zeta, the Sum over Z
// 		   of (Z*Z + Z)(X(Z)/A(Z)), to find zeta for fusion process models.
//               
//                      - Author: James A. Green, April 17, 2002.
//			- Compiler: Borland C/C++
//
//           SOLAR ABUNDANCES (via Clayton, pg.85, from Aller,1961).
//		 Typ for Population I stars.   X(Z) computed by J.A.Green.
//
//      Z    	A(Z)	Atom	By_Weight	X(Z) - mass fraction
//
//      0	1.0	None	0		0.0
//    	1	1.0097	H	1000		0.5976
//	2	4.0026	He	640		0.3825
//	3	6.939	Li	0		0.0
//  	4	9.0122	Be	0		0.0
//	5	10.811	B	0		0.0
//	6      12.01115	C	4.8		2.8685e-3
//	7	14.0067	N	1.5		8.9641e-4
//	8	15.9994	O	14.0		8.3665e-3
//	9	18.9984	F	0.02		1.1952e-5
//	10	20.183	Ne	10.0		5.9761e-3
//	11	22.9898	Na	0.05		2.9880e-5
//	12	24.312	Mg	0.6		3.5856e-4
//	13	26.9815	Al	0.05		2.9880e-5
//	14	28.086	Si	0.9		5.3785e-4
//	15	30.9738	P	0		0.0
//	16	32.064	S	0.7		4.1833e-4
//	17	35.453	Cl	0.07		4.1833e-5
//	18	39.948	Ar	0.3		1.7928e-4
//	19	39.102	K	0		0.0
//	20	40.08	Ca	0.08		4.78086e-5
// 	21	44.956	Sc	0		0.0
//	22	47.9	Ti	0		0.0
//	23	50.942	V	0		0.0
//	24	51.996	Cr	0		0.0
//	25	54.9380	Mn	0		0.0
//	26	55.847	Fe	0.2		1.1952e-4
//	27	58.9332 Co	0		0.0
//	28	58.71	Ni	0.06		3.5856e-5
//      29	60.0  > Ni      0.01		5.9761e-6   dummy for higher Z
//
//           SOLAR ABUNDANCES (via Martin Schwarzchild, pg.25, from Aller,1953).
//		          X(Z) computed by J.A.Green.
//
//      Z    	A(Z)	Atom	By_Weight	X(Z) - mass fraction
//
//      0	1.0	None	0		0.0
//    	1	1.0097	H	1000		0.74184
//	2	4.0026	He	320		0.2373887
//	3	6.939	Li	0		0.0
//  	4	9.0122	Be	0		0.0
//	5	10.811	B	0		0.0
//	6      12.01115	C	1		7.4184e-4
//	7	14.0067	N	3		2.22552e-3
//	8	15.9994	O	8		5.93472e-3
//	9	18.9984	F	0		0.0
//	10	20.183	Ne	10		7.4184e-3
//	11	22.9898	Na	0		0.0
//	12	24.312	Mg	1		7.4184e-3
//	13	26.9815	Al	0		0.0
//	14	28.086	Si	1		7.4184e-3
//	15	30.9738	P	0		0.0
//	16	32.064	S	1		7.4184e-3
//	17	35.453	Cl	0		0.0
//	18	39.948	Ar	2		1.48318e-3
//	19	39.102	K	0		0.0
//	20	40.08	Ca	0		0.0
// 	21	44.956	Sc	0		0.0
//	22	47.9	Ti	0		0.0
//	23	50.942	V	0		0.0
//	24	51.996	Cr	0		0.0
//	25	54.9380	Mn	0		0.0
//	26	55.847	Fe	1		7.4184e-3
//	27	58.9332 Co	0		0.0
//	28	58.71	Ni	0		0.0
//      29	60.0  > Ni      0		0.0   dummy for higher Z
//
// ***************************************************************************
double zetacal(void)
{

	double xf,yf,zf;    // mass fractions xf,yf,zf in the arbitrary stellar model.
	double zf1;	    // mass fraction of heavy elements in pop I, 1 - xf1 - yf1.
	
//
//  Solar abundances used for Population I abundance of the elements.
//
//                Atomic Weights in order by Z, 0-29.  1st element is a dummy.
//
	double a[30] = { 1.0, 1.0097, 4.0026, 6.939, 9.0122, 10.811,
			12.01115, 14.0067, 15.9994, 18.9984, 20.1830,
			22.9898, 24.312, 26.9815, 28.086, 30.9738,
			32.064, 35.453, 39.948, 39.102, 40.08,
			44.956, 47.9, 50.942, 51.996, 54.9380,
			55.847, 58.9332, 58.71, 60.0 }; // solar atomic weights, 1st dummy
	int z;
        double zcnt;
	double zeta1;  // Clayton zeta for the Sun, Clayton abundances.
	double zeta2;  // Scharzchild zeta for the Sun, Schwarzchild abundances.
	double zetamodel;
        double moreq;
	int more = 1;

	
	z = 0;
	zeta1 = 0.0;
	while(z < 30){
	   zcnt = z;
	   zeta1  = zeta1 + ((zcnt*zcnt) + zcnt)*(x[z]/a[z]);
	   ++z;
	}

	z = 0;
	zeta2 = 0.0;
	while(z < 30){
	   zcnt = z;
	   zeta2  = zeta2 + ((zcnt*zcnt) + zcnt)*(xx[z]/a[z]);
	   ++z;
	}

	xf = chemx[0];    // X[i] - abundance by weightof H in model i.
	yf = chemy[0];   // Y[i] - abundance by weight of He in model i.
	zf = 1.0 - xf - yf;    // compute Z.

	chem[1] = xf;  // For report purposes, save recomputed distribution in chem[Z].
	chem[2] = yf;

	z = 3;
	zf1 = 0.0;
	while(z < 30){
	   zcnt = z;
	   zf1  = zf1 + x[z];
	   ++z;
	}

//   Shape the heavy element abundances like the Clayton abundances x[z] for zeta.

	zetamodel = (1.0 + 1.0)*(xf/a[1]);
	zetamodel  = zetamodel + (4.0 + 1.0)*(yf/a[2]);
	z = 3;
	while(z < 30){
	   zcnt = z;
	   zetamodel  = zetamodel + ( ((zcnt*zcnt) + zcnt)*(zf/zf1)*(x[z]/a[z]) );
	   chem[z] = (zf/zf1)*x[z];
	   ++z;
	}
	return(zetamodel);

//	printf("zeta1 = %e, zeta2 = %e,  zetamodel = %e,\n", zeta1, zeta2, zetamodel);
//	more = 0;
//
// REFS -
//
// 1) Zeta is used in computing the electrostatic screening for fusion
//    in the Clayton screening model, pg.360, Principles of Stellar Evolution
//    and Nucleosynthesis, by Donald D. Clayton, McGraw-Hill, 1968.
// 2) The Clayton solar abundances: ibed, pp.85.
// 3) The Schwarzschild solar abundances: pp.25, Structure and Evolution
//    of the Stars, by Martin Schwarzschild, Dover Publications,1965.
//

}

/*
*     References  <REF>
*
* [1] - Richard Bowers & Terry Deeming, 1984. Astrophysics I: The Stars.
*       Jones & Barlett.
* [2] - Hong Yee Chiu, STELLAR PHYSICS, Ginn Blaisdell. Chewbacca from
*                                                  STAR WARS, of course.
* [3] - Stuart L. Shapiro & Saul A. Teukolsky, 1983.
*       Black Holes, White Dwarfs, and Neutron Stars. Wiley-Interscience.
* [4] - Green, James A., CHANDRA: Chandrasekhar Sets of White Dwarfs
*           and Supernova Cores with Stellar Structure Software, 1997.
* [5] - Green, James A., THERMONUCLEAR FUSION IN STARS, Greenwood
*          Research Books & Software, 1999. Son of Stromgren.
*       Shows pre-supernova stars have M/Msun > 6.23.
* [6] - A.C.Phillips, The Physics of Stars, John Wiley & Sons,pg.142-146.
* [7] - Donald D. Clayton, Principles of Stellar Evolution &
*       Nucleosynthesis, McGraw-Hill.
* [8] - Martin Schwarzschild, Structure & Evolution of the Stars,
*        Dover Pulbications, pg.9, bolometric corrections.
* [9] - Martin V. Zombeck, Handbook of Space Astronomy & Astrophysics,
*       Cambridge University Press, pp.68-69 for bolometric corrections.
* [10] - William K. Rose, Advanced Stellar Astrophysics,
*        Cambridge University Press.
* [11] - Martin V. Schwarzschild, STRUCTURE & EVOLUTION
*                                 OF THE STARS, Dover.
* [12] - E.E.Salpeter, "Nuclear Reactions in the Stars. I.
*        Proton-Proton Chain",
*        Physical Review, Volume 88, Number 3, Nov.1, 1952.
* [13] - H.A.Bethe and C.L.Critchfield, "The Formation of
*        Deuterons by Proton Combination", August 15, 1938,
*        Physical Review, pp.248-254.
* [14] - Chandrasekhar, Subrahmanyam, An Introduction to the Study
*          of Stellar Structure, Dover & Univ.of Chicago Press, 1939.
*          Chandrasekhar has many hisoric details and limit theorems.
* [15] - R.F.Stein & A.G.W.Cameron, eds, Stellar Evolution,
*        Plenum Press, 1966. Great articles from a conference.
* [16] - Cox & Giuli, Principles of Stellar Structure, Vols I & II,
*        Gordon & Breach, 1968.  Extensive analysis.
*
*          Email: james_a_green_iii@yahoo.com
*          Online: http://grnwd.tripod.com/home.html .
*          Greenwood Research Catalog: http://grnwd.tripod.com .
*/
